#ifndef GENERAL_H
#define GENERAL_H

/****************************************************************/
/* headers                                                      */
/****************************************************************/
#include "nmatrix.h"

/****************************************************************/
/* defines                                                      */
/****************************************************************/
#define MAXINT		65535
#define MAXRAND     	2147483647.0
#define INV_MAXRAND 	0.00000000046566128752
#define EPSILON         0.00000001
#define PI              3.14159265358979323846
#define TWOPI           6.28318530717958647692
#define LOG2            0.69314718055994529
#define INV_LOG2        1.4426950408889634
#define INV_255     	0.0039215686

#define SHORT		short
#define UCHAR		unsigned char
#define USHORT		unsigned short
#define UINT		unsigned int
#define ULONG		unsigned long
#define BOOLEAN		UCHAR

#define TRUE		1
#define FALSE		0

#define SUCCESS		1
#define ERROR		0

/*
 * utility macros
 */
#define ABS( a )          ( ((a) > 0 )? (a) : (-(a)) )
#define MIN( a, b )       ( ((a) < (b)) ? (a) : (b) )
#define MAX( a, b )       ( ((a) > (b)) ? (a) : (b) )
#define MINI( a, b )      ( ((a) < (b)) ? 0 : 1 )
#define MAXI( a, b )      ( ((a) > (b)) ? 0 : 1 )
#define MAX3( a, b, c )   ( MAX((a), MAX((b),(c))) )
#define MIN3( a, b, c )   ( MIN((a), MIN((b),(c))) )
#define CLAMP( n, a, b )  ( MAX( MIN((n),(b)), (a) ) )
#define SWAP( a, b )      { (a) ^= (b); (b) ^= (a); (a) ^= (b); }
#define SWAPI( a, b )     { int t = a; a = b; b = t; }
#define SWAP_UCHAR(a, b)  { unsigned char t = a; a = b; b = t; }
#define SWAPD( a, b )     { double t = a; a = b; b = t; }
#define FLOOR( a )        ( ((a) > 0) ? (int)(a) : -(int)(-a) )
#define CEILING( a )      ( ((a)==(int)(a))?(a):((a)>0)?1+(int)(a):-((int)(-a)))
#define ROUND( a )        ( ((a) > 0) ? (int)((a)+0.5) : -(int)(0.5-(a)) )
#define ABSOLUTE( i,j,k ) ( (i-j)*(k = ( (i-j)<0 ? -1 : 1)))
#define SQR( a )          ( (a)*(a) )
#define SGN( a )          ( ((a) < 0) ? -1 : 1 )
#define FLT_EQUAL( a, b ) ( ABS((a) - (b)) < EPSILON )
#define FLT_ZERO( a )     ( ABS((a)) <= EPSILON )
#define FLT_LT_ZERO( a )  ( (a) < EPSILON )
#define FLT_GT_ZERO( a )  ( (a) > EPSILON )
#define FLT_NON_ZERO( a ) ( ABS((a)) > EPSILON )
#define ENSURE_NONZERO(a) if( FLT_ZERO(a) ) (a) = EPSILON
#define DEG2RAD( a )      ( (double)((a) * 0.01745329251994329576) )
#define RAD2DEG( a )      ( (double)((a) * 57.29577951308232087684) )
#define ODD( a )          ( ((int)(a))%2 )
#define EVEN( a )         ( !ODD(a) )
#define lg(x)             ( log((double)(x)) * INV_LOG2 )
#define streq( a, b )	  ( strcmp( a, b ) == 0 )
#define irand(low, high)  ( (int)low + (rand()%(((int)high+1) - (int)low)) )
#define nrand()  		  ( rand()*INV_MAXRAND )

/*
 * antialiasing macros
 */
#define INTERP( a, b )    ( (a) + ((1.0 - (a)) * (b)) )

/*
#define DEBUG(s) 												\
{ 																\
	if( debug )													\
		printf( "in %s: ", s ); 								\
}

#define DEBUG2( s, x, y )   printf( s, x, y )
*/

#define DEBUG(s)
#define DEBUG2(s, x, y)

extern int debug;

/****************************************************************/
/* typedefs                                                     */
/****************************************************************/

#endif GENERAL_H
