/****************************************************************/
/* headers                                                      */
/****************************************************************/
#include <stdio.h>
#include <math.h>
#include "general.h"
#include "matrix.h"

/****************************************************************/
/* local defines                                                */
/****************************************************************/
#define M_PI_2         1.5707963268

/****************************************************************/
/*  typedefs                                                    */
/****************************************************************/

/****************************************************************/
/* function prototypes                                          */
/****************************************************************/
void print_mat( MAT4X4, char * );
void print_vect( VECT, char * );
void copy_mat( MAT4X4, MAT4X4 );
void transpose_mat( MAT4X4, MAT4X4 );
void ident_mat( MAT4X4 );
void zero_mat( MAT4X4 );
void init_trans( MAT4X4, double, double, double );
void init_scale( MAT4X4, double, double, double );
void init_xrot( MAT4X4, double );
void init_yrot( MAT4X4, double );
void init_zrot( MAT4X4, double );
void trans_mat( MAT4X4, double, double, double );
void scale_mat( MAT4X4, double, double, double );
void xrot_mat( MAT4X4, double );
void yrot_mat( MAT4X4, double );
void zrot_mat( MAT4X4, double );
void rotate_axis( VECT, double, MAT4X4 );
void scalar_mat_mult( double, MAT4X4 );
double ** mat_mult( MAT4X4, MAT4X4, MAT4X4 );
double *pt_mat_mult( VECT, MAT4X4, VECT );
double dist( VECT, VECT );
void load_mat_rows( MAT4X4, VECT, VECT, VECT, VECT );
void load_mat_cols( MAT4X4, VECT, VECT, VECT, VECT, long );
void trackball( int, int, int, int, int, int, MAT4X4 );
int intersect_sphere( VECT, VECT, VECT, double, VECT );

/****************************************************************/
/* external functions                                           */
/****************************************************************/

/***************************************************************n
/* Global variables                                             */
/****************************************************************/

/****************************************************************/
/*																*/
/****************************************************************/
void print_mat( mat, str )
MAT4X4 mat;
char *str;
{
	int i, j;

	printf( "%s:\n", str );
	for( i = 0; i < 4; i++ )
	{
		for( j = 0; j < 4; j++ )
			printf( "%5.3lf  ", mat[i][j] );
		printf( "\n" );
	}
}

/******************************************************************/
void print_vect( v, str )
VECT v;
char *str;
{
	int i, j;

	printf( "%s:\n", str );
	for( i = 0; i < 4; i++ )
		printf( "%5.3lf  ", v[i] );
	printf( "\n" );

}

/******************************************************************/
void copy_mat( src_mat, dst_mat )
MAT4X4 src_mat, dst_mat;
{
	int i, j;

	for( i = 0; i < 4; i++ )
		for( j = 0; j < 4; j++ )
			dst_mat[i][j] = src_mat[i][j];

}

/******************************************************************/
void transpose_mat( src_mat, dst_mat )
MAT4X4 src_mat, dst_mat;
{
	int i, j;

	for( i = 0; i < 4; i++ )
		for( j = 0; j < 4; j++ )
			dst_mat[j][i] = src_mat[i][j];

}
			
/******************************************************************/
void ident_mat( mat )
MAT4X4 mat;
{
	int i, j;

	zero_mat( mat );
	
	for( i = 0; i < 4; i++ )
		mat[i][i] = 1.0;
}

/******************************************************************/
void zero_mat( mat )
MAT4X4 mat;
{
	int i, j;

	for( i = 0; i < 4; i++ )
		for( j = 0; j < 4; j++ )
 			mat[i][j] = 0.0 ;
}

/******************************************************************/
void init_trans( mat, tx, ty, tz )
MAT4X4 mat;
double tx, ty, tz;
{
	ident_mat( mat );

	mat[0][3] = tx;
	mat[1][3] = ty;
	mat[2][3] = tz;

}

/******************************************************************/
void init_scale( mat, sx, sy, sz )
MAT4X4 mat;
double sx, sy, sz;
{
	ident_mat( mat );

	mat[0][0] = sx;
	mat[1][1] = sy;
	mat[2][2] = sz;

}

/******************************************************************/
void init_xrot( mat, theta )
MAT4X4 mat;
double theta;
{
	double cos_theta, sin_theta;

	cos_theta = cos( DEG2RAD(theta) );
	sin_theta = sin( DEG2RAD(theta) );

	ident_mat( mat );

	mat[1][1] = cos_theta;
	mat[1][2] = -sin_theta;
	mat[2][1] = sin_theta;
	mat[2][2] = cos_theta;

}

/******************************************************************/
void init_yrot( mat, theta )
MAT4X4 mat;
double theta;
{
	double cos_theta, sin_theta;

	cos_theta = cos( DEG2RAD(theta) );
	sin_theta = sin( DEG2RAD(theta) );

	ident_mat( mat );

	mat[0][0] = cos_theta;
	mat[0][2] = sin_theta;
	mat[2][0] = -sin_theta;
	mat[2][2] = cos_theta;

}

/******************************************************************/
void init_zrot( mat, theta )
MAT4X4 mat;
double theta;
{
	double cos_theta, sin_theta;

	cos_theta = cos( DEG2RAD(theta) );
	sin_theta = sin( DEG2RAD(theta) );

	ident_mat( mat );

	mat[0][0] = cos_theta;
	mat[0][1] = -sin_theta;
	mat[1][0] = sin_theta;
	mat[1][1] = cos_theta;
	
}

/******************************************************************/
void trans_mat( mat, tx, ty, tz )
MAT4X4 mat;
double tx, ty, tz;
{
	MAT4X4 trans, tmp;

	init_trans( trans, tx, ty, tz );
	copy_mat( mat, tmp );
	mat_mult( tmp, trans, mat );
}

	
/******************************************************************/
void scale_mat( mat, sx, sy, sz )
MAT4X4 mat;
double sx, sy, sz;
{
	MAT4X4 scale, tmp;

	init_scale( scale, sx, sy, sz );
	copy_mat( mat, tmp );
	mat_mult( tmp, scale, mat );
}

/******************************************************************/
void xrot_mat( mat, theta )
MAT4X4 mat;
double theta;
{
	MAT4X4 rot, tmp;

	init_xrot( rot, theta );
	copy_mat( mat, tmp );
	mat_mult( tmp, rot, mat );
}

/******************************************************************/
void yrot_mat( mat, theta )
MAT4X4 mat;
double theta;
{
	MAT4X4 rot, tmp;

	init_yrot( rot, theta );
	copy_mat( mat, tmp );
	mat_mult( tmp, rot, mat );
}

/******************************************************************/
void zrot_mat( mat, theta )
MAT4X4 mat;
double theta;
{
	MAT4X4 rot, tmp;

	init_zrot( rot, theta );
	copy_mat( mat, tmp );
	mat_mult( tmp, rot, mat );
}

/******************************************************************/
void scalar_mat_mult( double k, MAT4X4 M )
{
	int	i, j;

	for( i = 0; i < 4; i++ )
	{
		for( j = 0; j < 4; j++ )
			M[i][j] *= k;
	}
}

/******************************************************************/
double **mat_mult( mat1, mat2, result )
MAT4X4 mat1, mat2, result;
{
	int i, j, k;
	double tmp;

	for( i = 0; i < 4; i++ )
	{
		for( j = 0; j < 4; j++ )
		{
			tmp = 0.0;
			for( k = 0; k < 4; k++ )
				tmp += mat1[i][k] * mat2[k][j];
			result[i][j] = tmp;
		}
	}

	return( (double **)result );
}

/******************************************************************/
double *pt_mat_mult( pt, mat, result )
PT3D pt;
MAT4X4 mat;
PT3D result;
{
	int i, j;
	VECT r;
	
	for( i = 0; i < 4; i++ )
	{
		r[i] = 0.0;
		for( j = 0; j < 4; j++ )
			r[i] += mat[i][j] * pt[j];
	}

	copy_vect( result, r );
	return( result );

}

/******************************************************************/
double dist( v1, v2 )
VECT v1, v2;
{
        VECT delta;

        sub_vect( v2, v1, delta );

        return( veclen(delta) );
}

/******************************************************************/
void load_mat_rows( R, row1, row2, row3, row4 )
MAT4X4 R;
VECT row1, row2, row3, row4;
{
	int i;

	for( i = 0; i < 4; i++ )
	{
		R[0][i] = row1[i];
		R[1][i] = row2[i];
		R[2][i] = row3[i];
		R[3][i] = row4[i];
	}
}

/******************************************************************/
void load_mat_cols( R, col1, col2, col3, col4, len )
MAT4X4 R;
VECT col1, col2, col3, col4;
long len;
{
	int i;

	for( i = 0; i < len; i++ )
	{
		R[i][0] = col1[i];
		R[i][1] = col2[i];
		R[i][2] = col3[i];
		R[i][3] = col4[i];
	}
}

/****************************************************************/
/*	function:	rotate_axis() - calculates rotation matrix for	*/
/*				for rotating arouns axis from the origin		*/
/*				From "Graphics Gems", (p.466)					*/

/****************************************************************/
/*	function:	rotate_axis() - calculates rotation matrix for	*/
/*				for rotating arouns axis from the origin		*/
/*				From "Graphics Gems", (p.466)					*/
/*																*/
/*	Inputs:		axis	- vector around which to rotate			*/
/*				angle	- angle in degrees to rotate by			*/
/*				R		- matrix to store transform values in	*/
/*																*/
/*	Outputs:	none											*/
/****************************************************************/
void rotate_axis( VECT axis, double angle, MAT4X4 R )
{
	double	s, c;
	double	t;

	/*
	 * if angle is less than +/- 5 degrees
	 * return identity matrix
	 */
	if( FLT_ZERO(angle) )
		return;
	
	/*
	 * ensure axis is normal vector
	 */
	norm( axis );

	/*
	 * calculate terms
	 */
	c = cos(DEG2RAD(angle));
	s = sin(DEG2RAD(angle));
	t = 1.0 - c;

	/*
	 * fill rotation matrix to spin around
	 * axis by angle
	 */
	R[0][0] = t*axis[X]*axis[X] + c;
	R[1][0] = t*axis[X]*axis[Y] - s*axis[Z];
	R[2][0] = t*axis[X]*axis[Z] + s*axis[Y];

	R[0][1] = t*axis[Y]*axis[X] + s*axis[Z];
	R[1][1] = t*axis[Y]*axis[Y] + c;
	R[2][1] = t*axis[Y]*axis[Z] - s*axis[X];

	R[0][2] = t*axis[Z]*axis[X] - s*axis[Y];
	R[1][2] = t*axis[Z]*axis[Y] + s*axis[X];
	R[2][2] = t*axis[Z]*axis[Z] + c;

}

/****************************************************************/
/*	function:	trackball() - implements a virtual trackball	*/
/*				given 2 mouse click (x,y) coordinates, and		*/
/*				the width and height of the window, the			*/
/*				function computes an axis of rotation and the	*/
/*				matrix for rotating about the axis				*/
/*				From "Graphics Gems", (p.462)					*/
/*																*/
/*	Inputs:		x1		- integer x coord of old mouse position	*/
/*				y1		- integer y coord of old mouse position	*/
/*				x2		- integer x coord of new mouse position	*/
/*				y2		- integer y coord of new mouse position	*/
/*				width	- integer width of window				*/
/*				height	- integer height of window				*/
/*				R		- matrix to store transform values in	*/
/*																*/
/*	Outputs:	none											*/
/****************************************************************/
void trackball( int x1, int y1, int x2, int y2, int width, int height, MAT4X4 R)
{
	VECT	m1, m2;
	VECT	eye;
	VECT	hit1, hit2;
	VECT	center;
	VECT	v1, v2;
	VECT	axis;
	double	alpha;
	double	radius;

	/*
	 * initialize matrix to identity
	 */
	ident_mat( R );

	/*
	 * calculate radius of draw window
	 */
	radius = 0.5*(double)MAX( width, height );

	/*
	 * set virtual eye and center of trackball
	 * also normalize mouse clicks to -1.0 - 1.0
	 */
	set_vect( center, 0, 0, 0, 1 );
	set_vect( eye, 0, 0, -10.0, 1 );
	set_vect( m1, (x1-radius)/radius, (y1-radius)/radius, 0, 1 );
	set_vect( m2, (x2-radius)/radius, (y2-radius)/radius, 0, 1 );

	/*
	 * calculate where rays intersect sphere -
	 * these hit points are the same as the
	 * vectors from the center of the sphere
	 * since the sphere's center is at origin
	 * if either ray misses the sphere return
	 * identity matrix
	 */
	if( !intersect_sphere( eye, m1, center, 1.0, v1 ) ||
		!intersect_sphere( eye, m2, center, 1.0, v2 ) )
		return;

	/*
	 * determine perpendicular axis of rotation
	 * and angle of rotation in radians
	 */
	cross( v1, v2, axis );

	alpha = asin( veclen(axis) );
	if( dot(v1, v2) < 0 )
		alpha += M_PI_2;

	/*
	 * generate rotation matrix
	 */
	rotate_axis( axis, RAD2DEG(alpha), R );

}

/****************************************************************/
/*	function:	intersect_sphere() - calculates the				*/
/*				intersection of a ray and a sphere				*/
/*				From "Graphics Gems", (p.388)					*/
/*																*/
/*	Inputs:		pt1	- first point defining vector				*/
/*				pt2	- second point defining vector              */
/*				center	- center of sphere						*/
/*				radius	- double radius of sphere				*/
/*				hit		- point of intersected with sphere		*/
/*																*/
/*	Outputs:	TRUE if vector intersects sphere, else FALSE	*/
/****************************************************************/
int intersect_sphere( VECT pt1, VECT pt2, VECT center, double radius, VECT hit)
{
	VECT	V, EO;
	double	v, d, disc;

	/*
	 * calculate ray defined by two points
	 */
	sub_vect( pt2, pt1, V );
	sub_vect( center, pt1, EO );

	/*
	 * V is normal vect in direction of vector
	 */
	norm( V );

	v = dot( EO, V );
	if( (disc = SQR(radius) - (dot(EO,EO) - SQR(v))) < 0 )
		return( FALSE );

	d = sqrt(disc);
	mult_vect( (v-d), V );
	add_vect( pt1, V, hit );
	
	return( TRUE );
	
}
