#ifndef NMATRIX_H
#define NMATRIX_H

/****************************************************************/
/* headers														*/
/****************************************************************/

/****************************************************************/
/* defines														*/
/****************************************************************/
#define DOUBLE_TYPE		0
#define CHAR_TYPE		1
#define INT_TYPE		2
#define SHORT_TYPE		3
#define LONG_TYPE		4
#define FLOAT_TYPE		5

#define NO_NORM			0
#define SCALE			1
#define SUB_MEAN		2
#define UNIT_VARIANCE	3

#define KRUSKAL_STRESS1     0
#define KRUSKAL_STRESS2     1
#define LINGOES_STRESS      2
#define SAMMON_STRESS       3
#define J_EE_STRESS         4
#define J_FF_STRESS         5
#define J_EF_STRESS         6

/****************************************************************/ 
/* typedefs														*/
/****************************************************************/
typedef struct matrix{
	int		type;
    long    width;
    long    height;
    double  **data;
	char	**cdata;
	int		**idata;
	short	**sdata;
	long	**ldata;
	float	**fdata;
    }MATRIX;

/****************************************************************/ 
/* prototypes													*/
/****************************************************************/
extern void set_normalize( int mode );
extern MATRIX *alloc_matrix( long width, long height, int type );
extern void free_matrix_mem( MATRIX *A );
extern MATRIX *init_matrix( MATRIX *A, double val );
extern MATRIX *copy_matrix( MATRIX *dst, MATRIX *src );
extern MATRIX *dup_matrix( MATRIX *src );
extern void print_matrix( MATRIX *A, char *str );
extern MATRIX *matrix_matrix_mult( MATRIX *A, MATRIX *B );
extern MATRIX *scalar_matrix_mult( double k, MATRIX *A );
extern MATRIX *transpose_matrix( MATRIX *A );
extern MATRIX *matrix_matrix_add( MATRIX *A, MATRIX *B );
extern MATRIX *matrix_matrix_sub( MATRIX *A, MATRIX *B );
extern MATRIX *normalize_matrix( MATRIX *A );
extern MATRIX *matrix_transpose_mult( MATRIX *A );
extern MATRIX *correlation_matrix( MATRIX *A );
extern MATRIX *distance_matrix( MATRIX *A, int mask[] );
extern MATRIX *matrix_square( MATRIX *A );
extern double matrix_summation( MATRIX *A );
extern double column_mean( MATRIX *A, long j );
extern double column_variance( MATRIX *A, long j );
extern double column_median( MATRIX *A, long j );
extern double column_min( MATRIX *A, long j );
extern double column_max( MATRIX *A, long j );
extern void column_noop( MATRIX *A, long j );
extern void column_scale( MATRIX *A, long j );
extern void column_unitvar( MATRIX *A, long j );
extern void column_submean( MATRIX *A, long j );
extern double calc_dim_correlation( MATRIX *A, long dim1, long dim2 );
extern double kruskal_stress( MATRIX *A, MATRIX *P, int mask[] );
extern double sammon_stress( MATRIX *A, MATRIX *P, int mask[] );
extern double dist_correlation( MATRIX *A, MATRIX *P, int mask[] );
extern double calc_max_distance( MATRIX *A, int mask[], long *index1, long * );
extern double calc_min_distance( MATRIX *A, int mask[], long *index1, long * );
extern double calc_mean_distance( MATRIX *A, int mask[] );
extern double max_distance_from_i( MATRIX *A, int mask[], long i,long *j_index);
extern double min_distance_from_i( MATRIX *A, int mask[], long i,long *j_index);

#endif
