#ifndef VECT_H
#define VECT_H

/****************************************************************/
/* headers                                                      */
/****************************************************************/

/****************************************************************/
/* defines                                                      */
/****************************************************************/
#define EUCLIDIAN       0
#define MANHATTAN       1
#define	COSINE			2
#define	CORRELATE		3
#define MINKOWSKI       4

/****************************************************************/
/* typedefs                                                     */
/****************************************************************/
		
/****************************************************************/
/* function prototypes                                          */
/****************************************************************/
/*
 * utility functions
 */
extern double *alloc_nvect( long veclen );
extern int init_display_mask( long veclen);
extern void init_nvect( double vect[], long veclen );
extern void copy_nvect( double vect1[], double vect2[], long veclen );
extern void set_distance_func( int mode );

/*
 * distance functions
 */
extern double euclid_dist( double vect1[], double vect2[], int mask[], long );
extern double manhatten_dist( double vect1[], double vect2[], int mask[], long);
extern double minkowski_dist( double vect1[], double vect2[], int mask[], long);
extern double cosine_dist( double vect1[], double vect2[],int mask[], long );
extern double correlate_nvect( double vect1[], double vect2[], int mask[],long);

/*
 * n-vector math functions
 */
extern double vec_len( double vect[], int mask[], long veclen );
extern void norm_nvect( double vect[], int mask[], long veclen, double [] );
extern void div_nvect( double num, double vect[], long veclen, double [] );
extern void mult_nvect( double num, double vect[], long veclen, double [] );
extern void add_nvect( double vect1[], double vect2[], long veclen, double [] );
extern void sub_nvect( double vect1[], double vect2[], long veclen, double [] );
extern double min_nvect( double vect[], int mask[], long veclen );
extern double max_nvect( double vect[], int mask[], long veclen );
extern long mini_nvect( double vect[], int mask[], long veclen );
extern long maxi_nvect( double vect[], int mask[], long veclen );
extern double mean_nvect( double vect[], int mask[], long veclen );
extern double median_nvect( double vect[], int mask[], long veclen );
extern void sort_nvect( double vect[], long veclen );

/****************************************************************/
/* external variables											*/
/****************************************************************/
extern double (*distance)( double [], double [], int [], long );
extern int *display_mask;
extern double r;

#endif
