#include <stdio.h>
#include "FastClustering.h"

#pragma once

//--Constructor--
FastClustering::FastClustering(const char* fname,int opt,double quality)
{
	if (opt==0){ // OPEN OKC FILE
		OKCFile* okcf = new OKCFile(fname);
		//okcf->Print();
		if (okcf->ReadSuccess()){
			// if open succesful, then cluster
			Cluster(okcf,quality);
		}
	}
	if (opt==1){ // OPEN COV FILE
		COVFile* covf = new COVFile(fname);
		//okcf->Print();
		if (covf->ReadSuccess()){
			// if open succesful, then cluster
			Cluster(covf,quality);
		}
	}

}

//--Binary Similarity Metric--
//  Similarity between two binary vectors
//  Returns 0 (not similar) -> 1 (similar)
double FastClustering::BinarySimilarity(double* dataA,double* dataB,int ndims)
{
	int sum=0,sumA=0,sumB=0;
	double bisim;
	for(int i=0;i<ndims;i++)
	{
		if (dataA[i]==1 && dataB[i]==1)
			sum++;
		if (dataA[i]==1)
			sumA++;
		if (dataB[i]==1)
			sumB++;
	}

	if (sumA==0 && sumB==0)
		bisim=1;
	else{
		if (sumA==0 || sumB==0)
			bisim = 0;
		else
			bisim = ((double(sum)/double(sumA))+(double(sum)/double(sumB)))*0.5;
	}
	return bisim;
}

//--Clustering (OKCFILE)
void FastClustering::Cluster(OKCFile* okcf,double quality)
{
    int j;
	FILE* cgfile;
	double* meanA;
	double* meanB;
	double* medianA;
	double* medianB;
	int seed1,seed2;
	int clustersize;
	int randnum1,randnum2;
	vector<int> root;
	vector<int> parent;
	vector<int> childA;
	vector<int> childB;

	//-- Intialise progression bar--
/*	pbar->Minimum = 1;
	pbar->Maximum = okcf->GetNData();
	pbar->Value = 1;
	pbar->Step = 1;
	//------------------------------*/

	//get databset
	double** dataset = okcf->GetData();
	int ndims = okcf->GetNDims();
	int ndata = okcf->GetNData();

	//initialise CG File
	cgfile = OpenCGFile(okcf);

	//root cluster (contains all data items)
	for(int i=0;i<ndata;i++)
		root.push_back(i);

	//initialise list of clusters
	vector< vector<int> > Q;
	Q.push_back(root);
	
	//--cluster--
	int currentnodeid=0;
	vector<int> nodeidlist;
//	srand((unsigned)time(NULL));

	//write root
	WriteToCGFile(cgfile, &root, okcf,currentnodeid,-1);
	root.clear();

	nodeidlist.push_back(0);
	
	while (!Q.empty()){
		
		// progress progression bar
		//pbar->PerformStep();

		// take top cluster
		parent = Q.at(0);

		// randomly choose two data items from parent cluster
		clustersize = parent.size();
		randnum1 = (int)(clustersize*((double)rand()/(RAND_MAX+1)));
		randnum2 = randnum1;
		while (randnum1==randnum2)
			randnum2 = (int)clustersize*((double)rand()/(RAND_MAX+1));
		seed1 = parent.at(randnum1);
		seed2 = parent.at(randnum2);

		// initialise two new (children clusters)
		childA.push_back(seed1);
		childB.push_back(seed2);

		//-- split parent cluster between two children --
		meanA = new double[ndims];
		for (j=0;j<ndims;j++)
			meanA[j]=dataset[seed1][j];
		meanB = new double[ndims];
		for (j=0;j<ndims;j++)
			meanB[j]=dataset[seed2][j];
		
		medianA = new double[ndims];
		medianB = new double[ndims];
		// for every item in parent cluster

		// repeat 5 times or until centroid don't move
		for (int k=0;k<15;k++){
			childA.clear();
			childB.clear();
			// mean to median
			for (int j=0;j<ndims;j++){
				medianA[j] = (int)(meanA[j]+0.5);
				medianB[j] = (int)(meanB[j]+0.5);
			}
			// split between clusters
			for (int i=0;i<clustersize;i++)
			{
				if ((parent.at(i)!=seed1) && (parent.at(i)!=seed2)){
					
					// split items between children based on similarity to median
					if (BinarySimilarity(medianA,dataset[parent.at(i)],ndims)>BinarySimilarity(medianB,dataset[parent.at(i)],ndims)){
						childA.push_back(parent.at(i));
						//update mean
						for (int j=0;j<ndims;j++)
							meanA[j] = ((meanA[j]*(childA.size()-1)) + dataset[parent.at(i)][j])/childA.size() ;
					}
					else{
						childB.push_back(parent.at(i));
						//update mean
						for (int j=0;j<ndims;j++)
							meanB[j] = ((meanB[j]*(childB.size()-1)) + dataset[parent.at(i)][j])/childB.size() ;
					}
				}
			}
		}

		// write cg file
		currentnodeid++;
		WriteToCGFile(cgfile, &childA, okcf,currentnodeid,nodeidlist.at(0));
		if (childA.size()>1){
			Q.push_back(childA);
			nodeidlist.push_back(currentnodeid);
		}
		currentnodeid++;
		WriteToCGFile(cgfile, &childB, okcf,currentnodeid,nodeidlist.at(0));
		if (childB.size()>1){
			Q.push_back(childB);
			nodeidlist.push_back(currentnodeid);
		}
		childA.clear();
		childB.clear();

		if (!nodeidlist.empty())
			nodeidlist.erase(nodeidlist.begin());

		// remove parent from queue
		Q.erase(Q.begin());
		parent.clear();

	}

	//Close CG
	CloseCGFile(cgfile);
}

//--Clustering (COVFILE)
void FastClustering::Cluster(COVFile* covf,double quality)
{
	FILE* cgfile;
	int seed1,seed2;
	int clustersize;
	int randnum1,randnum2;
	vector<int> root;
	vector<int> parent;
	vector<int> childA;
	vector<int> childB;

	//-- Intialise progression bar--
	/*pbar->Minimum = 1;
	pbar->Maximum = covf->GetNData();
	pbar->Value = 1;
	pbar->Step = 1;*/
	//------------------------------

	//get dataset
	double** dataset = covf->GetData();
	int ndata = covf->GetNData();

	//initialise CG File
	cgfile = OpenCGFile(covf);

	//root cluster (contains all data items)
	for(int i=0;i<ndata;i++)
		root.push_back(i);

	//initialise list of clusters
	vector< vector<int> > Q;
	Q.push_back(root);
	
	//--cluster--
	int currentnodeid=0;
	vector<int> nodeidlist;
//	srand((unsigned)time(NULL));

	//write root
	WriteToCGFile(cgfile, &root, covf,currentnodeid,-1);
	root.clear();

	nodeidlist.push_back(0);
	
	while (!Q.empty()){
		
		// progress progression bar
		//pbar->PerformStep();

		// take top cluster
		parent = Q.at(0);
		
		clustersize = parent.size();
/*		int repeat = (int)(quality * (clustersize*clustersize))+1;
		double distance=0;
		double best_distance=-99999;
		for (int i=0;i<repeat;i++){
			randnum1 = (int)(clustersize*((double)rand()/(RAND_MAX+1)));
			randnum2 = randnum1;
			while (randnum1==randnum2)
				randnum2 = (int)clustersize*((double)rand()/(RAND_MAX+1));
			distance = dataset[parent.at(randnum1)][parent.at(randnum2)];
			if (distance>best_distance){
				seed1 = parent.at(randnum1);
				seed2 = parent.at(randnum2);
				best_distance = distance;
			}
		}
*/
/*		double distance=0;
		double best_distance=-99999;
		for (int i = 0; i < clustersize; i++)
        {
            for (int j = i+1; j < clustersize; j++)
            {
			    distance = dataset[parent.at(i)][parent.at(j)];
			    if (distance>best_distance){
				    seed1 = parent.at(i);
				    seed2 = parent.at(j);
				    best_distance = distance;
			    }
            }
        }
*/
/*		double distance;
		double largest_distance=-99999;
        int besti, bestj;
        int i, j;
		for (i = 0; i < clustersize; i++)
        {
            for (j = 0; j < clustersize; j++)
            {
			    distance = dataset[parent.at(i)][parent.at(j)];
                if (distance > largest_distance)
                {
                    largest_distance = distance;
                    besti = i;
                    bestj = j;
                }
            }
        }
        seed1 = parent.at(besti);
        seed2 = parent.at(bestj);
*/

		double distance=0;
		double best_distance=99999;
        int besti;
        int i, j;
		for (i = 0; i < clustersize; i++)
        {
            distance = 0;
            for (j = 0; j < clustersize; j++)
            {
			    distance += dataset[parent.at(i)][parent.at(j)];
            }
			if (distance<best_distance){
				seed1 = parent.at(i);
				best_distance = distance;
                besti = i;
            }
        }

        double max = -999999;
        for (i = 0; i < clustersize; i++)
        {
            if (seed1 != parent.at(i))
            {
                if (dataset[seed1][parent.at(i)] > max)
                {
                    seed2 = parent.at(i);
                    max = dataset[parent.at(besti)][parent.at(i)];
                }
            }
        }

		// initialise two new (children clusters)
		childA.push_back(seed1);
		childB.push_back(seed2);

		//-- split parent cluster between two children --

		// for every item in parent cluster

		// split between clusters
		for (i=0;i<clustersize;i++)
		{
			// split items between children
			if ((parent.at(i)!=seed1) && (parent.at(i)!=seed2)){
				if (dataset[parent.at(i)][seed1] < dataset[parent.at(i)][seed2]){
					childA.push_back(parent.at(i));
				}else{
					childB.push_back(parent.at(i));
				}
			}
		}

		// write cg file
		currentnodeid++;
		WriteToCGFile(cgfile, &childA, covf,currentnodeid,nodeidlist.at(0));
		if (childA.size()>1){
			Q.push_back(childA);
			nodeidlist.push_back(currentnodeid);
		}
		currentnodeid++;
		WriteToCGFile(cgfile, &childB, covf,currentnodeid,nodeidlist.at(0));
		if (childB.size()>1){
			Q.push_back(childB);
			nodeidlist.push_back(currentnodeid);
		}
		childA.clear();
		childB.clear();

		if (!nodeidlist.empty())
			nodeidlist.erase(nodeidlist.begin());

		// remove parent from queue
		Q.erase(Q.begin());
		parent.clear();

	}

	//Close CG
	CloseCGFile(cgfile);
}


//************************************************************
// WRITING CG FILE FUNCTIONS
//************************************************************

//--OpenCGFile--
//  Initialise file for writing too. 
FILE* FastClustering::OpenCGFile(OKCFile* okcf) //OKCFILE
{
	// open cg file
	int N = strlen(okcf->GetFileName());
	char *cg_filename = new char[N+1];
	strcpy(cg_filename, okcf->GetFileName());
    	strcpy(&cg_filename[N-4],".cg");
	FILE *cgfile = fopen(cg_filename,"w");
	//write header
	fprintf(cgfile, "%d %d\n", (okcf->GetNData()*2)-1, okcf->GetNDims());
	return cgfile;
}
FILE* FastClustering::OpenCGFile(COVFile* covf) //COVFILE
{
	// open cg file
	int N = strlen(covf->GetFileName());
	char *cg_filename = new char[N+1];
	strcpy(cg_filename, covf->GetFileName());
    	strcpy(&cg_filename[N-4],".cg");
	FILE *cgfile = fopen(cg_filename,"w");
	//write header
	fprintf(cgfile, "%d %d\n", (covf->GetNData()*2)-1,covf->GetNData());
	return cgfile;
}



//--WriteToCGFile--  
void FastClustering::WriteToCGFile(FILE* cgfile, vector<int> *cluster, OKCFile *okcf, int nodeid, int parentid) //OKCFile
{
	vector<double> cfdata = GetCGData(cluster,okcf);	
	fprintf(cgfile, "%d %d %d ", nodeid, parentid, cluster->size());
	for (int i=0;i<cfdata.size();i++){
		fprintf(cgfile,"%f ",cfdata.at(i));
	}
	fprintf(cgfile,"\n");
}
void FastClustering::WriteToCGFile(FILE* cgfile, vector<int> *cluster, COVFile *covf, int nodeid, int parentid) //COVFile
{
	vector<int> cfdata = GetCGData(cluster,covf);	
	fprintf(cgfile, "%d %d %d ", nodeid, parentid, cluster->size());
	fprintf(cgfile,"%d",cfdata.at(0));
	//fprintf(cgfile,"%d %f %f %f",cfdata.at(0),cfdata.at(1),cfdata.at(2),cfdata.at(3)); //min,max and mean
	fprintf(cgfile,"\n");
}

//--GetCGData
vector<double> FastClustering::GetCGData(vector<int>* cluster,OKCFile *okcf) // OKCFile
{
    int i, j;
	double sum;
	int csize = cluster->size();
	double**dataset = okcf->GetData();
	vector<double> cgdata;
	//1. Sx
	for (i=0;i<okcf->GetNDims();i++){
		sum=0.0;
		for(j=0;j<csize;j++){
			sum+=dataset[cluster->at(j)][i];
		}
		cgdata.push_back(double(sum)/double(csize));
	}
	//2. radius
	cgdata.push_back(-1);
	//3. max
	double currentdata;
	for (i=0;i<okcf->GetNDims();i++){
		double maxdata=-999;
		for(j=0;j<csize;j++){
			currentdata = dataset[cluster->at(j)][i];
			if (currentdata > maxdata)
				maxdata = currentdata;
		}
		cgdata.push_back(maxdata);
	}
	//4. min
	for (i=0;i<okcf->GetNDims();i++){
		double mindata=999999;
		for(j=0;j<csize;j++){
			currentdata = dataset[cluster->at(j)][i];
			if (currentdata < mindata)
				mindata = currentdata;
		}
		cgdata.push_back(mindata);
	}
	return cgdata;
}

vector<int> FastClustering::GetCGData(vector<int>* cluster,COVFile *covf) //COVFile
{
	int csize = cluster->size();
	vector<int> cgdata;

	// if cluster size==1, store the dataitem reference
	if (csize==1)
		cgdata.push_back(cluster->at(0));
	else
		cgdata.push_back(-1);
	
	// get min,max,and mean (can be uncommented - but slow)
	/*
	double**dataset = covf->GetData();
	double min=999999;
	double max=-999999;
	double sum=0;
	int count=0;
	for(int i=0;i<(csize-1);i++){
		for(int j=i+1;j<csize;j++){
			double distance = dataset[cluster->at(i)][cluster->at(j)];
			sum+=distance;
			if (distance<min)
				min=distance;
			if (distance>max)
				max=distance;
			count++;
		}
	}
	double mean = sum/(double)count;
	cgdata.push_back(min);
	cgdata.push_back(max);
	cgdata.push_back(mean);
    */
	
	return cgdata;
}

//--CloseCGFile--
//  Close file when complete
void FastClustering::CloseCGFile(FILE* cgfile)
{
	// close file
	fclose(cgfile);
}