#include <stdio.h>
#include "SlowClustering.H"                                                  

#define NODEELE(_node) ((ClusterNode *)_node->element)
#define PAIRELE(_node) ((DataPair *)_node->element)

SlowClustering::SlowClustering(const char* cov_file_name, int repeat_times)
{
    int N = strlen(cov_file_name);
	char *cg_file_name;
    cg_file_name = new char[N+1];
    strcpy(cg_file_name, cov_file_name);
    strcpy(&cg_file_name[N-4],".cg");

    OnlineCluster(cov_file_name, cg_file_name, repeat_times);
}

CLinkList* SlowClustering::GetInput(FILE *fp)
{
    CLinkList *data_link = NULL; 
    int i, j, temp;

    fscanf(fp, "%d %d\n", &size, &temp);

    covs = new double *[size];
    for (i = 0; i < size; i++)
        covs[i] = new double [size];

    for (i = 0; i < size; i++)
        for (j = i; j < size; j++)
        {
            fscanf(fp, "%lf", &covs[i][j]); 
            covs[j][i] = covs[i][j];
        }

    data_link = new CLinkList();

    for ( i = 0; i < size; i++ )
    {
        ClusterNode *new_clusternode = new ClusterNode();
        new_clusternode->itemID = i;
        (*(new_clusternode->itemHash))[i] = i;
        new_clusternode->entries = 1;
        new_clusternode->parent = NULL;
        data_link->AddNodeAtEnd(new_clusternode);
    }

    return data_link;
}

CLinkListNode* SlowClustering::WhoIsMaximum( CLinkList *data_link )
{
    CLinkListNode *node1, *who;
    int max, temp;
    if ( (node1 = data_link->GetFirst()) == NULL )
        return NULL;

    max = NODEELE(node1)->count;
    who = node1;

    while( node1 != NULL )
    {
        temp = NODEELE(node1)->count;

        if ( temp > max )
        {
            max = temp;
            who = node1;
        }

        node1 = data_link->GetNext(node1);
    }
    return who;
}

void SlowClustering::Clustering(CLinkList *data_link1, CLinkList *data_link2, double min_distance)
{
    CLinkListNode *node1, *node2, *center, *pair1, *pair2, *pair_temp;
    bool flag;

    //there is no cluster in link1
    if ( data_link1->GetLength() < 1 )
        return;
    //there is 1 cluster in link1
    else if ( data_link1->GetLength() == 1 )
    {
        //move the cluster from link1 to link2
        node1 = data_link1->GetFirst();
        data_link1->RemoveNode(node1);
        data_link2->AddNodeAtStart(node1);
        return;
    }

    //there are more than two clusters in link1
    CLinkList *data_pairs = new CLinkList();

    //set count of each cluster in link1 to 0
    node1 = data_link1->GetFirst();
    while( node1 != NULL )
    {
        NODEELE(node1)->count = 0;
        node1 = data_link1->GetNext(node1);
    }

    //find "close" pairs in node 2
    node1 = data_link1->GetFirst();
    IIHash *IIHash1, *IIHash2;
    while ( node1 != NULL )
    {
        IIHash1 = ((ClusterNode *)node1->element)->itemHash;

        node2 = data_link1->GetNext(node1);
        while ( node2 != NULL )
        {
            IIHash2 = ((ClusterNode *)node2->element)->itemHash;
            
            double dis = GetDistance(IIHash1, IIHash2);
            if (dis > min_distance)
                flag = false;
            else
                flag = true;

            if ( flag == true )
            {
                DataPair *pair = new DataPair(node1, node2);
                data_pairs->AddNodeAtEnd(pair);
                NODEELE(node1)->count++;
                NODEELE(node2)->count++;
            }

            node2 = data_link1->GetNext(node2);
        }

        node1 = data_link1->GetNext(node1);
    }

    IIHash *hash1, *newHash;
    while ( (center = WhoIsMaximum(data_link1)) != NULL &&  NODEELE(center)->count > 0 )
    {
        CLinkListNode *new_node = new CLinkListNode();
        NODEELE(center)->parent = new_node;
        data_link1->RemoveNode(center);
        data_link2->AddNodeAtStart(center);

        int entries = NODEELE(center)->entries;
        newHash = new IIHash();
        //move all items in center into the newhash
        hash1 = NODEELE(center)->itemHash;
        if (hash1->size() != 0)
        {
            IIHash::iterator it;
            it = hash1->begin();
            while (it != hash1->end())
            {
                (*newHash)[it->second] = it->second;
                it ++;
            }
        }
        hash1->clear();

        pair1 = data_pairs->GetFirst();
        while ( pair1 != NULL && NODEELE(center)->count > 0)
        {
            node1 = NULL;
            if ( PAIRELE(pair1)->p1 == center )
                node1 = PAIRELE(pair1)->p2;
            else if ( PAIRELE(pair1)->p2 == center )
                node1 = PAIRELE(pair1)->p1;

            if ( node1 != NULL )
            {
                
                NODEELE(center)->count--;
                NODEELE(node1)->count--;
                
                NODEELE(node1)->parent = new_node;
                data_link1->RemoveNode(node1);
                data_link2->AddNodeAtStart(node1);

                entries += NODEELE(node1)->entries;

                //move all items in node1 into the newhash
                hash1 = NODEELE(node1)->itemHash;
                if (hash1->size() != 0)
                {
                    IIHash::iterator it;
                    it = hash1->begin();
                    while (it != hash1->end())
                    {
                        (*newHash)[it->second] = it->second;
                        it ++;
                    }
                }
                hash1->clear();

                pair_temp = pair1;
                pair1 = data_pairs->GetNext(pair_temp);
                data_pairs->DeleteNode(pair_temp);

                pair2 = data_pairs->GetFirst();
                while ( pair2 != NULL && NODEELE(node1)->count > 0)
                {
                    node2 = NULL;
                    if ( PAIRELE(pair2)->p1 == node1 )
                        node2 = PAIRELE(pair2)->p2;
                    else if ( PAIRELE(pair2)->p2 == node1 )
                        node2 = PAIRELE(pair2)->p1;
                    if ( node2 != NULL )
                    {
                        NODEELE(node2)->count--;
                        NODEELE(node1)->count--;
                        pair_temp = pair2;
                        pair2 = data_pairs->GetNext(pair_temp);
                        if ( pair1 == pair_temp )
                            pair1 = pair2;
                        data_pairs->DeleteNode(pair_temp);
                    }
                    else
                        pair2 = data_pairs->GetNext(pair2);
                }
            }
            else
                pair1 = data_pairs->GetNext(pair1);
        }

        ClusterNode *new_clusternode = new ClusterNode();
        new_clusternode->count = 0;
        new_clusternode->entries = entries;
        new_clusternode->parent = NULL;
        new_clusternode->itemHash = newHash;
        
        new_node->element = new_clusternode;
        
        data_link1->AddNodeAtStart(new_node);
    }

    delete data_pairs;
    
    if ( data_link1->GetLength() == 1 )
    {
        node1 = data_link1->GetFirst();
        data_link1->RemoveNode(node1);
        data_link2->AddNodeAtStart(node1);
    }
    
    return;
}

void SlowClustering::WriteOutput(FILE *fp, CLinkList* data_link)
{
    CLinkListNode *node;

    int id = 0;
    node = data_link->GetFirst();
    while ( node != NULL )
    {
        NODEELE(node)->id = id;
        id++;
        node = data_link->GetNext(node);
    }

    fprintf(fp, "%d %d \n", id, size);
    
    int parent_id;
    ClusterNode *cnode;
    node = data_link->GetFirst();
    while ( node != NULL )
    {
        cnode = NODEELE(node);
        if ( cnode->parent != NULL )
            parent_id = NODEELE(cnode->parent)->id;
        else
            parent_id = -1;

        fprintf(fp, "%d %d %d %d \n", cnode->id, parent_id, cnode->entries, cnode->itemID);
        node = data_link->GetNext(node);
    }
}


bool SlowClustering::OnlineCluster(const char* cov_file_name, const char* cg_file_name, int times)
{
    int i;

    FILE *fp1 = fopen(cov_file_name, "r");
    
    if ( fp1 == NULL )
        return false;
    
    FILE *fp2 = fopen(cg_file_name, "w");

    CLinkList *data_link1 = GetInput(fp1);
    fclose(fp1);
    if ( data_link1 == NULL )
    {
        fclose(fp2);
        return false;
    }

    CLinkList *data_link2 = new CLinkList();

    double *max_radiuses = new double[times];
    max_radiuses[times-1] = 1.0;
    for ( i = times-1; i > 0; i-- )
         max_radiuses[i-1] = max_radiuses[i] - 1.0 / times;

    for ( i = 0; i < times; i++ )
    {
//        printf("%d: %lf\n", i, max_radiuses[i]);
        Clustering(data_link1, data_link2, max_radiuses[i]);
    }
	
    WriteOutput(fp2, data_link2);

    fclose(fp2);
    delete data_link1;
    delete data_link2;

    for (i = 0; i < size; i++)
        delete [] covs[i];
    delete [] covs;

    return true;
}

/* average
double SlowClustering::GetDistance(IIHash* hash1, IIHash* hash2)
{
    int count = hash1->size() * hash2->size();
    if (count == 0)
        return 0;

    int i, j;
    double total_distance = 0;
    IIHash::iterator it1, it2;
    it1 = hash1->begin();
    while (it1 != hash1->end())
    {
        i = it1->second;
        it2 = hash2->begin();
        while (it2 != hash2->end())
        {
            j = it2->second;
            total_distance += covs[i][j];
            it2 ++;
        }
        it1 ++;
    }

    return total_distance / count;
}
*/


// minimum
double SlowClustering::GetDistance(IIHash* hash1, IIHash* hash2)
{
    int count = hash1->size() * hash2->size();
    if (count == 0)
        return 0;

    int i, j;
    double total_distance = 10;
    IIHash::iterator it1, it2;
    it1 = hash1->begin();
    while (it1 != hash1->end())
    {
        i = it1->second;
        it2 = hash2->begin();
        while (it2 != hash2->end())
        {
            j = it2->second;
            if (covs[i][j] < total_distance)
                total_distance = covs[i][j];
            it2 ++;
        }
        it1 ++;
    }

    return total_distance;
} 

/*
// maximum
double SlowClustering::GetDistance(IIHash* hash1, IIHash* hash2)
{
    int count = hash1->size() * hash2->size();
    if (count == 0)
        return 0;

    int i, j;
    double total_distance = -10;
    IIHash::iterator it1, it2;
    it1 = hash1->begin();
    while (it1 != hash1->end())
    {
        i = it1->second;
        it2 = hash2->begin();
        while (it2 != hash2->end())
        {
            j = it2->second;
            if (covs[i][j] > total_distance)
                total_distance = covs[i][j];
            it2 ++;
        }
        it1 ++;
    }

    return total_distance;
}
*/