#include <iostream>
#include <fstream>
#include "ColorSetting.h"
#include "colorramp.h"
#include "../Basic/colorassign.h"


ColorSetting::ColorSetting()
{
    ramp = NULL;
	colors = NULL;

    SetColorScale(UNION);
    SetColor(RGBt(0xffffff));
}	

ColorSetting::ColorSetting(RGBt baseColor)
{
    SetColorScale(UNION);
    SetColor(baseColor);
    ramp = NULL;
}	

ColorSetting::~ColorSetting()
{
    if (colors != NULL)
    {
        for (int i = 0; i < 256; i++)
            delete colors [i];
        delete colors;
    }

    if (ramp != NULL)
        delete ramp;
}

bool ColorSetting::IsSingleColor()
{
    if (colorScale == UNION)
        return true;
    else 
        return false;
}

RGBt ColorSetting::GetColor()
{
    return baseColor;
}

RGBt ColorSetting::GetColor(float value)
{
    if (colorScale == UNION)
        return baseColor;

	RGBt c;
    if (colorScale == SBB_COLOR)
    {
   		int index	= (int)(value * 255);
//   		int index	= (int)(value * 215 + 20);
        c = RGBt(ramp->Evaluate(index, 0), 
                 ramp->Evaluate(index, 1), 
                 ramp->Evaluate(index, 2),
                 ramp->Evaluate(index, 3));
    }
	else if (colorScale == LIGHTNESS)
	{
		HLScolor hls (baseColor.GetColor());
		hls.lum = value;
		hls.toRGB(c);
	}
	else if (colorScale == SATURATION)
	{
		HLScolor hls (baseColor.GetColor());
		hls.sat = value;
        hls.toRGB(c);
	}
	else if (colorScale == SATURATION_LIGHTNESS)
	{
		HLScolor hls (baseColor.GetColor());
		hls.sat = value;
		hls.lum = value;
        hls.toRGB(c);
	}
    else if (colorScale == GREYS_CALE)
	{
		HLScolor hls (240, 0, 0);			//grey scale
		hls.lum = value;
		hls.toRGB(c);
	}
    else if (colorScale == HEATED_OBJECT)
	{
		int color_index	= (int)(value * 255);
        c = RGBt(colors[color_index]->R, colors[color_index]->G, colors[color_index]->B);
	}
    else if (colorScale == MAGENTA)
	{
		int color_index	= (int)(value * 255);
        c = RGBt(colors[color_index]->R, colors[color_index]->G, colors[color_index]->B);
	}

    return c;
}

void ColorSetting::SetColor(RGBt color)
{
    baseColor = color;
}

bool ColorSetting::SetColorScale(ColorScale colorScale)
{
    if (colors != NULL)
    {
        for (int i = 0; i < 256; i++)
            delete colors [i];
        delete colors;

        colors = NULL;
    }

    if (colorScale == MAGENTA)
    {
        if (!LoadMagentaColorData())
            return false;
    }
    else if (colorScale == HEATED_OBJECT)
    {
        if (!LoadHeatedObjectColorData())
            return false;
    }
    else if (colorScale == SBB_COLOR)
    {
        if (ramp != NULL)
            delete ramp;
        ramp = ColorRamp::GetPredefined2(256);
    }

    this->colorScale = colorScale;
//    singleColor = false;
    return true;
}

void ColorSetting::InvertColor()
{
    baseColor = RGBt(1.-baseColor.R, 1.-baseColor.G, 1.-baseColor.B, baseColor.A);
}

bool ColorSetting::LoadMagentaColorData()
{
    FILE *fptr = fopen("ColorData\\magenta.dat","rb");
    if (!fptr) 
        return false;
    else
        fclose(fptr);

    std::ifstream infile("ColorData\\magenta.dat");
   
    colors = new RGBt * [256];
    int r, g, b;
    for (int i = 0; i < 256; i++)
    {
        infile >> r;
        infile >> g;
        infile >> b;
        colors[i] = new RGBt(r/255., g/255., b/255.);
    }
    infile.close();

    return true;
}

bool ColorSetting::LoadHeatedObjectColorData()
{
    FILE *fptr = fopen("ColorData\\heatedobject.dat","rb");
    if (!fptr) 
        return false;
    else
        fclose(fptr);

    std::ifstream infile("ColorData\\heatedobject.dat");
   
    colors = new RGBt * [256];
    int r, g, b;
    for (int i = 0; i < 256; i++)
    {
        infile >> r >> g >> b;
        colors[i] = new RGBt(r/255., g/255., b/255.);
    }
    infile.close();

    return true;
}
