//You don't need to modify this file
#include <string>
#include "connection.h"
#include "Statement.h"
#include "PreparedStatement.h"
#include "odbc.h"


Connection :: Connection(std::string username,
						 std::string passwd,
						 std::string dsnSource)
{
	//Copying the values.
	this->userName = username;
	this->passwd = passwd;
	this->dsnSource = dsnSource;
	//Make a new data base connection.
	this->db = new otl_connect;
	//Forming the connection string
	std::string connectionString = username + "/" +passwd+ "@" + dsnSource;	
	//Making the connection
	this->db->rlogon(connectionString.c_str());
}


/**
 * Creates a statement object
 * \param sqlStatement 
 * \return  Statement *
 * @post It is the caller's responsibility to remove the object
 */
Statement* Connection ::createStatement()
{
	return(new Statement(this->db));	
}


/**
 * Creates a new Prepared Statement
 * \param sqlStatement 
 * \param bufferSize, gives the size of the buffer prepared statement uses.
 * Note it should not be less than one.
 * @post: The caller has to delete the object created by the connection.
 * If not deleted the state of the data base may be inconsistent with the
 * regards to the queries executed.
 * \return PreparedStatement
 */
PreparedStatement* Connection :: makePreparedStatement(std::string sqlStatement,
															const unsigned int bufferSize)
{	
	return(new PreparedStatement(sqlStatement,this->db,bufferSize));
}

/**
 * Commits all the pending transactions
 */
void Connection :: commit()
{
	this->db->commit();
}
/**
 * Destroys the connection.
 * \return 
 */
Connection::~Connection()
{
	this->db->logoff(); // disconnect from the database
	delete(this->db);
}


