#include "BinIterator.h"
#include "DocRow.h"
#include "../Interface/UserNotifier.h"


BinIterator::BinIterator(Iterator* rowIter, DimensionSpace* dimSpace)
{
    binNum = 0;
    binCounts = NULL;
    docBin = NULL;
    cardinalities = NULL;
    intervals = NULL;
    sortArray = NULL;
    xPos = NULL;
    yPos = NULL;

    this->dimSpace = dimSpace;

    dimNum = dimSpace->GetSize();

    if (dimNum == 0)
        return;

    cardinalities = new int [dimNum];
    intervals = new int [dimNum];

    int i;
    binNum = 1;
    for (i = 0; i < dimNum; i++)
    {
        cardinalities[i] = dimSpace->GetDimension(i)->GetCardinality();
        binNum *= cardinalities[i];
        if (binNum > 100000 || binNum <= 0)
        {
            binNum = 0;
            count = 0; 
            notEmptyBinNum = 0;
            docBin = new DocBin(dimSpace);
            UserNotifier::ShowMessageDialog("Error Message", "There are too many bins. Please reduce the number of dimensions selected. ");
            return;
        }

    }

    intervals[dimNum-1] = 1;
    for (i = dimNum-2; i >= 0; i--)
    {
        intervals[i] = cardinalities[i+1] * intervals[i+1];
    }

    binCounts = new int[binNum];

    for (i = 0; i < binNum; i++)
        binCounts[i] = 0; 

    int index;
    DocRow* row;
    total_count = 0;
    while (!rowIter->IsDone())
    {
        index = 0; 
        row = (DocRow*)(rowIter->GetNext());
        for (i = 0; i < dimNum; i++)
        {
            index += row->GetCard(i) * intervals[i];
        }
        binCounts[index] ++;
        total_count ++;
    }

    count = 0;
    docBin = new DocBin(dimSpace);
    docBin->SetTotalCount(total_count);
    ///sorting start

    notEmptyBinNum = 0;
    for (i = 0; i < binNum; i++)
    {
        if (binCounts[i] != 0)
            notEmptyBinNum ++;
    }
    docBin->SetBinNumber(notEmptyBinNum);

    sort = true;
    if (sort)
    {

	    sortArray = new SortStruct[notEmptyBinNum];
        int tempCount = 0; 
        int index;
        double value;
        for (i = 0; i < binNum; i++)
        {
            if (binCounts[i] != 0)
            {
                sortArray[tempCount].number = i;

                index = i;
                value = 0; 
                for (int j = 0; j < dimNum; j ++)
                {
                    value += index / intervals[j];
                    index = index % intervals[j];
                }
                value -= binCounts[i] / (float) total_count;

                sortArray[tempCount].value = value;
                tempCount ++;
            }
        }
	    
    //	if ( increase == true )
    //		qsort( sortArray, data_size, sizeof( SortStruct ), IncreaseSort );
    //	else 
		    qsort( sortArray, notEmptyBinNum, sizeof( SortStruct ), DecreaseSort );
    
         xPos = new double[notEmptyBinNum];
         yPos = new double[notEmptyBinNum];
         double x, y;
//         GetSpiralPositions(notEmptyBinNum, xPos, yPos, x, y);
         GetHorizontalPositions(notEmptyBinNum, xPos, yPos, x, y);
         docBin->SetXSize(x/2 - 0.001);
         docBin->SetYSize(y/2 - 0.001);
    }

    //sorting end
    
}	

BinIterator::~BinIterator()
{
    if (binCounts != NULL)
        delete [] binCounts;
    if (cardinalities != NULL)
        delete [] cardinalities;
    if (intervals != NULL)
        delete [] intervals;
    if (docBin != NULL)
        delete docBin;
    if (sortArray != NULL)
        delete [] sortArray;
    if (xPos != NULL)
        delete [] xPos;
    if (yPos != NULL)
        delete [] yPos;
}



void* BinIterator::GetNext()
{
    if (!sort)
    {
        if (count > binNum)
            return NULL;

        docBin->SetCount(binCounts[count]);
        int index = count;
        for (int i = 0; i < dimNum; i ++)
        {
            docBin->SetIndex(i, index / intervals[i]);
            index = index % intervals[i];
        }
        docBin->SetBinID(count);
        
        count ++;
        return docBin;
    }
    else
    {
        if (count > notEmptyBinNum)
            return NULL;

        int currBinID = sortArray[count].number;
        docBin->SetCount(binCounts[currBinID]);
        int index = currBinID;
        for (int i = 0; i < dimNum; i ++)
        {
            docBin->SetIndex(i, index / intervals[i]);
            index = index % intervals[i];
        }
        docBin->SetBinID(currBinID);
        docBin->SetXPosition(xPos[count]);
        docBin->SetYPosition(yPos[count]);
        
        count ++;
        return docBin;
    }
}
    

bool BinIterator::IsDone()
{
    if (!sort)
    {
	    if (count >= binNum)
		    return true;
	    else
		    return false;
    }
    else
    {
        if (count >= notEmptyBinNum)
		    return true;
	    else
		    return false;
    }
}

