#include<string>
#include<math.h>
#include <fstream>
#include <sstream>
#include <fstream>

#include "NumericSimilarityCalculator.h"

NumericSimilarityCalculator :: NumericSimilarityCalculator()
{
    bins = NULL;
    similarityMatrix = NULL;
}

NumericSimilarityCalculator :: NumericSimilarityCalculator(int numDims, int numBins)
{
    bins = NULL;
    similarityMatrix = NULL;

    init(numDims, numBins);
}

bool NumericSimilarityCalculator :: Calculate()
{
    CalculateSimilarityMatrix2();
    return true;
}

bool NumericSimilarityCalculator :: CalculateAroundCenter(int centerID)
{
    CalculateSimilarityMatrixAroundCenter(centerID);
    return true;
}


NumericSimilarityCalculator :: ~NumericSimilarityCalculator()
{
    int i;

    if (bins != NULL)
    {
        for (i = 0; i < numBinRows; i++)
        {
            delete [] bins[i];
        }
        delete [] bins;
    }

    if (similarityMatrix != NULL)
    {
        for (i = 0; i < numDims; i++)
        {
            delete [] similarityMatrix[i];
        }
        delete [] similarityMatrix;
    }
}

bool NumericSimilarityCalculator :: init(int numDims, int numBins)
{
    numDataItems = 0;
    //set numDims and numBins;
    if (numDims < 1)
        return false;
    if (numBins < 1)
        return false;
    this->numDims = numDims;
    this->numBins = numBins;
    this->numBinRows = (numDims+1)*numDims/2;

    int i, j, k;

    //init bins;
    bins = new double*[numBinRows];
    for (i = 0; i < numBinRows; i++)
    {
        bins[i] = new double[numBins];
    }

    for (i = 0; i < numBinRows; i++)
    {
        for (k = 0; k < numBins; k++)
        {
            bins[i][k] = 0;
        }
    }

    //init similarityMatrix;
    similarityMatrix = new double*[numDims];
    for (i = 0; i < numDims; i++)
    {
        similarityMatrix[i] = new double[numDims];
    }

    for (i = 0; i < numDims; i++)
    {
        for (j = 0; j < numDims; j++)
        {
            similarityMatrix[i][j] = similarityMatrix[j][i] = 1;
        }
    }
    return true;
}

bool NumericSimilarityCalculator :: Update(double* values)
{
    int i, j;

    for (i = 0; i < numDims; i++)
    {
        for (j = i; j < numDims; j++)
        {
            int index = (int)(fabs(values[i] - values[j])*numBins);
            if (index >= numBins)
                index = numBins-1;
            bins[GetIndex(i,j)][index] += 1;
//            if (i != j)
//                bins[j][i][index] += 1;
        }
    }

    numDataItems += 1;
    return true;
}

//threshold to handle outliers to be added
void NumericSimilarityCalculator :: CalculateSimilarityMatrix3()
{
    int i;
    int it = 10;
    double interval = 1.0/it;

    double thre = interval;
    double best_thre = thre;
    double max_var = 0;
    for (i = 1; i < it+1; i++)
    {
        CalculateSimilarityMatrix(thre);
        double var = CalculateDistanceVariance();
        if (var > max_var)
        {
            max_var = var;
            best_thre = thre;
        }
        
        thre += interval;
    }


    int i2;
    int it2 = 100;
    double interval2 = 1.0/it2;

    double thre2 = interval2;
    double best_thre2 = thre2;
    double max_var2 = 0;
    for (i2 = 1; i2 < it2/2; i2++)
    {
        CalculateSimilarityMatrix2(thre2);
        double var = CalculateDistanceVariance();
        if (var > max_var2)
        {
            max_var2 = var;
            best_thre2 = thre2;
        }
        
        thre2 += interval2;
    }

    if (max_var2 > max_var)
        CalculateSimilarityMatrix2(best_thre2);
    else
        CalculateSimilarityMatrix(best_thre);
}

//threshold to handle outliers to be added
void NumericSimilarityCalculator :: CalculateSimilarityMatrix()
{
    int i;
    int it = 10;
    double interval = 1.0/it;

    double thre = interval;
    double best_thre = thre;
    double max_var = 0;
    for (i = 1; i < it+1; i++)
    {
        CalculateSimilarityMatrix(thre);
        double var = CalculateDistanceVariance();
        if (var > max_var)
        {
            max_var = var;
            best_thre = thre;
        }
        
        thre += interval;
    }

    CalculateSimilarityMatrix(best_thre);

}

//threshold to handle outliers to be added
void NumericSimilarityCalculator :: CalculateSimilarityMatrix2()
{
    int i;
    int it = numBins;
    double interval = 1.0/it;

    double thre = interval;
    double best_thre = thre;
    double max_var = 0;
    for (i = 1; i < it/2; i++)
    {
        CalculateSimilarityMatrix2(thre);
        double var = CalculateDistanceVariance();
        if (var > max_var)
        {
            max_var = var;
            best_thre = thre;
        }
        
        thre += interval;
    }

    CalculateSimilarityMatrix2(best_thre);
//    SmoothDistanceMatrix();
}

//threshold to handle outliers to be added
void NumericSimilarityCalculator :: CalculateSimilarityMatrixAroundCenter(int centerID)
{
    int i;
    int it = numBins;
    double interval = 1.0/it;

    double thre = interval;
    double best_thre = thre;
    double max_var = 0;
    for (i = 1; i < it/2; i++)
    {
        CalculateSimilarityMatrix2(thre);
        double var = CalculateDistanceVarianceAroundCenter(centerID);
        if (var > max_var)
        {
            max_var = var;
            best_thre = thre;
        }
        
        thre += interval;
    }

    CalculateSimilarityMatrix2(best_thre);
//    SmoothDistanceMatrix();
}

double NumericSimilarityCalculator :: CalculateDistanceVariance()
{
    int i, j;

    double ave = 0;
    int count = 0;
    for (i = 0; i < numDims; i++)
    {
        for (j = i+1; j < numDims; j++)
        {
            ave += similarityMatrix[i][j];
            count ++;
        }
    }
    ave /= count;

    double var = 0;
    for (i = 0; i < numDims; i++)
    {
        for (j = i+1; j < numDims; j++)
        {
            var += (ave - similarityMatrix[i][j]) * (ave - similarityMatrix[i][j]);
        }
    }

    return var;
}

double NumericSimilarityCalculator :: CalculateDistanceVarianceAroundCenter(int centerID)
{
    int i;

    double ave = 0;
    int count = 0;
    for (i = 0; i < numDims; i++)
    {
        if (i != centerID)
        {
            ave += similarityMatrix[centerID][i];
            count ++;
        }
    }
    ave /= numDims;

    double var = 0;
    for (i = 0; i < numDims; i++)
    {
        if (i != centerID)
        {
            var += (ave - similarityMatrix[centerID][i]) * (ave - similarityMatrix[centerID][i]);
        }
    }

    return var;
}

void NumericSimilarityCalculator :: SmoothDistanceMatrix()
{
    int i, j;

    double ave = 0;
    int count = 0;
    for (i = 0; i < numDims; i++)
    {
        for (j = i+1; j < numDims; j++)
        {
            ave += similarityMatrix[i][j];
            count ++;
        }
    }
    ave /= count;

    double var = 0;
    for (i = 0; i < numDims; i++)
    {
        for (j = i+1; j < numDims; j++)
        {
            var += (ave - similarityMatrix[i][j]) * (ave - similarityMatrix[i][j]);
        }
    }

    var = sqrt(var/(count-1));

    double max_distance = ave + var;
    if (max_distance >= 1)
        return;

    for (i = 0; i < numDims; i++)
    {
        for (j = 0; j < numDims; j++)
        {
            if (similarityMatrix[i][j] > max_distance)
                similarityMatrix[i][j] = max_distance;
        }
    }
    
    return;
}

//threshold to handle outliers to be added
void NumericSimilarityCalculator :: CalculateSimilarityMatrix(double threshold)
{
    int i, j, k;

    int valid_num = (int)(threshold * numDataItems);
    int curr_num;
    for (i = 0; i < numDims; i++)
    {
        for (j = i; j < numDims; j++)
        {
            double sum = 0;
            int index = GetIndex(i,j);

            curr_num = 0;
            for (k = 0; k < numBins; k++)
            {
                if (curr_num < valid_num)
                {
                    curr_num += bins[index][k];
                    if (curr_num <= valid_num )
                        sum += (k) * (k) * bins[index][k];
                    else
                        sum += (k) * (k) * (bins[index][k] - curr_num + valid_num);
                }
            }
            similarityMatrix[i][j] = similarityMatrix[j][i] = sqrt(sum/valid_num) / numBins;
        }
    }

//    for (i = 0; i < numDims; i++)                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
//        similarityMatrix[i][i] = 1;
}

//threshold to handle outliers to be added
void NumericSimilarityCalculator :: CalculateSimilarityMatrix2(double threshold)
{
    int i, j, k, m;

    int valid_bins = (int)(threshold * numBins);
    for (i = 0; i < numDims; i++)
    {
        for (j = i; j < numDims; j++)
        {
            int index = GetIndex(i,j);

            int *temp = new int[numBins];
            
            for (k = 0; k < numBins; k++)
            {
                temp[k] = bins[index][k];
            }

            int total = 0;

            for (m = 0; m < valid_bins; m++)
            {
                int max = 0;
                int which = -1;
                for (k = 0; k < numBins; k++)
                {
                    if (temp[k] > max)
                    {
                        max = temp[k];
                        which = k;
                    }
                }

                if (which != -1)
                {
                    temp[which] = 0;
                    total += max;
                }
            }

            delete [] temp;

            similarityMatrix[i][j] = similarityMatrix[j][i] = 1 - ((double)total)/numDataItems;
        }
    }

}


int NumericSimilarityCalculator ::  GetIndex(int i, int j)
{
    int index = i*numDims-(i-1)*i/2+j-i;
    return index;
}

