#include <FLOAT.h>
#include <string>
#include <fstream>
#include <stdio.h>

#include "DateDimension.h"


//constructor
DateDimension::DateDimension() 
{
    type = DATE_DIMENSION;
    max = -1;
    min = 0;
    firstDate.ParseDate("11/11/1911");
    lastDate.ParseDate("01/11/00");

    origMinString = firstDate.FormatDate().c_str();
    origMaxString = lastDate.FormatDate().c_str();

}
    
//deconstructor
DateDimension::~DateDimension() 
{
}
    
double DateDimension::GetNorValue(std::string valueString)
{
    double days = GetValue(valueString);
    return (days - min) / (max - min);
}

double DateDimension::GetValue(std::string valueString)
{
    wxDateTime thisDate;
    thisDate.ParseDate(valueString.c_str());
    int days = thisDate.Subtract(firstDate).GetDays();
    return days;
}

/*
void DateDimension::SetCardinality(int cardinality)
{
    if (cardinality < 1)
        cardinality = 1;
    this->cardinality = cardinality;
    
    if (cardValues != NULL)
        delete [] cardValues;

    cardValues = new wxDateTime[cardinality + 1];

    cardValues[0] = firstDate;
    int days = lastDate.Subtract(firstDate).GetDays();
    days /= cardinality;
    wxTimeSpan interval = wxTimespan::Days(days);

    for (int i = 1; i < cardinality; i++)
    {
        cardValues[i] = cardValues[i-1].Add(interval);
    }
    cardValues[cardinality] = lastDate;
}*/

void DateDimension::GetLabels(std::string* &labels, double* &values, int &labelNum)
{
    labelNum = cardinality + 1;
    labels = new std::string [labelNum];
    values = new double [labelNum];

    std::string *weekdays = new std::string [labelNum];

    labels[0] = origMinString;
    labels[1] = origMaxString;

    wxDateTime temp; 
    for (int i = 0; i < labelNum; i++)
    {   
        if (i >= 2)
        {
            temp = firstDate;
            temp = temp.Add(wxTimeSpan::Days((cardValues[i-1]*(max - min) + min)));
            labels[i] = temp. FormatDate();
        }
        else if (i == 0)
            temp = firstDate;
        else 
            temp = lastDate;

        wxDateTime::WeekDay weekday = temp.GetWeekDay();
           
        switch (weekday) {
        case wxDateTime::WeekDay::Sun:
            weekdays[i] = "Sunday";
            break;
        case wxDateTime::WeekDay::Mon:
            weekdays[i] = "Monday";
            break;
        case wxDateTime::WeekDay::Tue:
            weekdays[i] = "Tuesday";
            break;
        case wxDateTime::WeekDay::Wed:
            weekdays[i] = "Wednesday";
            break;
        case wxDateTime::WeekDay::Thu:
            weekdays[i] = "Thursday";
            break;
        case wxDateTime::WeekDay::Fri:
            weekdays[i] = "Friday";
            break;
        case wxDateTime::WeekDay::Sat:
            weekdays[i] = "Saturday";
            break;
        }
    }

    for (i = 0; i < labelNum; i++)
    {
        values[i] = GetNorValue(labels[i]);
    }

    for (i = 0; i < labelNum; i++)
    {
        labels[i] = labels[i] + " " + weekdays[i];
    }

    delete [] weekdays;

}

void DateDimension::SetOriginalMaximumString(std::string maxString) 
{
    origMaxString = maxString;
    lastDate.ParseDate(origMaxString.c_str());

    min = -0.5;
    max = lastDate.Subtract(firstDate).GetDays() + 0.5;

}
    
void DateDimension::SetOriginalMinimumString(std::string minString) {
    origMinString = minString;
    firstDate.ParseDate(origMinString.c_str());

    min = -0.5;
    max = lastDate.Subtract(firstDate).GetDays() + 0.5;
}


void DateDimension::ReadMetaInformation(std::ifstream *infile)
{
    char buf[2000];
	std::string str;
	
	(*infile) >> buf;
	str = buf;

    if (str.compare("*") == 0)
    {
        mappingUncertain = true;
	    SetCardinality(5);
    }
    else
    {
        mappingUncertain = false;

	    SetOriginalMinimumString(str);

	    (*infile) >> buf;
	    str = buf;
	    SetOriginalMaximumString(str);

	    int card;
	    (*infile) >> card;
	    SetCardinality(card);
    }
    (*infile).getline(buf, 2000);
}

void DateDimension::ChangeMinMax(std::string valueString)
{
    wxDateTime thisDate;
    thisDate.ParseDate(valueString.c_str());

    if (thisDate.IsEarlierThan(firstDate))
    {
        firstDate = thisDate;
        origMinString = valueString;
    }
    if (thisDate.IsLaterThan(lastDate))
    {
        lastDate = thisDate;
        origMaxString = valueString;
    }

    min = 0.5;
    max = lastDate.Subtract(firstDate).GetDays() + 0.5;
    cardinality = max - min;
}

double DateDimension::GetNorValue(Data data)
{
    std::string *ptr;
    ptr = (std::string *)data.p;
    return GetNorValue(*ptr);
}

