//Here, Data means a row in a table, namely a data item. 

#include "DocRow.h"

DocRow::DocRow(DimensionSpace *dimSpace, bool dataCluster, bool dimCluster)
{
    this->dimSpace = dimSpace;
    this->dimNum = dimSpace->GetSize();
    this->dataCluster = dataCluster;
    this->dimCluster = dimCluster;
//    this->leafDataCluster = true;
    
    if (dataCluster)
        num1 = 3;  //0: mean of data cluster, 1: min of data cluster, 2: max of data cluster
    else
        num1 = 1;

    if (dimCluster)
        num2 = 3;  //0: mean of dim cluster, 1: min of dim cluster, 2: max of dim cluster
    else
        num2 = 1;

    if (dimNum > 0)
    {
        norValues = new double ** [num1];
        for (int i = 0; i < num1; i++)
        {
            norValues[i] = new double * [num2];
            
            for (int j = 0; j < num2; j++)
            {
                norValues[i][j] = new double [dimNum];
            }
        }
        card = new int[dimNum];
    }

    norValueForColor = 0;
    originalDataItem = true;
}

DocRow:: ~DocRow()
{
    if (dimNum > 0)
    {
        for (int i = 0; i < num1; i++)
        {
            for (int j = 0; j < num2; j++)
            {
                delete [] norValues[i][j];
            }
            delete [] norValues[i];
        }
        delete [] norValues;
        delete [] card;
    }
}

void DocRow::SetOriginalDataItem(bool original)
{
    originalDataItem = original;
}

bool DocRow::IsOriginalDataItem()
{
    return originalDataItem;
}

double DocRow::GetNorValue(int dataClusterType, int dimClusterType, int dimID)
{
    return norValues[dataClusterType][dimClusterType][dimID];
}

void DocRow::SetNorValue(int dataClusterType, int dimClusterType, int dimID, double value)
{
    norValues[dataClusterType][dimClusterType][dimID] = value;
}

void DocRow::SetCard(int dimID, int value)
{
    card[dimID] = value;
}

int DocRow::GetCard(int dimID)
{
    return card[dimID];
}

double DocRow::GetNorValueForColor()
{
    return norValueForColor;
}

void DocRow::ResetData(Data* data)
{
    Dimension* dim;
    LinkListIterator* iter = dimSpace->IterateDimensions();
    int i = 0;
    while (!iter->IsDone())
    {
        dim = (Dimension *)(iter->GetNext());
        if (dimCluster)
        {
            double v, vMin, vMax;
            dim->GetNorValueWithMinMax(data, v, vMin, vMax);
            norValues[0][0][i] = v;
            norValues[0][1][i] = vMin;
            norValues[0][2][i] = vMax;
        }
        else
        {
            double v;
            dim->GetNorValue(data, v);
            norValues[0][0][i] = v;
        }
        i++;
    }
    delete iter;

    dim = dimSpace->GetColorDimension();  
    if (dim!= NULL)
       dim->GetNorValue(data, norValueForColor);
}

void DocRow::ResetCards(Data* data)
{
    Dimension* dim;
    LinkListIterator* iter = dimSpace->IterateDimensions();
    int i = 0;
    while (!iter->IsDone())
    {
        dim = (Dimension *)(iter->GetNext());
        card[i] = dim->GetCard(data);
        i++;
    }
    delete iter;
}


void DocRow::ResetData(Cluster* cluster)
{
    originalDataItem = cluster->IsLeaf();
    Dimension* dim;
    LinkListIterator* iter = dimSpace->IterateDimensions();
    int i = 0;
    double v, vMin, vMax;
    while (!iter->IsDone())
    {
        dim = (Dimension *)(iter->GetNext());
        int dimID = dim->GetDimensionID();
        if (dimCluster)
        {
            dim->GetNorValueWithMinMax(cluster->GetMean(dimID), v, vMin, vMax);
            norValues[0][0][i] = v;
            norValues[0][1][i] = vMin;
            norValues[0][2][i] = vMax;

            if ( ! originalDataItem)
            {
                dim->GetNorValueWithMinMax(cluster->GetMin(dimID), v, vMin, vMax);
                norValues[1][0][i] = v;
                norValues[1][1][i] = vMin;
                norValues[1][2][i] = vMax;
                dim->GetNorValueWithMinMax(cluster->GetMax(dimID), v, vMin, vMax);
                norValues[2][0][i] = v;
                norValues[2][1][i] = vMin;
                norValues[2][2][i] = vMax;
            }
        }
        else
        {
            v = dim->GetNorValue(cluster->GetMean(dimID));
            norValues[0][0][i] = v;
            if ( ! originalDataItem)
            {
                v = dim->GetNorValue(cluster->GetMin(dimID));
                norValues[1][0][i] = v;
                v = dim->GetNorValue(cluster->GetMax(dimID));
                norValues[2][0][i] = v;
            }
        }
        i++;
    }
    delete iter;

    norValueForColor = cluster->GetNorPosition();
}


/*
void DocRow::ResetData(Data* data)
{
    Dimension* dim;
    int index;
    LinkListIterator* iter = dimSpace->IterateDimensions();
    int i = 0;
    while (!iter->IsDone())
    {
        dim = (Dimension *)(iter->GetNext());
        index = dim->GetDimensionID();
        norValues[i] = dim->GetNorValue(data[index]);
        i++;
    }

    dim = dimSpace->GetColorDimension();  
    if (dim!= NULL)
    {
       index = dim->GetDimensionID();
       norValueForColor = dim->GetNorValue(data[index]);
    }
    delete iter;
}
*/

/*
DocRow::	DocRow(int dimNum, bool extents, int valuesForColorNum, int descriptiveNum)
{
    int i;

    this->dimNum = dimNum;
    this->extents = extents;
    
    if (dimNum > 0)
    {
        for (i = 0; i < dimNum; i++)
            norAverages = new double[dimNum];

        if (extents)
        {
            for (i = 0; i < dimNum; i++)
            {
                norMaxes = new double[dimNum];
                norMins = new double[dimNum];
            }
        }
    }

    if (valuesForColorNum > 0)
    {
        norValuesForColor = new double[valuesForColorNum];
    }

    if (descriptiveNum > 0)
    {
        descritpiveValues = new void*[descriptiveNum];
    }

    coverage = 1;
}	

DocRow:: ~DocRow()
{
    if (dimNum > 0)
    {
        delete [] norAverages;

        if (extents)
        {
            delete [] norMaxes;
            delete [] norMins;
        }
    }

    if (valuesForColorNum > 0)
    {
        delete [] norValuesForColor;
    }

    if (descriptiveNum > 0)
    {
        delete [] descritpiveValues;
    }
}

void DocRow::SetNorAverages(double* norAverages)
{
    for (int i = 0; i < dimNum; i++)
    {
        this->norAverages[i] = norAverages[i];
    }
}
void DocRow::SetNorMaxes(double* norMaxes)
{
    if (!extents)
        return;

    for (int i = 0; i < dimNum; i++)
    {
        this->norMaxes[i] = norMaxes[i];
    }
}

void DocRow::SetNorMins(double* norMins)
{
    if (!extents)
        return;

    for (int i = 0; i < dimNum; i++)
    {
        this->norMins[i] = norMins[i];
    }
}

void DocRow::SetNorValuesForColor(double* value)
{
    for (int i = 0; i < valuesForColorNum; i++)
    {
        this->norValuesForColor[i] = norValuesForColor[i];
    }
}

void DocRow::SetCoverage(double coverage)
{
    this->coverage = coverage;
}

void DocRow::GetNorAverages(ARR<double> &norAverages)
{
    for(int i = 0; i < dimNum; i++)
        norAverages[i] = this->norAverages[i];
}

void DocRow::GetNorMaxes(ARR<double> &norMaxes)
{
    if (!extents)
    {
        for(int i = 0; i < dimNum; i++)
            norMaxes[i] = this->norAverages[i];
    }
    else
    {
        for(int i = 0; i < dimNum; i++)
            norMaxes[i] = this->norMaxes[i];
    }
}

void DocRow::GetNorMins(ARR<double> &norMins)
{
    if (!extents)
    {
        for(int i = 0; i < dimNum; i++)
            norMins[i] = this->norAverages[i];
    }
    else
    {
        for(int i = 0; i < dimNum; i++)
            norMins[i] = this->norMins[i];
    }
}

void DocRow::GetNorValuesForColor(ARR<double> &norValuesForColor)
{
    for (int i = 0; i < valuesForColorNum; i++)
    {
        norValuesForColor[i] = this->norValuesForColor[i];
    }
}

void DocRow::GetCoverage(double coverage)
{
    coverage = this->coverage;
}
*/


/*void DocRow::GetNorValues(ARR<double> &norValues)
{
    for(int i = 0; i < dimNum; i++)
        norValues[i] = this->norValues[i];
}*/
