#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include "InMemoryHierDataSet.h"
#include "InMemoryHierRowIterator.h"

InMemoryHierDataSet::InMemoryHierDataSet()
{
    clusters = NULL;
    root = NULL;
    sbBrush = NULL;

    dimNum = 0;
    nodeNum = 0;
    leafNodeNum = 0;
    maxLevel = 0; 
}

InMemoryHierDataSet::~InMemoryHierDataSet()
{
    if (clusters != NULL)
    {
        for (int i = 0; i < nodeNum; i++)
        {
            delete clusters[i];
        }
    }
    delete clusters;

    if (sbBrush != NULL)
        delete sbBrush;
}


bool InMemoryHierDataSet::ReadData(std::string fname, DimensionSpace* dimSpace)
{
    //test if file exists. If not, online generate one.
    FILE *fptr = fopen(fname.c_str(),"rb");
/*    if (fptr==NULL) {
        OnlineClusterOkcFileToCgFile(g_globals.dataset->datafile, 100, true);
        fptr = fopen(fname,"rb");
        if (fptr==NULL) 
            return false;
    }
*/
    //test if binary file.
    unsigned int magic_code;
    fread(&magic_code, sizeof(unsigned int), 1, fptr);
    
    fclose(fptr);

    bool success;
    if ( magic_code == 0xcecfcecf )
        success = ReadBinaryData(fname, dimSpace);
    else
        success = ReadNormalData(fname, dimSpace);

    if (success)
    {
        root = clusters[0];
        AssignPositions(root, 0, 1);
    }

    sbBrush = new SBBrush();
    sbBrush->SetHierDataSet(this);
    return success;
}


bool InMemoryHierDataSet::ReadNormalData(std::string fname, DimensionSpace* dimSpace)
{
    int k, i;

    std::ifstream infile (fname.c_str());

    infile >> nodeNum >> dimNum; 

    if (dimNum != dimSpace->GetSize() || nodeNum <= 0)
        return false;

    double *min, *max;
    min = new double [dimNum];
    max = new double [dimNum];

    Dimension *dim;
    for (i = 0; i < dimNum; i++)
    {
        dim = dimSpace->GetDimension(i);
        max[i] = dim->GetValue(dim->GetOrigMaxString());
        min[i] = dim->GetValue(dim->GetOrigMinString());
    }

    clusters = new Cluster* [nodeNum];
    for (i = 0; i < nodeNum; i++)
        clusters[i] = new Cluster(dimNum); 

    clusters[0]->SetLevel(1);
    leafNodeNum = 0; 
    maxLevel = 1;

    int id, parent, entries, level;
    double sx, cmax, cmin, sum_dim, sqrt_radius;
    Cluster* cluster;
    for (k = 0; k < nodeNum; k++)
    {
        cluster = clusters[k];

        infile >> id >> parent >> entries;
        
        if (k != 0)
        {
            cluster->SetParent(clusters[parent]);
            clusters[parent]->AddChild(cluster);
            level = clusters[parent]->GetLevel() + 1;
            if (level > maxLevel)
                maxLevel = level;
            cluster->SetLevel(level);
        }
        cluster->SetEntries(entries);

        for (i = 0; i < dimNum; i++)
		{
            infile >> sx;
            if (entries > 0) 
            {
                sum_dim = sx * (max[i] - min[i]) + entries * min[i];
                cluster->SetMean(i, sum_dim/entries);
            }

        }

        infile >> sqrt_radius;
        cluster->SetSqrtRadius(sqrt_radius);

        for (i = 0; i < dimNum; i++)
		{
            infile >> cmax;
            if (cluster->IsLeaf() == false)
            {
                cmax = cmax * (max[i] - min[i]) + min[i];
                cluster->SetMax(i, cmax);
            }
		}

        for (i = 0; i < dimNum; i++)
		{
            infile >> cmin;
            if (cluster->IsLeaf() == false)
            {
                cmin = cmin * (max[i] - min[i]) + min[i];
                cluster->SetMin(i, cmin);
            }
		}

        if (cluster->IsLeaf())
            leafNodeNum ++;
    }

    infile.close();

/*
    // Init the min/max extents
    //
    min_extents.SetLen(g_globals.dataset->all_dims);
    max_extents.SetLen(g_globals.dataset->all_dims);

    for (int d=0; d<g_globals.dataset->all_dims; d++) {
        min_extents[d] = HUGE_VAL;
        max_extents[d] = -HUGE_VAL;
    }

    treemap_limit_number = (int) (nodes[0].entries * TREEMAP_LIMIT);
    if ( treemap_limit_number < 1 )
        treemap_limit_number = 1;
*/
    return true;
}


bool InMemoryHierDataSet::ReadBinaryData(std::string fname, DimensionSpace* dimSpace)
{
    int k, i;
   
    FILE *fptr = fopen(fname.c_str(),"rb");
    if (fptr==NULL) 
        return false;

    unsigned int magic_code;
    fread(&magic_code, sizeof(unsigned int), 1, fptr);

    if ( fread(&nodeNum, sizeof(int), 1, fptr) != 1 || fread(&dimNum, sizeof(int), 1, fptr) != 1 ) 
        return false;

    if (dimNum != dimSpace->GetSize() || nodeNum <= 0)
        return false;

    double * buffer;
    buffer = new double[dimNum];

    double *min, *max;
    min = new double [dimNum];
    max = new double [dimNum];

    Dimension *dim;
    for (i = 0; i < dimNum; i++)
    {
        dim = dimSpace->GetDimension(i);
        max[i] = dim->GetValue(dim->GetOrigMaxString());
        min[i] = dim->GetValue(dim->GetOrigMinString());
    }

    clusters = new Cluster* [nodeNum];
    for (i = 0; i < nodeNum; i++)
        clusters[i] = new Cluster(dimNum); 

    clusters[0]->SetLevel(1);
    leafNodeNum = 0; 
    maxLevel = 1;

    int id, parent, entries, level;
    double sx, cmax, cmin, sum_dim, sqrt_radius;
    Cluster* cluster;
    for (k = 0; k < nodeNum; k++)
    {
        cluster = clusters[k];

        fread(&id, sizeof(int), 1, fptr);
        fread(&parent, sizeof(int), 1, fptr);
        fread(&entries, sizeof(int), 1, fptr);
        
        if (k != 0)
        {
            cluster->SetParent(clusters[parent]);
            clusters[parent]->AddChild(cluster);
            level = clusters[parent]->GetLevel() + 1;
            if (level > maxLevel)
                maxLevel = level;
            cluster->SetLevel(level);
        }
        cluster->SetEntries(entries);

        fread(buffer, sizeof(double), dimNum, fptr);
        for (i = 0; i < dimNum; i++)
		{
            sx = buffer[i];
            if (entries > 0) 
            {
                sum_dim = sx * (max[i] - min[i]) + entries * min[i];
                cluster->SetMean(i, sum_dim/entries);
            }

        }

        fread(&sqrt_radius, sizeof(double), 1, fptr);
        cluster->SetSqrtRadius(sqrt_radius);

        fread(buffer, sizeof(double), dimNum, fptr);
        if (cluster->IsLeaf() == false)
        {
            for (i = 0; i < dimNum; i++)
		    {
                cmax = buffer[i];
                cmax = cmax * (max[i] - min[i]) + min[i];
                cluster->SetMax(i, cmax);
		    }
        }

        fread(buffer, sizeof(double), dimNum, fptr);
        if (cluster->IsLeaf() == false)
        {
            for (i = 0; i < dimNum; i++)
		    {
                cmin = buffer[i];
                cmin = cmin * (max[i] - min[i]) + min[i];
                cluster->SetMin(i, cmin);
		    }
        }
        if (cluster->IsLeaf())
            leafNodeNum ++;
    }

    fclose(fptr);
    delete [] buffer;
    return true;
}

void InMemoryHierDataSet::MarkBrushedClusters(SBBrush *brush)
//void InMemoryHierDataSet::MarkBrushedClusters()
{
    this->sbBrush = brush;
    MarkBrushedClusters(root);
}

void InMemoryHierDataSet::MarkBrushedClusters(Cluster* node)
{
    BITCLEAR(node->flags,IS_BRUSHED);

    if (node->GetChildrenNum() == 0)
    {
        if (sbBrush->IsClusterCovered(node)) 
        {
            BITSET(node->flags,IS_BRUSHED);
        }
        return;
    }

    LinkListIterator* iter = node->IterateChildren();
    
    while (!iter->IsDone())
    {
        MarkBrushedClusters((Cluster *)iter->GetNext());
    }

    if (sbBrush->GetSemantic() == BRUSH_ALL) //all
    {
        bool brushed_all = true;

        iter = node->IterateChildren();
        Cluster *cluster;
        while (!iter->IsDone())
        {
            cluster = (Cluster *)iter->GetNext();
            if (brushed_all == true)
            {
                if (!BITTEST(cluster->flags, IS_BRUSHED)) 
                {
                    brushed_all = false;
                    break;
                }
            }
        }
		if (brushed_all)
            BITSET(node->flags,IS_BRUSHED);
    }
    else
    {
        bool brushed_any = false;

        iter = node->IterateChildren();
        Cluster *cluster;
        while (!iter->IsDone())
        {
            cluster = (Cluster *)iter->GetNext();
            if (brushed_any == false)
            {
                if (BITTEST(cluster->flags, IS_BRUSHED)) 
                {
                    brushed_any = true;
                    break;
                }
            }
        }

        if (brushed_any)
            BITSET(node->flags,IS_BRUSHED);
    }
}


void InMemoryHierDataSet::MarkContourClusters(double contour_radius, int FLAG)
{
    // A node c is on contour if:
    // c.parent.radius > contour_radius AND
    //  (a)  c.radius <= contour_radius, OR
    //  (b)  c.radius > contour_radius AND c is a leaf
    //
    Cluster *cluster, *parent;
    for (int i = 0;i < nodeNum; i++)
    {
        cluster = clusters[i];

        BITCLEAR(cluster->flags, FLAG);

        double par_radius = HUGE_VAL;
        
        parent = cluster->GetParent();
        if (parent != NULL) 
            par_radius = parent->GetSqrtRadius();

        if (par_radius > contour_radius)
        {
            if (cluster->GetSqrtRadius() <= contour_radius 
                || cluster->GetChildrenNum() ==0)
            {
                BITSET(cluster->flags, FLAG);
            }
        }
    }
}

double InMemoryHierDataSet::GetRootSqrtRadius()
{
    return root->GetSqrtRadius();
}


void InMemoryHierDataSet::AssignPositions(Cluster* node, double L, double R)
{
    node->SetNorPosition(0.5*(L+R));

    int childNum = node->GetChildrenNum(); 
    if (childNum == 0) return;

    double k = (R-L)/(double)(childNum);
    
    LinkListIterator* iter = node->IterateChildren();
    
    int i = 0;
    while (!iter->IsDone())
    {
        AssignPositions((Cluster *)iter->GetNext(), L+i*k, L+(i+1)*k);
        i++;
    }
}

Iterator* InMemoryHierDataSet::LoadData(DimensionSpace* dimSpace, bool dimCluster, bool showBand, bool brushed)
{
	return (new InMemoryHierRowIterator(clusters, nodeNum, dimSpace, 
        dimCluster, showBand, brushed));
}

void InMemoryHierDataSet::GetLeafSBBContour(int &num, SBBContourNode* &nodes)
{
    num = leafNodeNum;
    nodes = new SBBContourNode[num];

    int k = 0;
    for (int i = 0; i < nodeNum; i++)
    {
        if (clusters[i]->IsLeaf())
        {
            nodes[k].SetLevel(clusters[i]->GetLevel());
            nodes[k].SetNorPosition(clusters[i]->GetNorPosition());
            k++;
        }
    }
}

void InMemoryHierDataSet::GetSBBContour(bool brushed, int &num, SBBContourNode* &nodes)
{
//	InMemoryHierDataIterator* iter = new InMemoryHierDataIterator(clusters, nodeNum, dimSpace, 
//        false, false, brushed);

}


/*
//jing_flat_sb_start
static bool FlatSBtestCoverage(Brush *brush, Cluster* node)
{
    bool in = true;
	for ( int j =0; j < g_globals.dataset->dims; j++) 
	{
		if ( node->max[j] < brush->Min(j)) 
        {
            in = false;
            continue;
        }
		if ( node->min[j] > brush->Max(j)) 
        {
            in = false;
            continue;
        }
	}

    return in;
}

void hierMarkFlatBrushedClustersByExtents(Brush *brush, Cluster* node)
{
    node->flat_brushed = false;
    node->contain_flat_brushed = false;

    if ( FlatSBtestCoverage(brush, node) == true )
    {
        node->contain_flat_brushed = true;
    }
    else 
    {
        return;
    }

    if ( node->num_children == 0 || node->sqrt_radius <= g_globals.hier_info->brushed_radius )
        node->flat_brushed = true;
    else
    {
        for (int i=0; i<node->num_children; i++)
            hierMarkFlatBrushedClustersByExtents(brush, &(g_globals.btree->nodes[node->child[i]]));
    }
        return;
}

void HierChangeFlatBrushAccordingToClusters()
{
    int i;
    ARR<double> brush_min, brush_max;
    brush_min.SetLen(g_globals.dataset->dims);
    brush_max.SetLen(g_globals.dataset->dims);
    for ( i = 0; i < g_globals.dataset->dims; i++ )
    {
        brush_min[i] = g_globals.dataset->dim_max[i];
        brush_max[i] = g_globals.dataset->dim_min[i];
    }
    TraceJoinFSBBrushedWithClusters(g_globals.btree->nodes[0], brush_min, brush_max);
    for ( i = 0; i < g_globals.dataset->dims; i++ )
    {
        if ( brush_min[i] > brush_max[i] )
            return;
    }
    for ( i = 0; i < g_globals.dataset->dims; i++ )
    {
        g_globals.curr_brush->SetMinMax(i, brush_min[i], brush_max[i]);
    }
}


void JoinBrush( Cluster node, ARR<double> &brush_min, ARR<double> &brush_max)
{
    for ( int i = 0; i < g_globals.dataset->dims; i++ )
    {
        if ( brush_min[i] > node.min[i] )
            brush_min[i] = node.min[i] - 1e-5;
        if ( brush_max[i] < node.max[i] )
            brush_max[i] = node.max[i] + 1e-5;
    }
}

void TraceJoinFSBBrushedWithClusters( Cluster node, ARR<double> &brush_min, ARR<double> &brush_max)
{
    if ( !node.contain_flat_brushed )
        return;

    if ( node.flat_brushed )
    {
       JoinBrush(node, brush_min, brush_max);
    }
    else
    {
        for (int j=0; j<node.num_children; j++) 
            TraceJoinFSBBrushedWithClusters(g_globals.btree->nodes[node.child[j]], brush_min, brush_max);
    }
}

//jing_flat_sb_end
*/

/*
void hierInitClusterTree()
{
    if (g_globals.btree) return;

    g_globals.btree = new ClusterTree;

    int N = strlen(g_globals.dataset->datafile);
//    char str[N+1];
//jing
	char* str = new char[N+1];
//jing
    strcpy(str,g_globals.dataset->datafile);
//jing    strcpy(&str[N-4],".cf");
    strcpy(&str[N-4],".cg");

//jing
	if ( g_globals.hier_info->database )
	{
//jing for future changes		if (!g_globals.btree->ReadClusterTreeFileWithDatabase(str)) 
		if (!g_globals.btree->ReadClusterTreeFile(str)) 
		{
			hierDeleteClusterTree();
			return;
		}
	}
	else
	{
		if (!g_globals.btree->ReadClusterTreeFile(str)) 
		{
			hierDeleteClusterTree();
			return;
		}
	}

    hierInitClusterColors(&g_globals.btree->nodes[0], 0, 1);
    hierClearNodeFlags();
    hierModifyExtentsByBrushParameters();
    hierMarkBrushedClustersByExtents(&g_globals.btree->nodes[0]);

    if ( g_globals.hier_info->database )
    {
        init_memory(g_globals.dataset->datafile);
    }

	CHierHist::cfHistoInit();
    cfNavInit();
    cfTreeMapInit();

    double root_radius = g_globals.btree->nodes[0].sqrt_radius;
    hierMarkContourNodes( root_radius*g_globals.hier_info->cluster_radius, IS_CONTOUR0 );
    hierMarkContourNodes( root_radius*g_globals.hier_info->brushed_radius, IS_CONTOUR1 );
   	g_globals.hier_info->root_radius = g_globals.btree->nodes[0].sqrt_radius;

	//jing
    //try
    hierMarkFlatBrushedClustersByExtents(g_globals.curr_brush, &g_globals.btree->nodes[0]);
	SetupBrushParameters();

//If the data base is on we need to delete the cluster tree from
	if(g_globals.hier_info->database) {
		hierDeleteClusterTree();
	}

	delete [] str;
}

void hierDeleteClusterTree()
{
	if (g_globals.btree) { delete g_globals.btree; g_globals.btree = NULL; }
	CHierHist::cfHistoFinalize();
}
*/
