#include <FLOAT.h>
#include <string>
#include <fstream>
#include <stdio.h>

#include "NorminalDimension.h"


//constructor
NorminalDimension::NorminalDimension() 
{
    type = NORMINAL_DIMENSION;
    max = -1;
    min = 0;

    interval = 1.0;

    groupValues = NULL;
    groupSizes = NULL;
    groupNum = 0;

}
    
//deconstructor
NorminalDimension::~NorminalDimension() 
{
    NorminalHash::iterator it;
    it = hash.begin();
    while (it != hash.end())
    {
        delete (std::string *)(it->second.pointer);
        it ++;
    }

    if (groupValues != NULL)
        delete [] groupValues;
    if (groupSizes != NULL)
        delete [] groupSizes;
}
    
void NorminalDimension::AddString(std::string str, double value)
{
    wxString wStr(str.c_str(), str.length());

    std::string *strPtr;
    strPtr = new std::string(str.c_str());

    hash[wStr] = PAV(strPtr, value);

    if (value >= max)
    {
        max = value;
        origMaxString = str;
        maxString = str;
    }
    if (value <= min)
    {
        min = value;
        origMinString = str;
        minString = str;
    }
}

/*
void NorminalDimension::AddString(std::string str)
{
    currentValue += interval;
    AddString(str, currentValue);
}*/

double NorminalDimension::GetValue(std::string str)
{
    wxString wStr(str.c_str(), str.length());
    return hash[wStr].value;
}

/*int NorminalDimension::GetCard(std::string str)
{
    wxString wStr(str.c_str(), str.length());
    return (int) (hash[wStr].value);
}*/

void NorminalDimension::ChangeMinMax(std::string str)
{
    NorminalHash::iterator it;

    wxString wStr(str.c_str(), str.length());
    
    it = hash.find(wStr);
    if (it == hash.end())
    {
        AddString(str, max + interval);
    }
}

double NorminalDimension::GetNorValue(std::string str)
{
    GetPointer(str);
    wxString wStr(str.c_str(), str.length());
    return Dimension::GetNorValue(hash[wStr].value);

    /* NorminalHash::iterator it;

    wxString wStr(str.c_str(), str.length());
    
    it = hash.find(wStr);
    GetNorValue(it->second.value);
    if (it == textHash.end())
    {
        GetNorValue(defaultValue);
    }
    else
    {
        GetNorValue(it->second.value);
    }*/
}

/*
double NorminalDimension::GetNorValue(double value)
{
    if (min == max)
    {
        if (value == min)
            return 1;
        else
            return 0;
    }
    
    double v = (value - min) / (max - min);

    if (v < 0)
        v = 0;
    else if (v > 1)
        v = 1;

    return v;
}*/

void* NorminalDimension::GetPointer(std::string str)
{
    NorminalHash::iterator it;

    wxString wStr(str.c_str(), str.length());
    
    it = hash.find(wStr);
    if (it == hash.end())
    {
        AddString(str, max + interval);
        return hash[wStr].pointer;
    }
    else
    {
        return it->second.pointer;
    }
}


double NorminalDimension::GetNorValue(Data data)
{
    std::string *ptr;
    ptr = (std::string *)data.p;
    return GetNorValue(*ptr);
}

void NorminalDimension::SetMinMaxString(std::string string1, std::string string2)
{
    double value1, value2;
    value1 = GetNorValue(string1);
    value2 = GetNorValue(string2);

    if (value1 < value2)
    {
        min = value1;
        minString = string1;

        max = value2;
        maxString = string2;
    }
    else
    {
        min = value2;
        minString = string2;

        max = value1;
        maxString = string1;
    }
}

void NorminalDimension::GetLabels(std::string* &labels, double* &values, int &labelNum)
{
    double value, value0, value1;
    value0 = GetNorValue(minString);
    value1 = GetNorValue(maxString);
    
    labelNum = 2;

    NorminalHash::iterator it;
    it = hash.begin();
    while (it != hash.end())
    {
        value = Dimension::GetNorValue(it->second.value);
        if (value > value0 && value < value1)
        {
            labelNum ++;
        }
        it++;
    }

    values = new double [labelNum];
    labels = new std::string [labelNum];

    labels[0] = minString;
    labels[1] = maxString;
    values[0] = value0;
    values[1] = value1;

    int i = 2;
    it = hash.begin();
    while (it != hash.end())
    {
        value = Dimension::GetNorValue(it->second.value);
        if (value > value0 && value < value1)
        {
            labels[i] = it->first.c_str();
            values[i] = value;
            i++;
        }
        it++;
    }
}

void NorminalDimension::GetNorValueWithMinMax(Data* data, double &norValue, double &norMin, double &norMax)
{
    norValue = GetNorValue(data[dimID]);
}

void NorminalDimension::GetNorValueWithMinMax(double value, double &norValue, double &norMin, double &norMax)
{
    norValue = Dimension::GetNorValue(value);
}


/*
void NorminalDimension::GetLabels(std::string* &labels, double* &values, int &labelNum)
{
    labelNum = hash.size();
    labels = new std::string [labelNum];
    values = new double [labelNum];
    labels[0] = origMinString;
    labels[labelNum-1] = origMaxString;
    values[0] = GetNorValue(origMinString);
    values[labelNum-1] = GetNorValue(origMaxString);

    NorminalHash::iterator it;
    it = hash.begin();
    int j = 1;
    for (int i = 0; i < labelNum; i++)
    {
        std::string str = it->first.c_str();
        if (str != origMinString && str != origMaxString)
        {
            labels[j] = str;
            values[j] = Dimension::GetNorValue(it->second.value);
        }
        it ++;
    }
}
*/

void NorminalDimension::WriteMetaInformation(std::ofstream *outfile)
{
    int labelNum = hash.size();

    NorminalHash::iterator it;
    it = hash.begin();
    int j = 1;
    for (int i = 0; i < labelNum; i++)
    {
        (*outfile) << it->first.c_str() << " ("
		        << it->second.value << ") ";
        it ++;
    }

    (*outfile) << std::endl;
}

void NorminalDimension::ReadMetaInformation(std::ifstream *infile)
{
    char buf[25000];
    char *separator = ", \t\n";  // separators required by strtok() function

    std::string str1, str2;
    double value;

    (*infile).getline(buf, 25000); // 
    char *token = strtok(buf, separator);  

    str1 = token;
    while (str1.compare("*") != 0)
	{
        token = strtok( NULL, separator);
        str2 = token;
        str2 = str2.substr(1, str2.length()-1);
        value = atof(str2.c_str());

        AddString(str1, value);
        token = strtok( NULL, separator);
        if (token == NULL)
            break;
        str1 = token;
    } 

    if (token != NULL)
        mappingUncertain = true;
    else
        mappingUncertain = false;

    cardinality = hash.size();

}

void NorminalDimension::GetGroups(std::string* &groupValues, int* &groupSizes, int &groupNum)
{
    groupValues = this->groupValues;
    groupSizes = this->groupSizes;
    groupNum = this->groupNum;
}

void NorminalDimension::SetGroups(std::string* groupValues, int* groupSizes, int groupNum)
{
    this->groupValues = groupValues;
    this->groupSizes = groupSizes;
    this->groupNum = groupNum;
}
