#ifndef _Query_H
#define _Query_H

#include <string>

#define SINGLE_QUERY 0
#define SCATTER_QUERY 1
#define XY_QUERY 2

#define NONE_GROUP 0
#define MASK_GROUP 1
#define HIDE_OTHER_GROUPS 2
#define HIDE_GROUPS 3

#define RANDOM_SAMPLING 1
#define BIASED_SAMPLING 2

class Query
{
public:
    int queryMode;
    int groupingMode;

    int orderByDimID;
    int scatterDimID;
    int xDimID;
    int yDimID;
    int groupByDimID;
    std::string groupValue;

    int timeDimID;
    
    bool isSampling;
    int samplingMode;
    int approximateSize;
    bool isIncreasingOrder;

    int rowNum;
    double start;
    double end;

    bool selectedOnly;

public:
    Query();
    ~Query();

    void Reset();

    void SetQueryMode(int mode) {queryMode = mode;}
    int GetQueryMode();

    void SetRowNum(int num);
    void Resample();
    double GetStart(){return start;}
    double GetEnd(){return end;}

    void SetGroupingMode(int mode) {groupingMode = mode;}
    int GetGroupingMode();

    void SetIncreasingOrder(bool increase) {isIncreasingOrder = increase;}
    bool IsIncreasingOrder() {return isIncreasingOrder;}

    void SetOrderByDimensionID(int id) {orderByDimID = id;}
    void SetXDimensionID(int id) {xDimID = id;}
    void SetYDimensionID(int id) {yDimID = id;}
    void SetScatterDimensionID(int id) {scatterDimID = id;}
    void SetGroupByDimensionID(int id) {groupByDimID = id;}
    void SetGroupValue(std::string groupValue);

    void SetSampling(bool isSampling) {this->isSampling = isSampling;}
    void SetSamplingMode(int samplingMode) {this->samplingMode = samplingMode;}
    void SetApproximateSampleSize(int size);

    int GetOrderByDimensionID() {return orderByDimID;}
    int GetScatterDimensionID() {return scatterDimID;}
    int GetXDimensionID() {return xDimID;}
    int GetYDimensionID() {return yDimID;}
    int GetGroupByDimensionID() {return groupByDimID;}
    std::string GetGroupValue() {return groupValue;}
 
    bool IsSampling() {return isSampling;}
    int GetSamplingMode() {return samplingMode;}
    int GetApproximateSampleSize() {return approximateSize;}

    void SetTimeDimensionID(int id) {timeDimID = id;}
    int GetTimeDimensionID() {return timeDimID;}
    bool IsMaskable();

    void SetSelectedOnly(bool bo) {selectedOnly = bo;}
    bool IsSelectedOnly() {return selectedOnly;}

   
};


#endif
