#include "TimeHistogramIterator.h"
#include "DocRow.h"
#include "../Interface/UserNotifier.h"


TimeHistogramIterator::TimeHistogramIterator(Query *query, IIHash *selectedItemHash, Iterator* rowIter, DimensionSpace* dimSpace, Dimension* timeDim)
{
    this->dimSpace = dimSpace;

//    this->intervalValues = timeDim->GetHistogramIntervalValues();
//    this->intervalNum = timeDim->GetHistogramIntervalNum();
    this->intervalNum = timeDim->GetCardinality();

    docTimeHistogram = NULL;
    histograms = NULL;
    itemHashs = NULL;
    importanceHashs = NULL;
    //the last dimension is the time dimension for building timehistogram
    dimNum = dimSpace->GetSize() - 2;

    if (dimNum == 0)
        return;

    int i, j;

    histograms = new double *[dimNum];
    for (i = 0; i < dimNum; i++)
    {
        histograms[i] = new double [intervalNum];
        for (j = 0; j < intervalNum; j++)
            histograms[i][j] = 0;
    }

    itemHashs = new ISHash * [dimNum];
    importanceHashs = new IIHash * [dimNum];

    for (i = 0; i < dimNum; i++)
    {
        itemHashs[i] = new ISHash;
        importanceHashs[i] = new IIHash;
    }

    DocRow* row;
    int index, itemID;
    int totalNum = 0;
    bool selectedOnly = query->IsSelectedOnly();
    while (!rowIter->IsDone())
    {
        row = (DocRow*)(rowIter->GetNext());
        
        if (!selectedOnly)
        {
            totalNum += row->GetCard(dimNum + 1);
            index = row->GetCard(dimNum); 
            if (index > -1)
            {
                for (i = 0; i < dimNum; i++)
                {
                    if (row->GetCard(i) > 0)
                    {
    //                    histograms[i][index] ++;
                        histograms[i][index] += row->GetCard(dimNum + 1);
                        AddRepresentativeItems(itemHashs[i], importanceHashs[i], index, row->GetCard(dimNum + 1), row->GetItemDescription());
                    }
                }
            }
        }
        else
        {
            itemID = row->GetItemID();
            if (selectedItemHash->find(itemID) != selectedItemHash->end())
            {
                totalNum += row->GetCard(dimNum + 1);
                index = row->GetCard(dimNum); 
                if (index > -1)
                {
                    for (i = 0; i < dimNum; i++)
                    {
                        if (row->GetCard(i) > 0)
                        {
        //                    histograms[i][index] ++;
                            histograms[i][index] += row->GetCard(dimNum + 1);
                            AddRepresentativeItems(itemHashs[i], importanceHashs[i], index, row->GetCard(dimNum + 1), row->GetItemDescription());
                        }
                    }
                }
            }
        }
    }

    int maxNum = histograms[0][0];
    for (i = 0; i < dimNum; i++)
    {
        for (j = 0; j < intervalNum; j++)
        {
            if (maxNum < histograms[i][j])
                maxNum = histograms[i][j];
        }
    }

    count = 0;
    docTimeHistogram = new DocTimeHistogram(dimSpace);
    docTimeHistogram->SetBinNum(intervalNum);
//    docTimeHistogram->SetMaxNum(maxNum);
    docTimeHistogram->SetMaxNum(totalNum);
   
}	

void TimeHistogramIterator::AddRepresentativeItems(ISHash *hash, IIHash *ihash, int index, int importance, std::string destr)
{
    if (hash->find(index) == hash->end())
    {
        (*hash)[index] = destr;
        (*ihash)[index] = importance;
    }
    else
    {
        if (importance > (*ihash)[index])
        {
            (*hash)[index] = destr;
            (*ihash)[index] = importance;
        }
    }
}


TimeHistogramIterator::~TimeHistogramIterator()
{
    if (histograms != NULL)
    {
        for (int i = 0; i < dimNum; i++)
            delete [] histograms[i];
        delete [] histograms;
    }

    if (docTimeHistogram != NULL)
        delete docTimeHistogram;

    if (itemHashs != NULL)
    {
        delete [] itemHashs;
    }

    if (importanceHashs != NULL)
    {
        delete [] importanceHashs;
    }
}


void* TimeHistogramIterator::GetNext()
{
    if (count < dimNum)
    {
        docTimeHistogram->SetHistogram(histograms[count]);
        int dimID = dimSpace->GetDimension(count)->GetDimensionID();
	    docTimeHistogram->SetDimensionID(dimID);
        
        int maxNum = histograms[count][0];
        for (int j = 0; j < intervalNum; j++)
        {
            if (maxNum < histograms[count][j])
                maxNum = histograms[count][j];
        }
        
        docTimeHistogram->SetMaxNum(maxNum);
        docTimeHistogram->SetItemHash(itemHashs[count]);
        docTimeHistogram->SetImportanceHash(importanceHashs[count]);

        count ++;
        return docTimeHistogram;
    }
    else
        return NULL;
}
    

bool TimeHistogramIterator::IsDone()
{
    if (count >= dimNum)
		return true;
	else
		return false;
}

int TimeHistogramIterator::FindIndex(double norValue)
{
    int i = -1;
    if (i < intervalValues[0] || i > intervalValues[intervalNum])
        return i;

    i = 1; 
    while (i <= intervalNum && norValue >= intervalValues[i])
        i ++;

    return (i-1); 
}