#include <FLOAT.h>
#include <sstream>

#include "../../MainApplication.h"
#include "DimstackDrawer.h"
#include "DimstackVisualBin.h"
#include "DimstackVisualBinMeta.h"
#include "DimstackVisualAxesMeta.h"
#include "../../Interface/ViewImageWindow.h"


DimstackDrawer::DimstackDrawer() : TableDrawer()
{
	settings = new DimstackDrawerSettings();
    AddLayer(BIN);
    AddLayer(AXES);
}


DimstackDrawer::~DimstackDrawer()
{
    delete settings;
}

void DimstackDrawer::PrepareCanvas()
{
    canvas->SetCurrent();
//    canvas->SetLowerLeftOrigin();
    canvas->SetTopLeftOrigin();
    canvas->SetViewport();

//    canvas->SetBackground(colorManager->GetColor(BG));
    canvas->SetBackground(RGBt(0, 0, 0, 1));
    canvas->Clear();

    glEnable(GL_DEPTH_TEST);
    glClear(GL_DEPTH_BUFFER_BIT);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);


}

//before drawer draws a canvas
void DimstackDrawer::PrepareDrawLayer(Layer* layer)
{
    TableDrawer::PrepareDrawLayer(layer);
    canvas->SetCurrent();
    if (layer->IsTranslucent())
    {
        // Turn on Translucency
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    }
    else
    {
        //Turn off Translucency
        glDisable(GL_BLEND);
    }
}

//after drawer draws a canvas
void DimstackDrawer::AfterDrawLayer(Layer* layer)
{
    if (layer->IsTranslucent())
    {
        //Turn off Translucency
  //      glDisable(GL_BLEND);
    }
}

void DimstackDrawer::DrawBin(VisualBin *Bin)
{
    if (Bin->IsEmpty())
        return;
    DimstackVisualBin* visBin= (DimstackVisualBin *)Bin;
    int DimstackID = visBin->GetBinID();
    if (settings->IsShowSelectedOnly())
    {
        if (settings->IsTextureSelected(DimstackID) == false)
            return;
    }
	float xPos = visBin->GetXPosition();
	float yPos = visBin->GetYPosition();
	float zPos = visBin->GetZPosition();
	float xSize = visBin->GetXSize();
	float ySize = visBin->GetYSize();

	glColor4f(1,1,1,1);
	glEnable( GL_TEXTURE_2D );
	
    glBindTexture( GL_TEXTURE_2D, visBin->GetTexture());

 	glBegin(GL_QUADS);
	glTexCoord2d(0.0,0.0); glVertex3d(xPos-xSize,yPos-ySize, zPos);
	glTexCoord2d(1.0,0.0); glVertex3d(xPos+xSize,yPos-ySize, zPos);
	glTexCoord2d(1.0,1.0); glVertex3d(xPos+xSize,yPos+ySize, zPos);
	glTexCoord2d(0.0,1.0); glVertex3d(xPos-xSize,yPos+ySize, zPos);
	glEnd();

	glDisable( GL_TEXTURE_2D );

    if (settings->IsSelectedTextureHighlighted())
    {
        if (DimstackID == settings->GetFocusTextureID())
        {
            glColor4f(1.f, 0.f, 0.f, 1.f);
	        glLineWidth(1.5);
            glBegin(GL_LINE_STRIP);
            float pad = 0.00001f;
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glEnd();
        }
        else if (DimstackID == settings->GetBaseTextureID())
        {
            glColor4f(1.f, 0.f, 1.f, 1.f);
	        glLineWidth(1.5);
            glBegin(GL_LINE_STRIP);
            double pad = 0.00001f;
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glEnd();
        }
        else if (settings->IsTextureSelected(DimstackID))
        {
            glColor4f(0.f, 1.f, 0.f, 1.f);
	        glLineWidth(1.5);
            glBegin(GL_LINE_STRIP);
            float pad = 0.00001f;
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glEnd();
        }
    }
}


void DimstackDrawer::DrawAxes(VisualAxes *visAxes)
{
    DimstackVisualAxes* axes = (DimstackVisualAxes *)visAxes;
    
    int i, j, axesNum;
    bool vertical;

    int dimNum = axes->GetDimensionNumber();

    int deep = 3;

    if (dimNum > deep * 2)
        dimNum = deep*2;
//    for (i = dimNum - 1; i >= 0; i--)
    for (i = 0; i < dimNum; i++)
    {
        RGBt color = colorManager->GetColor(GRID, i);
        glColor4f(color.R, color.G, color.B, color.A);

        glLineWidth(axes->GetDimensionAxesWidth(i));

        float *pos;
        axesNum = axes->GetAxes(i, pos, vertical);
       
        glLineWidth((dimNum - i)*2);
        if (vertical == true)
        {
            glBegin(GL_LINES);
            for (j = 0; j < axesNum; j++)
            {
                glVertex2f(pos[j], 0);
                glVertex2f(pos[j], 1);
            }
    	    glEnd();
        }
        else
        {
            glBegin(GL_LINES);
            for (j = 0; j < axesNum; j++)
            {
                glVertex2f(0, pos[j]);
                glVertex2f(1, pos[j]);
            }
    	    glEnd();
        }
        glLineWidth(1);
        delete [] pos;
    }
}


int DimstackDrawer::FindBinID(float x, float y)
{
    return settings->GetTextureID(x, y);
}

void DimstackDrawer::DoubleClick(grCanvas* canvas, int button, Vec2 p)
{
/*    Drawer::DoubleClick(canvas, button, p);
    int binID = FindBinID(p.X, p.Y);
    if (binID >= 0)
    {
        if (button == 0)
        {
            wxString imgFile = wxString::Format("C:\\yangjing\\Fan\\Querydb\\%d.jpg", binID);
            ViewImageWindow *win = new ViewImageWindow(imgFile);
            win->Show(TRUE);
        }
    }*/
}

void DimstackDrawer::ButtonDown(grCanvas* canvas, int button, Vec2 p)
{
/*    Drawer::ButtonDown(canvas, button, p);
    inputMode = Dimstack_NONE;
    int binID = FindBinID(p.X, p.Y);
    if (binID >= 0)
    {
        if (button == 0)
        {
            if (canvas->IsShiftKey())
            {
                inputMode = Dimstack_MOVE;
                canvas->SetHandCursor();
                currentBin = binID;
                settings->BringTextureToFront(binID);
            }
            else
            {
                settings->BringTextureToFront(binID);
                Refresh();
            }
        }
    }*/
}


void DimstackDrawer::ButtonUp(grCanvas* canvas, int button, Vec2 p)
{
/*    Drawer::ButtonUp(canvas, button, p);
    if (inputMode == Dimstack_MOVE)
    {
        canvas->SetDefaultCursor();
        inputMode = Dimstack_NONE;
    }
    else
    {
        if (button == 2)
        {
            int binID = FindBinID(p.X, p.Y);
            if (binID >= 0)
            {
                if (!settings->IsTextureDistorted(binID))
                    settings->AddDistortedTexture(binID);
                else
                    settings->RemoveDistortedTexture(binID);

                settings->BringTextureToFront(binID);
            }
            Refresh();
        }
        else if (button == 1)
        {
            int binID = FindBinID(p.X, p.Y);
            if (binID >= 0)
            {
                std::ostringstream oss;
                oss << binID << ".jpg";
                
                MainApplication *app = &wxGetApp();
                app->GetMDSControlFrame()->SetBaseItem(oss.str());
            }
        }
    }*/
}

void DimstackDrawer::MouseMotion(grCanvas* canvas, Vec2 p)
{
/*    Drawer::MouseMotion(canvas, p);
    if (inputMode == Dimstack_MOVE)
    {
        settings->SetTextureXYPosition(currentBin, p.X, p.Y);
        canvas->Refresh();      
    }
    else
    {
        int binID = FindBinID(p.X, p.Y);
        if (binID >= 0)
        {
            std::ostringstream oss;
            oss << binID << ".jpg";
            canvas->WriteMessage(oss.str().c_str(), 0);
        }
    }*/
}


VisualElementMeta* DimstackDrawer::CreateVisualBinMeta(DocumentManager* docManager, bool brushed)
{
    DimstackVisualBinMeta *meta = new DimstackVisualBinMeta(this, docManager, brushed);
    return meta;
}

VisualElementMeta* DimstackDrawer::CreateVisualAxesMeta()
{
    DimstackVisualAxesMeta *meta = new DimstackVisualAxesMeta(this);
    return meta;
}

void DimstackDrawer::CurrentDimensionSpaceChanged() 
{
    LinkListIterator* iter;

    iter = new LinkListIterator(&canvasList);
    grCanvas* canv;
    while (! iter->IsDone())
    {
        canv = (grCanvas *) iter->GetNext();
        canv->DeleteTextures(DIMSTACK_HASH);
        canv->Refresh();
    }

 
    delete iter;
}

void DimstackDrawer::AllDimensionSpaceDeleted()
{
    LinkListIterator* iter;

    iter = new LinkListIterator(&canvasList);
    grCanvas* canv;
    while (! iter->IsDone())
    {
        canv = (grCanvas *) iter->GetNext();
        canv->DeleteTextures(DIMSTACK_HASH);
    }

    delete iter;
}
