#include <cmath>
#include <wx/image.h>
#include "GlyphVisualRowMeta.h"
#include "GlyphVisualRow.h"
#include "../../DataStructure/DocRow.h"
#include "../../MainApplication.h"

GlyphVisualRowMeta::GlyphVisualRowMeta(GlyphDrawer* drawer, 
                                       DocumentManager* docManager, bool brushed, bool hier)
{
    this->drawer = drawer;

    axesLayer = new Layer(AXES);
    axesMeta = axesLayer->CreateVisualElementMeta(drawer, docManager);
    axesIter = docManager->GetElementsIterator(AXES, axesMeta);
    visAxesIter  = new VisualElementsIterator(axesMeta, axesIter); 
    
    axes = (GlyphVisualAxes *)(visAxesIter->GetNext());

    canvas = drawer->GetCanvas();
    count = 0;
    wxInitAllImageHandlers();
    colorData = NULL;
    plainData = NULL;

    focusItem = docManager->GetFocusItemID();
    this->globalPath = docManager->GetGlobalPath();

    settings = (GlyphDrawerSettings *)(drawer->GetSettings());
    settings->ClearDisplayedTextureHash();

    settings->ChangeGlyphSize(0.001);

}

GlyphVisualRowMeta::~GlyphVisualRowMeta()
{
    delete axesLayer;
    delete visAxesIter;
    delete axesMeta;
    delete axesIter; 
    if (colorData != NULL)
        delete [] colorData;
    if (plainData != NULL)
        delete [] plainData;
}

VisualElement* GlyphVisualRowMeta::InstantVisualElement()
{
    GlyphVisualRow* visRow = new GlyphVisualRow();
    
    return visRow;
}

void GlyphVisualRowMeta::Convert(VisualElement* visualEle, DocElement* docEle)
{
    GlyphVisualRow* visualRow = (GlyphVisualRow *)visualEle;
    DocRow* docRow = (DocRow *)docEle;

    GLuint texture;
    int width;
    int height;

    int glyphID = docRow->GetItemID();
    settings->AddDisplayedTexture(glyphID);
    float xpos, ypos, zpos, xsize, ysize;

    visualRow->SetTexture(-1);
    axes->GetItemXYSize(glyphID, xsize, ysize);

    axes->GetItemXYPos(glyphID, xpos, ypos);
    zpos = axes->GetItemZPos(glyphID);
    visualRow->SetColor(RGBt(0.2, 0.8, 0.3, 1));
    visualRow->SetXPosition(xpos);
    visualRow->SetYPosition(ypos);
    visualRow->SetZPosition(zpos);
    visualRow->SetXSize(xsize);
    visualRow->SetYSize(ysize);
    visualRow->SetItemID(glyphID);

    count ++;
}

void GlyphVisualRowMeta::Convert1(VisualElement* visualEle, DocElement* docEle)
{
    GlyphVisualRow* visualRow = (GlyphVisualRow *)visualEle;
    DocRow* docRow = (DocRow *)docEle;

    GLuint texture;
    int width;
    int height;

    int glyphID = docRow->GetItemID();
    settings->AddDisplayedTexture(glyphID);
 //   w = 128;
 //   h = 128;
    float xpos, ypos, zpos, xsize, ysize;

    if (!canvas->GetTexture(GLYPH_HASH, glyphID, texture, width, height))
    {
        w = 64;
        h = 64;
        if (colorData == NULL)
            colorData = new float[w*h*4];
        if (GetBitMap(glyphID, docRow->GetItemDescription(), w, h, NULL, colorData))
        {
            canvas->CreateTexture(GLYPH_HASH, glyphID, w, h, colorData, texture);
            width = w;
            height = h;
        }
    }
 /*       else
   //     {
            visualRow->SetTexture(-1);
            xsize = 100;
            visualRow->SetColor(RGBt(0.5, 1, 0.5, 1));
/*            w = 4;
            h = 4;
            if (!canvas->GetTexture(GLYPH_HASH, -1, texture, width, height))
            {
                if (plainData == NULL)
                plainData = new float [w * h * 4];
                RGBt color = RGBt(0.5, 1, 1, 1);
                GetPlainBitMap(color, 4, 4, plainData);
                canvas->CreateTexture(GLYPH_HASH, -1, w, h, plainData, texture);
                width = 4;
                height = 4;

            }*/
    //    }
  //  }

    visualRow->SetTexture(texture);
    visualRow->SetWidth(width);        //width: the width of the original texture object
    visualRow->SetHeight(height);      //height: the height of the origianl texture object
    axes->GetItemXYSize(glyphID, xsize, ysize);

    axes->GetItemXYPos(glyphID, xpos, ypos);
    zpos = axes->GetItemZPos(glyphID);
    visualRow->SetXPosition(xpos);
    visualRow->SetYPosition(ypos);
 //
 //   visualRow->SetZPosition(-1./glyphID);
    visualRow->SetZPosition(zpos);
   // visualRow->SetZPosition(-1.5);
    visualRow->SetXSize(xsize);
    visualRow->SetYSize(ysize);
    visualRow->SetItemID(glyphID);

    count ++;
}

void GlyphVisualRowMeta:: GetPlainBitMap(RGBt color, int w, int h, float* cdata)
{
    for (int i = 0; i < w*h; i++)
    {
        cdata[i*4+0] = 1 - color.R;
        cdata[i*4+1] = 1 - color.G;
        cdata[i*4+2] = 1 - color.B;
        cdata[i*4+3] = color.A;
    }
}

/*void GlyphVisualRowMeta::GetBitMap(int glyphID, int &w, int &h, float* data, float*& colorData)
{
//    wxString imgFile = wxString::Format("C:\\yangjing\\Fan\\Querydb\\%d.jpg", glyphID);
    wxString imgFile = wxString::Format("C:\\yangjing\\Fan\\Querydb\\%d.jpg", 0);
    wxImage image(imgFile);
    int w1 = image.GetWidth();
    int h1 = image.GetHeight();
    unsigned char* imgData = image.GetData();
    int A, B;
    if (w * h1 < h * w1)
    {
        A = w1;
        B = w;
    }
    else
    {
        A = h1;
        B = h;
    }

    int pos, pos1;
    int i, j, i1, j1;
    for (i = 0; i < h; i++)
    {
        i1 = i * A / B;
        if (i1 >= h1)
        {
            for (j = 0; j < w; j++)
            {
                pos = i * w + j;
		        colorData[pos*4+0] = 0.f;
		        colorData[pos*4+1] = 0.f;
		        colorData[pos*4+2] = 0.f;
		        colorData[pos*4+3] = 0.01f;
            }
        }
        else
        {
            for (j = 0; j < w; j++)
            {
                pos = i * w + j;
                j1 = j * A / B;
                if (j1 >= w1)
                {
		            colorData[pos*4+0] = 0.f;
		            colorData[pos*4+1] = 0.f;
		            colorData[pos*4+2] = 0.f;
		            colorData[pos*4+3] = 0.01f;
                }
                else
                {
                    pos1 = i1 * w1 + j1;
		            colorData[pos*4+0] = 1-imgData[pos1*3] / 255.f;
		            colorData[pos*4+1] = 1-imgData[pos1*3+1] / 255.f;
		            colorData[pos*4+2] = 1-imgData[pos1*3+2] / 255.f;
		            colorData[pos*4+3] = 1.f;
                }
            }
        }
    }
}*/

bool GlyphVisualRowMeta::GetBitMap(int glyphID, std::string imageName, int &w, int &h, float* data, float*& colorData)
{
    wxString imgFile = wxString::Format("%s\\%s", globalPath.c_str(), imageName.c_str());
//    wxString imgFile = wxString::Format("C:\\yangjing\\Fan\\Querydb\\%s", imageName.c_str());
//    wxString imgFile = wxString::Format("C:\\yangjing\\Fan\\Querydb\\%d.jpg", glyphID);
//    wxString imgFile = wxString::Format("C:\\yangjing\\Fan\\Querydb\\%d.jpg", 0);
    if (!wxImage::CanRead(imgFile))
        return false;

    wxImage image(imgFile);
    int w1 = image.GetWidth();
    int h1 = image.GetHeight();
    unsigned char* imgData = image.GetData();
    int A, B;
    if (w * h1 < h * w1)
    {
        A = w1;
        B = w;
        h = w * h1 / w1;
    }
    else
    {
        A = h1;
        B = h;
        w = h * w1 / h1;
    }

    int pos, pos1;
    int i, j, i1, j1;
    for (i = 0; i < h; i++)
    {
        i1 = i * A / B;
        if (i1 >= h1)
        {
            for (j = 0; j < w; j++)
            {
                pos = i * w + j;
		        colorData[pos*4+0] = 0.f;
		        colorData[pos*4+1] = 0.f;
		        colorData[pos*4+2] = 0.f;
		        colorData[pos*4+3] = 0.01f;
            }
        }
        else
        {
            for (j = 0; j < w; j++)
            {
                pos = i * w + j;
                j1 = j * A / B;
                if (j1 >= w1)
                {
		            colorData[pos*4+0] = 0.f;
		            colorData[pos*4+1] = 0.f;
		            colorData[pos*4+2] = 0.f;
		            colorData[pos*4+3] = 0.01f;
                }
                else
                {
                    pos1 = i1 * w1 + j1;
		            colorData[pos*4+0] = 1-imgData[pos1*3] / 255.f;
		            colorData[pos*4+1] = 1-imgData[pos1*3+1] / 255.f;
		            colorData[pos*4+2] = 1-imgData[pos1*3+2] / 255.f;
		            colorData[pos*4+3] = 1.f;
                }
            }
        }
    }

    if (glyphID != focusItem)
    {
        for (i = 0; i < h; i++)
        {
            pos = i * w;
		    colorData[pos*4+0] = 1.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = i * w + w - 1;
		    colorData[pos*4+0] = 1.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
        }
        for (j = 0; j < w; j++)
        {
            pos = j;
		    colorData[pos*4+0] = 1.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = (h - 1) * w + j;
		    colorData[pos*4+0] = 1.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
        }
    }
    else
    {
        for (i = 0; i < h; i++)
        {
            pos = i * w;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = i * w + 1;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = i * w + w - 2;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = i * w + w - 1;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
        }
        for (j = 0; j < w; j++)
        {
            pos = j;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = w + j;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = (h - 2) * w + j;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
            pos = (h - 1) * w + j;
		    colorData[pos*4+0] = 0.f;
		    colorData[pos*4+1] = 1.f;
		    colorData[pos*4+2] = 1.f;
		    colorData[pos*4+3] = 1.f;
        }
    }
    return true;
}

