#include "ParcoordVisualLabelMeta.h"
#include "../VisualLabel.h"
#include "../../DataStructure/DocLabel.h"
#include "../../MainApplication.h"

ParcoordVisualLabelMeta::ParcoordVisualLabelMeta(ParcoordDrawer* drawer, DocumentManager* docManager, float angle, float scale, float shiftX, float shiftY, float minShiftX, float minShiftY)
{
    this->drawer = drawer;

    axesLayer = new Layer(AXES);
    axesMeta = axesLayer->CreateVisualElementMeta(drawer, docManager);
    axesIter = docManager->GetElementsIterator(AXES, axesMeta);
    visAxesIter  = new VisualElementsIterator(axesMeta, axesIter); 
    
    axes = (ParcoordVisualAxes *)(visAxesIter->GetNext());

    this->angle = angle;
    this->scale = scale;
    this->shiftX = shiftX;
    this->shiftY = shiftY;
    this->minShiftX = minShiftX;
    this->minShiftY = minShiftY;

}

ParcoordVisualLabelMeta::~ParcoordVisualLabelMeta()
{
    delete axesLayer;
    delete visAxesIter;
    delete axesMeta;
    delete axesIter; 
}

VisualElement* ParcoordVisualLabelMeta::InstantVisualElement()
{
    VisualLabel* visLabel= new VisualLabel();
    
    visLabel->SetAngle(angle);
    visLabel->SetScale(scale);

    return visLabel;
}

void ParcoordVisualLabelMeta::Convert(VisualElement* visualEle, DocElement* docEle)
{
    VisualLabel* visualLabel = (VisualLabel *)visualEle;
    DocLabel* docLabel= (DocLabel *)docEle;

    visualLabel->SetText(docLabel->GetText());
    
    if (docLabel->IsMin())
    {
        visualLabel->SetX(axes->GetDimensionPositionByDimensionID(docLabel->GetDimensionID()) + minShiftX);
        visualLabel->SetY(docLabel->GetNorValue() + minShiftY);
    }
    else
    {
        visualLabel->SetX(axes->GetDimensionPositionByDimensionID(docLabel->GetDimensionID()) + shiftX);
        visualLabel->SetY(docLabel->GetNorValue() + shiftY);
    }
    visualLabel->SetAngle(45);
}