#include <stdio.h>
#include "ParcoordVisualRow.h"

ParcoordVisualRow::ParcoordVisualRow() : VisualRow()
{
    segNum = 0;
    vNum = 0;
    
    vPositions = NULL;
    segPositions = NULL;

    colors = NULL;
    showBand = false;
}

ParcoordVisualRow::~ParcoordVisualRow()
{
    if (vPositions != NULL)
    {
        for (int i = 0; i < vNum; i++)
        {
            delete [] vPositions[i];
        }
        delete [] vPositions;
    }

    if (segPositions != NULL)
        delete [] segPositions;

    if (colors != NULL)
        delete [] colors;

}

void ParcoordVisualRow::Init()
{
    vPositions = new float * [vNum];
    for (int i = 0; i < vNum; i++)
    {
        vPositions[i] = new float [segNum];
    }
    colors = new RGBt [vNum];
}

bool ParcoordVisualRow::SetSegNum(int num)
{
    if (num <= 0)
        return false;

    segNum = num;
    segPositions = new float [segNum];

    if (vNum != 0 && vPositions == NULL)
        Init();

    return true;
}

bool ParcoordVisualRow::SetVNum(int num)
{
    if (num != 1 && num != 3)
        return false;

    vNum = num;

    if (segNum != 0 && vPositions == NULL)
        Init();

    return true;
}

void ParcoordVisualRow::SetColor(int vID, RGBt color)
{
    if (vID >= vNum)
        return;

    colors[vID] = color;
}

void ParcoordVisualRow::SetSegPosition(int segID, float pos)
{
    if (segID >= segNum)
        return;

    segPositions[segID] = pos;
}

void ParcoordVisualRow::SetVPosition(int vID, int segID, float pos)
{
    if (vID >= vNum || segID >= segNum)
        return;

    vPositions[vID][segID] = pos;
}

float ParcoordVisualRow::GetSegPosition(int segID)
{
    if (segID >= segNum)
        return 0;

    return segPositions[segID];
}

float ParcoordVisualRow::GetVPosition(int vID, int segID)
{
    if (vID >= vNum || segID >= segNum)
        return 0;
    if (segID >= segNum)
        return 0;

    return vPositions[vID][segID];
}

RGBt ParcoordVisualRow::GetVColor(int vID)
{
    if (vID >= vNum)
        return RGBt(0xffffff);

    return colors[vID];
}

int ParcoordVisualRow::GetSegNum()
{
    return segNum;
}

int ParcoordVisualRow::GetVNum()
{
    return vNum;
}