#include "ParcoordVisualRowMeta.h"
#include "../VisualRow.h"
#include "../../DataStructure/DocRow.h"
#include "../../MainApplication.h"

ParcoordVisualRowMeta::ParcoordVisualRowMeta(ParcoordDrawer* drawer, 
                          DocumentManager* docManager, bool brushed, bool hier)
{
    this->drawer = drawer;
    this->brushed = brushed;

    axesLayer = new Layer(AXES);
    axesMeta = axesLayer->CreateVisualElementMeta(drawer, docManager);
    axesIter = docManager->GetElementsIterator(AXES, axesMeta);
    visAxesIter  = new VisualElementsIterator(axesMeta, axesIter); 
    
    axes = (ParcoordVisualAxes *)(visAxesIter->GetNext());

    if (axes->GetAxNum() > 1)
        showDimCluster = true;
    else
        showDimCluster = false;
    
    if (hier && drawer->IsShowBand())
    {
        showBand= true;
        bandColorSetting = (drawer->GetColorManager())->GetColorSetting(BAND);
    }
    else
        showBand = false;

    if (hier == false)
    {
        if (brushed == false)
            colorSetting = (drawer->GetColorManager())->GetColorSetting(FLAT_UNBRUSHED_DATA);
        else
            colorSetting = (drawer->GetColorManager())->GetColorSetting(FLAT_BRUSHED_DATA);
    }
    else //hier display
    {
        if (brushed == false)
            colorSetting = (drawer->GetColorManager())->GetColorSetting(HIER_UNBRUSHED_DATA);
        else
            colorSetting = (drawer->GetColorManager())->GetColorSetting(HIER_BRUSHED_DATA);
    }

    omitColor = colorSetting->IsSingleColor() && (!showBand);
}

ParcoordVisualRowMeta::~ParcoordVisualRowMeta()
{
    delete axesLayer;
    delete visAxesIter;
    delete axesMeta;
    delete axesIter; 
}

VisualElement* ParcoordVisualRowMeta::InstantVisualElement()
{
    ParcoordVisualRow* visData = new ParcoordVisualRow();
    
    int segNum = axes->GetSegNum();

    visData->SetSegNum(segNum);

    if (showBand)
        visData->SetVNum(3);
    else 
        visData->SetVNum(1);

    for(int i = 0; i < segNum; i++)
    {
        visData->SetSegPosition(i, axes->GetSegPosition(i));
    }
    
    if (omitColor)
        visData->SetColor(0, colorSetting->GetColor());

    return visData;
}

void ParcoordVisualRowMeta::Convert(VisualElement* visualEle, DocElement* docEle)
{
    ParcoordVisualRow* visualData = (ParcoordVisualRow *)visualEle;
    DocRow* docData = (DocRow *)docEle;

    if (showBand && !docData->IsOriginalDataItem())
        visualData->SetShowBand(true);
    else
        visualData->SetShowBand(false);

    int segNum = axes->GetSegNum();
    int type, dimID;
    for (int i = 0; i < segNum; i++)
    {
        axes->GetSegInformation(i, dimID, type);
        //type = 0, nor axis, type = 1, min axis (dim min), type = 2, max axis (dim max)
        visualData->SetVPosition(0, dimID, docData->GetNorValue(0, type, dimID));
        if (visualData->IsShowBand())
        {
            //here 0, 1, 2, indicates data cluster mean, min, and max;
            visualData->SetVPosition(1, dimID, docData->GetNorValue(1, type, dimID));
            visualData->SetVPosition(2, dimID, docData->GetNorValue(2, type, dimID));
        }
    }

    if (!omitColor)
    {
        float value = docData->GetNorValueForColor();
        visualData->SetColor(0, colorSetting->GetColor(value));
    }

    if (visualData->IsShowBand())
    {
        float value = docData->GetNorValueForColor();
        if (brushed)
        {
            RGBt color = bandColorSetting->GetColor(value);
            visualData->SetColor(1, color);
            visualData->SetColor(2, color);
        }
        else
        {
            visualData->SetColor(1, visualData->GetVColor(0));
            visualData->SetColor(2, visualData->GetVColor(0));
        }
    }
    
}


