#include "TableDrawer.h"
#include "VisualElementsIterator.h"
#include "../MainApplication.h"
#include "../Interface/UserNotifier.h"

TableDrawer::TableDrawer()
{
    showBand = true;
    showMean = true;
}

TableDrawer::~TableDrawer()
{
}

//drawer draws a canvas
void TableDrawer::RefreshCanvas(grCanvas* canvas)
{
    Drawer::RefreshCanvas(canvas);

	//get the canvas ready
	PrepareCanvas();

    if (docManager->IsEmpty())
        return; 

    LinkListIterator *iter = IterateLayers();
    Layer *layer;
	//draw layers
	while (!iter->IsDone())
	{
		//draw a layer
		//get a layer
		layer = (Layer *)(iter->GetNext());
		//check if a layer needs to be draw
        if (layer->GetDisplayMode() == Layer::DRAW)
		{
			//set common setting for drawing in drawer
			PrepareDrawLayer(layer);

            VisualElementMeta *meta = layer->CreateVisualElementMeta(this, docManager);
            Iterator *eleIter = docManager->GetElementsIterator(layer->GetLayerType(), meta);
            VisualElementsIterator *visEleIter = new VisualElementsIterator(meta, eleIter); 
			VisualElement* ele;
			//draw all visual elements in a layer
			while (!visEleIter->IsDone())
			{
				//draw a single visual elements
				ele = (VisualElement *)(visEleIter->GetNext());
                ele->Draw(this);
			}
			delete visEleIter;
            delete eleIter;
            delete meta;
			//cleanup drawer
			AfterDrawLayer(layer);
        }
 	} 
	delete iter;
}

void TableDrawer::DrawLabel(VisualLabel *label)
{
//    glColor4f(1.0, 1.0, 1.0, 1);
    glColor4f(1.0, 1.0, .0, 1);

//    canvas->DrawString(label->GetText(), label->GetX(), label->GetY(), label->GetZ(), 
//        label->GetAngle(), label->GetScale());
    canvas->DrawString(label->GetText(), label->GetX(), label->GetY(), 
        label->GetAngle(), label->GetScale());
}

void TableDrawer::PrepareCanvas()
{
    canvas->SetCurrent();
    canvas->SetTopLeftOrigin();
    canvas->SetViewport();

    canvas->SetBackground(colorManager->GetColor(BG));
    canvas->Clear();
}

void TableDrawer::PrepareDrawLayer(Layer* layer)
{
}


void TableDrawer::AfterDrawLayer(Layer* layer)
{
}

void TableDrawer::DrawData(VisualRow* data)
{
}

void TableDrawer::DrawColumn(VisualColumn* column)
{
}

void TableDrawer::DrawTimeHistogram(VisualTimeHistogram* timeHistogram)
{
}

void TableDrawer::DrawBin(VisualBin* bin)
{
}

void TableDrawer::DrawAxes(VisualAxes* axes)
{
}

void TableDrawer::DrawRuler(VisualRuler* ruler)
{
}

VisualElementMeta* TableDrawer::CreateVisualRowMeta(DocumentManager* docManager, bool brushed, bool hier)
{
    return NULL;
}

VisualElementMeta* TableDrawer::CreateVisualBinMeta(DocumentManager* docManager, bool brushed)
{
    return NULL;
}

VisualElementMeta* TableDrawer::CreateVisualColumnMeta(DocumentManager* docManager, bool brushed, bool hier)
{
    return NULL;
}

VisualElementMeta* TableDrawer::CreateVisualTimeHistogramMeta(DocumentManager* docManager, bool brushed, bool hier)
{
    return NULL;
}

VisualElementMeta* TableDrawer::CreateVisualMaskMeta(DocumentManager* docManager, bool brushed, bool hier)
{
    return NULL;
}

VisualElementMeta* TableDrawer::CreateVisualAxesMeta()
{
    return NULL;
}

VisualElementMeta* TableDrawer::CreateVisualDimensionNameMeta(DocumentManager* docManager)
{
    return NULL;
}

VisualElementMeta* TableDrawer::CreateVisualDimensionLabelMeta(DocumentManager* docManager)
{
    return NULL;
}

VisualElementMeta* TableDrawer::CreateVisualRulerMeta(DocumentManager* docManager)
{
    return NULL;
}

bool TableDrawer::SetShowBand(bool showBand)
{
    if (this->showBand == showBand)
        return false;

    else 
    {
        this->showBand = showBand;
        return true;
    }
}

bool TableDrawer::SetShowMean(bool showMean)
{
    if (this->showMean == showMean)
        return false;

    else 
    {
        this->showMean = showMean;
        return true;
    }
}

bool TableDrawer::IsNeedDimensionClusterMinMaxInfo()
{
    return false;
}

bool TableDrawer::IsNeedDataClusterMinMaxInfo()
{
    if (showBand)
        return true;
    else
        return false;
}

