#include <ctime>
#include "TextureDrawerSettings.h"
#include "../Basic/macros.h"
#include "../Basic/Vec2.h"

TextureDrawerSettings::TextureDrawerSettings()
{
    Reset();
    zPosStyle = ZPOS_RANDOM;

    normalXSize = 0.04f;
    normalYSize = 0.04f;

    normalZPos= -1.0f;
    maxZ = -0.5f;
    minZ = -1.5f;


    distortion = true;
	distortFac = 2;

//    srand( (unsigned)time( NULL ));

    focusTextureID = -1;
    baseTextureID = -1;

    showSelectedOnly = false;
    displayMode = 0;
    selectedTextureHash = NULL;
}

int TextureDrawerSettings::GetTextureNum()
{
    if (showSelectedOnly && selectedTextureHash != NULL)
        return selectedTextureHash->size();
    else
        return centerXYHash.size();
}

void TextureDrawerSettings::GetTextureIDs(int* ids)
{
    if (showSelectedOnly && selectedTextureHash != NULL)
    {
        IIHash::iterator it;
        it = selectedTextureHash->begin();
        int i = 0;
        while (it != selectedTextureHash->end())
        {
            ids[i] = it->first;
            it ++;
            i ++;
        }
    }
    else
    {
        IPHash::iterator it;
        it = centerXYHash.begin();
        int i = 0;
        while (it != centerXYHash.end())
        {
            ids[i] = it->first;
            it ++;
            i ++;
        }
    }
}

void TextureDrawerSettings::Reset()
{
    SetXYFromDataSet(true);

    zHash.clear();

    distortedTextureHash.clear();

}

void TextureDrawerSettings::TextureSpaceChanged()
{
    SetXYFromDataSet(true);
}

TextureDrawerSettings::~TextureDrawerSettings()
{
    if (centerXYHash.size() >= 0)
        ClearCenterXYHash();
}

void TextureDrawerSettings::ClearCenterXYHash()
{
    IPHash::iterator it;
    it = centerXYHash.begin();
    while (it != centerXYHash.end())
    {
        delete (Vec2 *)(it->second);
        it ++;
    }
    centerXYHash.clear();
}

void TextureDrawerSettings::ResetDistortion()
{
	distortedTextureHash.clear();
}

void TextureDrawerSettings::ResetZPos()
{
	zHash.clear();
}

void TextureDrawerSettings::SetDistortion(bool distort)
{
	distortion = distort;
}

bool TextureDrawerSettings::IsDistortionEnabled()
{
	return distortion;
}

void TextureDrawerSettings::AddDistortedTexture(int itemID)
{
	distortedTextureHash[itemID] = 1;

}

void TextureDrawerSettings::RemoveDistortedTexture(int itemID)
{
    distortedTextureHash.erase(itemID);
}

bool TextureDrawerSettings::IsTextureDistorted(int itemID)
{
	if (distortion == false)
		return false;

	if (distortedTextureHash.find(itemID) == distortedTextureHash.end())
		return false;
	else
		return true;
}

bool TextureDrawerSettings::IsTextureSelected(int itemID)
{
	if (selectedTextureHash->find(itemID) == selectedTextureHash->end())
		return false;
	else
		return true;
}

void TextureDrawerSettings::GetTextureXYPosition(int itemID, float &centerx, float &centery)
{
    Vec2 *p;
    p = (Vec2 *)(centerXYHash[itemID]);
    
    if (p != NULL)
    {
        centerx = (*p).X;
        centery = (*p).Y;
    }
    else
    {
    	centerx = (double)rand()/RAND_MAX;
    	centery = (double)rand()/RAND_MAX;
        centerXYHash[itemID] = new Vec2(centerx, centery);
    }

    float xSize, ySize;
    GetTextureXYSize(itemID, xSize, ySize);
    if (centerx-xSize< 0)
        centerx = xSize;
    if (centerx+xSize> 1)
        centerx = 1-xSize;
    if (centery-ySize < 0)
        centery = ySize;
    if (centery+ySize > 1)
        centery = 1-ySize;
}

void TextureDrawerSettings::GetTextureXYPosition(int itemID, float &centerx, float &centery, float &xSize, float &ySize)
{
    Vec2 *p;
    p = (Vec2 *)(centerXYHash[itemID]);
    
    if (p != NULL)
    {
        centerx = (*p).X;
        centery = (*p).Y;
    }
    else
    {
    	centerx = (double)rand()/RAND_MAX;
    	centery = (double)rand()/RAND_MAX;
        centerXYHash[itemID] = new Vec2(centerx, centery);
    }

    GetTextureXYSize(itemID, xSize, ySize);
    if (centerx-xSize< 0)
        centerx = xSize;
    else if (centerx+xSize> 1)
        centerx = 1-xSize;
    if (centery-ySize < 0)
        centery = ySize;
    else if (centery+ySize > 1)
        centery = 1-ySize;
}


int TextureDrawerSettings::GetTextureID(float x, float y)
{
    float xpos, ypos, zpos, xsize, ysize;

    IIHash::iterator it1;
    IPHash::iterator it;
    int id;
    float min_zpos = -100;
    int n = -1;
    Vec2 *p;

    for( it1 = displayedTextureHash.begin(); it1 != displayedTextureHash.end(); ++it1 )
    {
        id = it1->first;
        it = centerXYHash.find(id);
        p = (Vec2 *)(it->second);
        xpos = (*p).X;
        ypos = (*p).Y;
        zpos = GetTextureZPosition(id);
        GetTextureXYSize(id, xsize, ysize);

        if (xpos-xsize< 0)
            xpos = xsize;
        else if (xpos+xsize> 1)
            xpos = 1-xsize;
        if (ypos-ysize < 0)
            ypos = ysize;
        else if (ypos+ysize > 1)
            ypos = 1-ysize;

        if (fabs(x - xpos) < xsize && fabs(y - ypos) < ysize)
        {
            if (zpos > min_zpos)
            {
                min_zpos = zpos;
                n = id;
            }
        }
    }
    return n;
}

bool TextureDrawerSettings::IsLabelVisible(int itemID)
{
    if (labelHash == NULL)
        return true;
    else
    {
        if (labelHash->find(itemID) == labelHash->end())
            return false;
        else
            return true;
    }
}


float TextureDrawerSettings::GetTextureZPosition(int itemID)
{
    IFHash::iterator it = zHash.find(itemID);
    if (it != zHash.end())
    {
        return it->second;
    }
    else
    {
        if (zPosStyle == ZPOS_CONSTANT)
        {
            zHash[itemID] = normalZPos;
            return normalZPos;
        }
        else// if (zPosStyle == ZPOS_RANDOM)
        {
    	    float z = (double)rand()/RAND_MAX;
            z = z - 1.5;
            zHash[itemID] = z;
            return z;
        }
    }
}

void TextureDrawerSettings::BringTextureToFront(int itemID)
{
    if (itemID != -1)
    {
        maxZ += 1e-5f;
        if (maxZ >= -1e-5f)
        {
            maxZ = -0.5;
            ResetZPos();
        }
        zHash[itemID] = maxZ;
    }
}

void TextureDrawerSettings::SetTextureZPosition(int itemID, float zpos)
{
    zHash[itemID] = zpos;
}

void TextureDrawerSettings::SetTextureXYPosition(int itemID, float centerx, float centery)
{
    IPHash::iterator it = centerXYHash.find(itemID);
    if (it != centerXYHash.end())
    {
        delete (Vec2 *)(it->second);
    }

    Vec2 *p = new Vec2(centerx, centery);
    centerXYHash[itemID] = p;
}

void TextureDrawerSettings::GetTextureXYSize(int itemID, float &xSize, float &ySize)
{
	if (IsTextureDistorted(itemID))
	{
		xSize = normalXSize * distortFac;
        ySize = normalYSize * distortFac;
	}
	else
	{
        xSize = normalXSize;
        ySize = normalYSize;
	}
}

void TextureDrawerSettings::SetXYFromDataSet(bool itemXYFromDataSet)
{
    this->itemXYFromDataSet = itemXYFromDataSet;
    if (itemXYFromDataSet)
    {
        if (centerXYHash.size() >= 0)
            ClearCenterXYHash();
    }
}

bool TextureDrawerSettings::IsXYFromDataSet()
{
    return itemXYFromDataSet;
}

void TextureDrawerSettings::ChangeGlyphSize(float newSize)
{
    if (newSize < 0)
        return;
    else if (newSize > 1)
        return;

    normalXSize = newSize;
    normalYSize = newSize;
}

void TextureDrawerSettings::ChangeDistortionFactor(float newSize)
{
    distortFac = newSize;
}

void TextureDrawerSettings::SetSelectedTextureHash(IIHash *hash) 
{
    selectedTextureHash = hash;
    if (refreshZofSelectedTexture == true)
    {
        IIHash::iterator it;
        for (it = hash->begin(); it != hash->end(); it++)
        {
            BringTextureToFront(it->first);
        }
        refreshZofSelectedTexture = false;
        BringTextureToFront(baseTextureID);
    }
}

void TextureDrawerSettings::SetLabelHash(IIHash *hash) 
{
    labelHash = hash;
}

void TextureDrawerSettings::SetShowSelectedOnly(bool only) 
{
    showSelectedOnly = only;
}

void TextureDrawerSettings::SelectedItemsChanged() 
{
    refreshZofSelectedTexture = true;
    if (displayMode == 2 || displayMode == 3)
    {
        SetXYFromDataSet(true);
    }
}

