#ifndef TEXTURE_DRAWER_Settings_H
#define TEXTURE_DRAWER_Settings_H

#include "TableDrawerSettings.h"
#include "../Basic/macros.h"
#include "../Basic/RGB.h"

enum
{ 
	ZPOS_CONSTANT  =   1,
    ZPOS_RANDOM  =   2,
};

//displays using textures share many common parameters, such as distortion factors etc. 
//this class provides the common functions and variable of settings for those displays 
class TextureDrawerSettings : public TableDrawerSettings
{
protected:
    int zPosStyle;
    float normalZPos;
    float maxZ;
    float minZ;
    IFHash zHash;

    bool itemXYFromDataSet;
    float normalXSize;
    float normalYSize;
    IPHash centerXYHash;

    bool distortion;
	float distortFac;
    IIHash distortedTextureHash;

    IIHash* selectedTextureHash;
    IIHash* labelHash;
    bool highlight;

    int focusTextureID;
    int baseTextureID;
    bool refreshZofSelectedTexture;
    
    bool showSelectedOnly;
    int displayMode;

    IIHash displayedTextureHash;

public:
    TextureDrawerSettings();
    ~TextureDrawerSettings();

    void ClearDisplayedTextureHash() {displayedTextureHash.clear();}
    void AddDisplayedTexture(int id) {displayedTextureHash[id] = id;}
    bool IsTextureDisplayed(int id) {return (displayedTextureHash.find(id) != displayedTextureHash.end());}

    void SetZPosStyle(int style) {zPosStyle = style;}
    void Reset();
    void TextureSpaceChanged();
    void ClearCenterXYHash();
    void ResetDistortion();
    void ResetZPos();
    void SetDistortion(bool distort);
    bool IsDistortionEnabled();
    void AddDistortedTexture(int itemID);
    void RemoveDistortedTexture(int itemID);
    bool IsTextureDistorted(int itemID);
    void GetTextureXYPosition(int itemID, float &centerx, float &centery);
    float GetTextureZPosition(int itemID);
    void SetTextureXYPosition(int itemID, float centerx, float centery);
    void SetTextureZPosition(int itemID, float zpos);

    void GetTextureXYSize(int itemID, float &xSize, float &ySize);
    void SetXYFromDataSet(bool itemXYFromDataSet);
    bool IsXYFromDataSet();
    void ChangeGlyphSize(float newSize);
    IPHash* GetCenterXYHash() {return &centerXYHash;}
    void ChangeDistortionFactor(float newSize);

    void BringTextureToFront(int itemID);

    bool IsSelectedTextureHighlighted() {return highlight;}
    void SetSelectedTextureHighlight(bool highlight) {this->highlight = highlight;}
    bool IsTextureSelected(int itemID);

    void SetSelectedTextureHash(IIHash *hash);
    void SetBaseTextureID(int id) {baseTextureID = id;}
    void SetFocusTextureID(int id) {focusTextureID = id;}
    int GetBaseTextureID() {return baseTextureID;}
    int GetFocusTextureID() {return focusTextureID;}
    void SelectedItemsChanged();

    void SetShowSelectedOnly(bool only);
    bool IsShowSelectedOnly() {return showSelectedOnly;}
    virtual void SetDisplayMode(int i) {}

    int GetTextureID(float x, float y);
    void GetTextureXYPosition(int itemID, float &centerx, float &centery, float &xSize, float &ySize);

    int GetTextureNum();
    void GetTextureIDs(int* ids);
    void SetLabelHash(IIHash *hash); 
    bool IsLabelVisible(int itemID);


};

#endif

