#include <FLOAT.h>
#include "VarDrawer.h"
#include "VarDrawerSettings.h"
#include "VarVisualColumn.h"
#include "VarVisualColumnMeta.h"
#include "VarVisualMaskMeta.h"
#include "VarVisualLabelMeta.h"
#include "VarVisualTimeHistogram.h"
#include "VarVisualTimeHistogramMeta.h"
#include "../../MainApplication.h"
#include "../../DataStructure/VideoShotIterator.h"
//#include "../../DataStructure/DateDimension.h"
#include "../../interface/ViewMultipleVideoWindow.h"

#include <wx/file.h>
#include <sstream>

VarDrawer::VarDrawer() : TableDrawer()
{
	settings = new VarDrawerSettings();
  //  AddLayer(AXES);
    AddLayer(COLUMN);
//    AddLayer(MASK);
//    AddLayer(TIMEHISTOGRAM);
    AddLayer(DIMENSIONNAME);
}


VarDrawer::~VarDrawer()
{
    delete settings;
}

void VarDrawer::PrepareCanvas()
{
    canvas->SetViewPortMode(VAR);
    canvas->SetCurrent();
//    canvas->SetLowerLeftOrigin();
    canvas->SetTopLeftOrigin();
    canvas->SetViewport();

//    canvas->SetBackground(colorManager->GetColor(BG));
    canvas->SetBackground(RGBt(0, 0, 0, 1));
    canvas->Clear();

    glEnable(GL_DEPTH_TEST);
    glClear(GL_DEPTH_BUFFER_BIT);
//    glEnable(GL_BLEND);
  //  glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
}

//before drawer draws a canvas
void VarDrawer::PrepareDrawLayer(Layer* layer)
{
    TableDrawer::PrepareDrawLayer(layer);
    canvas->SetCurrent();
    if (layer->GetLayerType() == COLUMN)
    {
        glEnable(GL_DEPTH_TEST);
        glClear(GL_DEPTH_BUFFER_BIT);
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    }
    if (layer->GetLayerType() == DIMENSIONNAME)
    {
        glDisable(GL_DEPTH_TEST);
        //glClear(GL_DEPTH_BUFFER_BIT);
    }
}

//after drawer draws a canvas
void VarDrawer::AfterDrawLayer(Layer* layer)
{
    if (layer->IsTranslucent())
    {
        // Turn off Translucency
    //    canvas->SetTranslucent(false);
    }
}

void VarDrawer::DrawColumn(VisualColumn *column)
{
    VarVisualColumn* visColumn= (VarVisualColumn *)column;

    int glyphID = visColumn->GetDimensionID();
    if (settings->IsShowSelectedOnly())
    {
        if (settings->IsTextureSelected(glyphID) == false)
            return;
    }

    float xPos = visColumn->GetXPosition();
	float yPos = visColumn->GetYPosition();
	float zPos = visColumn->GetZPosition();
	float xSize = visColumn->GetXSize();
	float ySize = visColumn->GetYSize();

	glColor4f(1,1,1,1);
	glEnable( GL_TEXTURE_2D );
	
    glBindTexture( GL_TEXTURE_2D, visColumn->GetTexture());

 	glBegin(GL_QUADS);
	glTexCoord2d(0.0,0.0); glVertex3d(xPos-xSize,yPos-ySize, zPos);
	glTexCoord2d(1.0,0.0); glVertex3d(xPos+xSize,yPos-ySize, zPos);
	glTexCoord2d(1.0,1.0); glVertex3d(xPos+xSize,yPos+ySize, zPos);
	glTexCoord2d(0.0,1.0); glVertex3d(xPos-xSize,yPos+ySize, zPos);
	glEnd();

    if (visColumn->GetMaskMode() == true)
    {
        glBindTexture( GL_TEXTURE_2D, visColumn->GetMaskTexture());

 	    glBegin(GL_QUADS);
	    glTexCoord2d(0.0,0.0); glVertex3d(xPos-xSize,yPos-ySize, zPos + 1E-5);
	    glTexCoord2d(1.0,0.0); glVertex3d(xPos+xSize,yPos-ySize, zPos + 1E-5);
	    glTexCoord2d(1.0,1.0); glVertex3d(xPos+xSize,yPos+ySize, zPos + 1E-5);
	    glTexCoord2d(0.0,1.0); glVertex3d(xPos-xSize,yPos+ySize, zPos + 1E-5);
	    glEnd();
    }

	glDisable( GL_TEXTURE_2D );

    if (settings->IsSelectedTextureHighlighted())
    {
        if (glyphID == settings->GetFocusTextureID())
        {
            glColor4f(1.f, 0.f, 0.f, 1.f);
	        glLineWidth(1.5);
            glBegin(GL_LINE_STRIP);
            float pad = 0.00001f;
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glEnd();
        }
        else if (glyphID == settings->GetBaseTextureID())
        {
            glColor4f(1.f, 0.f, 1.f, 1.f);
	        glLineWidth(1.5);
            glBegin(GL_LINE_STRIP);
            float pad = 0.00001f;
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glEnd();
        }
        else if (settings->IsTextureSelected(glyphID))
        {
//            glColor4f(0.f, 1.f, 0.f, 1.f);
            glColor4f(1.f, 0.f, 0.f, 1.f);
	        glLineWidth(2);
            glBegin(GL_LINE_STRIP);
            float pad = 0.00001f;
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glEnd();
        }
    }

}

void VarDrawer::DrawMask(VarVisualMask *visMask)
{
//    VarVisualMask* visMask= (VarVisualMask *)mask;

    if (visMask->GetTexture() == -1)
        return;

    int glyphID = visMask->GetDimensionID();
    if (settings->IsShowSelectedOnly())
    {
        if (settings->IsTextureSelected(glyphID) == false)
            return;
    }

    float xPos = visMask->GetXPosition();
	float yPos = visMask->GetYPosition();
	float zPos = visMask->GetZPosition();
	float xSize = visMask->GetXSize();
	float ySize = visMask->GetYSize();

	glColor4f(1,1,1,1);
	glEnable( GL_TEXTURE_2D );
	
    glBindTexture( GL_TEXTURE_2D, visMask->GetTexture());

 	glBegin(GL_QUADS);
	glTexCoord2d(0.0,0.0); glVertex3d(xPos-xSize,yPos-ySize, zPos);
	glTexCoord2d(1.0,0.0); glVertex3d(xPos+xSize,yPos-ySize, zPos);
	glTexCoord2d(1.0,1.0); glVertex3d(xPos+xSize,yPos+ySize, zPos);
	glTexCoord2d(0.0,1.0); glVertex3d(xPos-xSize,yPos+ySize, zPos);
	glEnd();

	glDisable( GL_TEXTURE_2D );
}

void VarDrawer::DrawTimeHistogram(VisualTimeHistogram *timeHistogram)
{
    VarVisualTimeHistogram* visTimeHistogram = (VarVisualTimeHistogram *)timeHistogram;

    int glyphID = visTimeHistogram->GetDimensionID();
    if (settings->IsShowSelectedOnly())
    {
        if (settings->IsTextureSelected(glyphID) == false)
            return;
    }

    float xPos = visTimeHistogram->GetXPosition();
	float yPos = visTimeHistogram->GetYPosition();
	float zPos = visTimeHistogram->GetZPosition();
	float xSize = visTimeHistogram->GetXSize();
	float ySize = visTimeHistogram->GetYSize();

	glColor4f(1,1,1,1);
	glEnable( GL_TEXTURE_2D );
	
    glBindTexture( GL_TEXTURE_2D, visTimeHistogram->GetTexture());

 	glBegin(GL_QUADS);
	glTexCoord2d(0.0,0.0); glVertex3d(xPos-xSize,yPos-ySize, zPos);
	glTexCoord2d(1.0,0.0); glVertex3d(xPos+xSize,yPos-ySize, zPos);
	glTexCoord2d(1.0,1.0); glVertex3d(xPos+xSize,yPos+ySize, zPos);
	glTexCoord2d(0.0,1.0); glVertex3d(xPos-xSize,yPos+ySize, zPos);
	glEnd();

/*    if (visColumn->GetMaskMode() == true)
    {
        glBindTexture( GL_TEXTURE_2D, visColumn->GetMaskTexture());

 	    glBegin(GL_QUADS);
	    glTexCoord2d(0.0,0.0); glVertex3d(xPos-xSize,yPos-ySize, zPos + 1E-5);
	    glTexCoord2d(1.0,0.0); glVertex3d(xPos+xSize,yPos-ySize, zPos + 1E-5);
	    glTexCoord2d(1.0,1.0); glVertex3d(xPos+xSize,yPos+ySize, zPos + 1E-5);
	    glTexCoord2d(0.0,1.0); glVertex3d(xPos-xSize,yPos+ySize, zPos + 1E-5);
	    glEnd();
    }*/

	glDisable( GL_TEXTURE_2D );

    if (settings->IsSelectedTextureHighlighted())
    {
        if (glyphID == settings->GetFocusTextureID())
        {
            glColor4f(1.f, 0.f, 0.f, 1.f);
	        glLineWidth(1.5);
            glBegin(GL_LINE_STRIP);
            float pad = 0.00001f;
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glEnd();
        }
        else if (glyphID == settings->GetBaseTextureID())
        {
            glColor4f(1.f, 0.f, 1.f, 1.f);
	        glLineWidth(1.5);
            glBegin(GL_LINE_STRIP);
            float pad = 0.00001f;
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glEnd();
        }
        else if (settings->IsTextureSelected(glyphID))
        {
            glColor4f(0.f, 1.f, 0.f, 1.f);
//            glColor4f(1.f, 0.f, 0.f, 1.f);
	        glLineWidth(2);
            glBegin(GL_LINE_STRIP);
            float pad = 0.00001f;
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos-ySize, zPos+pad);
            glVertex3f(xPos+xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos+ySize, zPos+pad);
            glVertex3f(xPos-xSize,yPos-ySize, zPos+pad);
            glEnd();
        }
    }

}

void VarDrawer::DrawAxes(VisualAxes *axes)
{/*
    VarVisualAxes* visAxes= (VarVisualAxes *)axes;

    MainApplication *app = &wxGetApp(); 
    DocumentManager* docManager = app->GetDocumentManager();
    int activeDimID = docManager->GetActiveDimensionID();

    int dimNum = visAxes->GetDimensionNum();
    int id;
    int n = -1;
    for (int i = 0; i < dimNum; i++)
    {
        id = visAxes->GetDimensionID(i);
        if (id == activeDimID)
        {
            n = i;
            break;
        }
    }

    if (n != -1)
    {
        float xPos = visAxes->GetXPosition(n);
	    float yPos = visAxes->GetYPosition(n);
	    float xSize = visAxes->GetXSize(n);
	    float ySize = visAxes->GetYSize(n);

	    canvas->SetForeground(RGBt(1,0,0));
        canvas->SetLineWidth(3.0);

        canvas->DrawRectangle2(xPos-xSize, xPos+xSize, yPos-ySize, yPos+ySize);

    }*/
}

VisualElementMeta* VarDrawer::CreateVisualColumnMeta(DocumentManager* docManager, bool brushed, bool hier)
{
    VarVisualColumnMeta *meta = new VarVisualColumnMeta(this, docManager, brushed, hier);
    return meta;
}

VisualElementMeta* VarDrawer::CreateVisualMaskMeta(DocumentManager* docManager, bool brushed, bool hier)
{
    VarVisualMaskMeta *meta = new VarVisualMaskMeta(this, docManager, brushed, hier);
    return meta;
}

VisualElementMeta* VarDrawer::CreateVisualTimeHistogramMeta(DocumentManager* docManager, bool brushed, bool hier)
{
    VarVisualTimeHistogramMeta *meta = new VarVisualTimeHistogramMeta(this, docManager, brushed, hier);
    return meta;
}

VisualElementMeta* VarDrawer::CreateVisualAxesMeta()
{
    VarVisualAxesMeta *meta = new VarVisualAxesMeta(this);
    return meta;
}


void VarDrawer::ButtonDown(grCanvas* canvas, int button, Vec2 p)
{
    Drawer::ButtonDown(canvas, button, p);
    inputMode = VAR_NONE;
    int dimID = FindDimensionID(p.X, p.Y);
    if (dimID >= 0)
    {
        if (button == 0)
        {
            if (canvas->IsControlKey())
            {
                inputMode = VAR_MOVE;
                canvas->SetHandCursor();
                currentDim = dimID;
                settings->BringTextureToFront(dimID);
            }
            else if (canvas->IsAltKey())
            {
                std::string str = docManager->GetCurrentDimensionSpace()->GetDimensionByID(dimID)->GetDimensionName();
                if (viewManager->GetMainFrame()->GetVarControlFrame() != NULL)
                    viewManager->GetMainFrame()->GetVarControlFrame()->SetBaseDimension(str);
            }
            else if (canvas->IsShiftKey())
            {
 //               canvas->SetBullseyeCursor();
                if (!settings->IsTextureDistorted(dimID))
                    settings->AddDistortedTexture(dimID);
                else
                    settings->RemoveDistortedTexture(dimID);
                settings->BringTextureToFront(dimID);
                Refresh();
            }
            else
            {
                settings->BringTextureToFront(dimID);
                Refresh();
            }
        }
    }
}

void VarDrawer::ButtonUp(grCanvas* canvas, int button, Vec2 p)
{
    Drawer::ButtonUp(canvas, button, p);
//    canvas->SetDefaultCursor();
    if (inputMode == VAR_MOVE)
    {
        canvas->SetDefaultCursor();
        inputMode = VAR_NONE;
    }
    else
    {
       if (button == 1)
        {
            int dimID = FindDimensionID(p.X, p.Y);
            if (dimID >= 0)
            {
                if (docManager->GetQuery()->GetQueryMode() == SINGLE_QUERY)
                {
                    docManager->SetOrderbyDimension(dimID);
                    canvas->DeleteTextures(VAR_HASH);
                }
                else
                {
                    std::string str = docManager->GetCurrentDimensionSpace()->GetDimensionByID(dimID)->GetDimensionName();
                    if (viewManager->GetMainFrame()->GetVarControlFrame() != NULL)
                        viewManager->GetMainFrame()->GetVarControlFrame()->SetVarGlyphBaseDimension(str);
                }
            }
            Refresh();
        }
        else if (button == 2 && !canvas->IsAltKey())
        {
            int dimID = FindDimensionID(p.X, p.Y);
            if (dimID >= 0)
            {
                float value1, value2;
                if (canvas->IsShiftKey())
                {
                    value1 = -1;
                    value2 = 0.5;
                }
                else
                {
                    value1 = 0.5;
                    value2 = 2;
                }
                docManager->SelectSelectedItemByDimensionValue(dimID, value1, value2);
                canvas->DeleteTextures(VAR_HASH);
                canvas->DeleteTextures(TIME_HISTOGRAM_HASH);
            }
            Refresh();
        }
        else if (button == 2 && canvas->IsAltKey())
        {
            int dimID = FindDimensionID(p.X, p.Y);
            if (dimID >= 0)
            {
                float value1, value2;
                if (canvas->IsShiftKey())
                {
                    value1 = -1;
                    value2 = 0.5;
                }
                else
                {
                    value1 = 0.5;
                    value2 = 2;
                }
                docManager->SelectItemByDimensionValue(dimID, value1, value2);
            }
            Refresh();
        }
    }
}

void VarDrawer::MouseMotion(grCanvas* canvas, Vec2 p)
{
    Drawer::MouseMotion(canvas, p);
    if (inputMode == VAR_MOVE)
    {
        settings->SetTextureXYPosition(currentDim, p.X, p.Y);
        canvas->Refresh();      
    }
    else
    {
        int dimID = FindDimensionID(p.X, p.Y);
        if (dimID >= 0)
        {
            std::string str = docManager->GetCurrentDimensionSpace()->GetDimensionByID(dimID)->GetDimensionName();
            canvas->WriteMessage(str, 0);
        }
    }
}


void VarDrawer::DoubleClick(grCanvas* canvas, int button, Vec2 p) 
{
    Drawer::DoubleClick(canvas, button, p);
    if (button == 0)
    {
        int dimID = FindDimensionID(p.X, p.Y);
        if (dimID >= 0)
        {
            ISHash *hash = settings->GetItemHash(dimID);
            if (hash != NULL)
            {
                std::string localpath = "c:\\yangjing\\video\\multichannel\\cnn";
                std::string globalPath = docManager->GetGlobalPath();
                int timeDimID = docManager->GetQuery()->GetTimeDimensionID();
                int num = docManager->GetDimensionByID(timeDimID)->GetCardinality();
                wxString* names = new wxString [num];
                ISHash::iterator it;

                std::string *captions;
                captions = new std::string [num];

                int *types;
                types = new int [num];
                for (int i = 0; i < num; i++)
                {
                    it = hash->find(i);
                    if (it != hash->end())
                    {
                        wxString tempstr = wxString::Format("%s\\%s", 
                            localpath.c_str(), (it->second).c_str());
                        if (wxFile::Exists(tempstr))
                        {    
                            names[i] = tempstr;
//                            captions[i] = "From Cache";
                            types[i] = 0;
                        }
                        else
                        {
                            names[i] = wxString::Format("%s\\%s", 
                                globalPath.c_str(), (it->second).c_str()); 
//                            captions[i] = "From External disk";
                            types[i] = 1;
                        }
                    }
                    else
                    {
                        names[i] = "";
                        types[i] = -1;
                    }
                }
                std::string title = docManager->GetDimensionByID(dimID)->GetDimensionName();
                Dimension *timeDim = docManager->GetDimensionByID(timeDimID);
                
                std::string *labels;
                double* values;
                int labelNum;
                timeDim->GetLabels(labels, values, labelNum);

                for (i = 0; i < labelNum; i++)
                {
                    captions[timeDim->GetCardFromNorValue(values[i])] = labels[i];
                }
                
                delete [] labels;
                delete [] values;

                IIHash *chash = settings->GetImportanceHash(dimID);
                wxColour *colors = new wxColor[num];
                IIHash::iterator cit;

                for (i = 0; i < num; i++)
                {
                    cit = chash->find(i);
                    if (cit != chash->end())
                    {
                        RGBt color = GetColorManager()->GetColorSetting(NUERIC, cit->second / 10)->GetColor();
                        colors[i] = wxColour(color.R * 255, color.G * 255, color.B * 255);
                    }
                    else
                    {
                        colors[i] = wxColor(100, 100, 100);
                    }
                }


//                MainFrame *mainFrame = docManager->GetMainFrame();
//                mainFrame->NewViewMultipleVideoWindow(num, names, types, title, captions, colors);

ViewMultipleVideoWindow *videoWnd = new ViewMultipleVideoWindow(num, names, types, title, captions, colors);
videoWnd->Maximize(true);
videoWnd->Show(true);
videoWnd->Play();

//                delete [] names;
            }
        }
//        canvas->Refresh();      
    }
    else if (button == 2)
    {
         std::string globalPath = docManager->GetGlobalPath();
         VideoShotIterator *it = new VideoShotIterator(docManager->GetDBManager(), docManager->GetSelectedItemHash());
         MainFrame *mainFrame = docManager->GetMainFrame();
         mainFrame->ShowContiunsVideoPlay(it);
    }
}                              


/*
void VarDrawer::DoubleClick(grCanvas* canvas, int button, Vec2 p) 
{
    Drawer::DoubleClick(canvas, button, p);
    if (button == 0)
    {
        int dimID = FindDimensionID(p.X, p.Y);
        if (dimID >= 0)
        {
            ISHash *hash = settings->GetItemHash(dimID);
            if (hash != NULL)
            {
                int num = hash->size();
                wxString* names = new wxString [num];
                ISHash::iterator it;

                it = hash->begin();
                int count = 0;
                std::string globalPath = docManager->GetGlobalPath();
                while (it != hash->end())
                {
                    names[count] = wxString::Format("%s\\%s", globalPath.c_str(), (it->second).c_str());
                    it ++;
                    count ++;
                }
                MainFrame *mainFrame = docManager->GetMainFrame();
                mainFrame->NewViewMultipleVideoWindow(num, names);
                delete [] names;
            }
        }
        canvas->Refresh();      
    }
    else if (button == 2)
    {
         std::string globalPath = docManager->GetGlobalPath();
         VideoShotIterator *it = new VideoShotIterator(docManager->GetDBManager(), docManager->GetSelectedItemHash());
         MainFrame *mainFrame = docManager->GetMainFrame();
         mainFrame->ShowContiunsVideoPlay(it);
    }
}                              
*/

void VarDrawer::AllDimensionSpaceDeleted()
{
    settings->Reset();

    LinkListIterator* iter;

    iter = new LinkListIterator(&canvasList);
    grCanvas* canv;
    while (! iter->IsDone())
    {
        canv = (grCanvas *) iter->GetNext();
        canv->DeleteTextures(VAR_HASH);
        canv->DeleteTextures(TIME_HISTOGRAM_HASH);
    }

    delete iter;

}


void VarDrawer::CurrentDimensionSpaceChanged()
{
    settings->TextureSpaceChanged();
}

int VarDrawer::FindDimensionID(float x, float y)
{
    return settings->GetTextureID(x, y);
}

void VarDrawer::SelectedItemsChanged()
{
    LinkListIterator* iter;

    iter = new LinkListIterator(&canvasList);
    grCanvas* canv;
    while (! iter->IsDone())
    {
        canv = (grCanvas *) iter->GetNext();
        canv->DeleteTextures(VAR_HASH);
        canv->DeleteTextures(TIME_HISTOGRAM_HASH);
    }

    delete iter;
}

VisualElementMeta* VarDrawer::CreateVisualDimensionNameMeta(DocumentManager* docManager)
{
    float angle, scale, shiftX, shiftY;
    angle = 45;
    scale = 1.;
    
    Vec2 p = canvas->MapScreeVectorToDataVector(Vec2(2, 2));
    shiftX = p.X;
    shiftY = p.Y;
    
    float fontHeight = canvas->GetFontHeight();

    float rim = docManager->GetCurrentDimensionSpace()->GetDimension(0)->GetRimFactor();

    if (rim < fontHeight)
        shiftY += fontHeight - rim;

    VarVisualLabelMeta *meta = new VarVisualLabelMeta(this, docManager, 
        angle, scale, shiftX, shiftY, shiftX, shiftY);
    return meta;
}
