#include <cmath>
#include "VarVisualColumnMeta.h"
#include "VarVisualColumn.h"
#include "../../DataStructure/DocColumn.h"
#include "../../MainApplication.h"

VarVisualColumnMeta::VarVisualColumnMeta(VarDrawer* drawer, 
                          DocumentManager* docManager, bool brushed, bool hier)
{
    this->drawer = drawer;
    binaryMode = docManager->IsBinaryMode();
    settings = drawer->GetSettings();
	colorSetting = drawer->GetColorManager()->GetColorSetting(SBB, 0);
//    VarDrawerSettings = drawer->GetSettings();
    glyphStyle = drawer->GetSettings()->GetGlyphStyle();

    axesLayer = new Layer(AXES);
    axesMeta = axesLayer->CreateVisualElementMeta(drawer, docManager);
    axesIter = docManager->GetElementsIterator(AXES, axesMeta);
    visAxesIter  = new VisualElementsIterator(axesMeta, axesIter); 
    
    axes = (VarVisualAxes *)(visAxesIter->GetNext());

    int dimNum = axes->GetDimensionNum();
    int *ids = new int[dimNum];
    axes->GetDimensionIDs(ids);
    canvas = drawer->GetCanvas();
    GLuint texture;
    int width, height;
	int dimID;

    //notload[i] indicates if column i needs to be loaded. If the texture of it already
    //exists, then it doesn't need to be loaded. 
    notLoad = new bool [dimNum];
    for (int i = 0; i < dimNum; i++)
    {
		dimID = ids[i];
        if (canvas->GetTexture(VAR_HASH, dimID, texture, width, height))
            notLoad[i] = true;
        else
            notLoad[i] = false;
    }

    //count records the order of the dimension to be converted. 
    count = 0;

    pos = NULL;
    colorData = NULL;

    maskTexture = -1;
    delete [] ids;

}

VarVisualColumnMeta::~VarVisualColumnMeta()
{
    delete axesLayer;
    delete visAxesIter;
    delete axesMeta;
    delete axesIter; 
    delete [] notLoad;
    if (pos != NULL)
        delete [] pos;
    if (colorData != NULL)
        delete [] colorData;
}

VisualElement* VarVisualColumnMeta::InstantVisualElement()
{
    VarVisualColumn* visColumn = new VarVisualColumn();
    
    return visColumn;
}

void VarVisualColumnMeta::Convert(VisualElement* visualEle, DocElement* docEle)
{
    VarVisualColumn* visualColumn = (VarVisualColumn *)visualEle;
    DocColumn* docColumn = (DocColumn *)docEle;

    GLuint texture;
    int width;
    int height;

    int dimID = docColumn->GetDimensionID();
    if (docColumn->NotLoad() == true)
        canvas->GetTexture(VAR_HASH, dimID, texture, width, height);
    else
    {
        if (docColumn->GetMode() == TRIPLE)
            GetBitMapWithThreeDimension(docColumn->GetSize(), docColumn->GetData(), docColumn->GetXData(), docColumn->GetYData());
        else if (docColumn->GetMode() == DUAL)
            GetBitMapWithTwoDimension2(docColumn->GetSize(), docColumn->GetData(), docColumn->GetBaseData());
        if (docColumn->GetMode() == SINGLE)
        {
            if (!binaryMode)
            {
                GetBitMapWithSingleDimensionInNonBinaryMode(docColumn->GetSize(), docColumn->GetData(),
                                         docColumn->GetItemIDs());
            }
            else
            {
                GetBitMapWithSingleDimensionInBinaryMode(docColumn->GetSize(), docColumn->GetData(),
                                         docColumn->GetItemIDs());
            }
        }
        width = w;
        height = h;
        canvas->CreateTexture(VAR_HASH, dimID, w, h, colorData, texture);
    }

    if (docColumn->IsMaskMode())
    {
        if (maskTexture == -1)
        {
            int temp1, temp2;
            if (!canvas->GetTexture(VAR_HASH, -1, maskTexture, temp1, temp2))
            {
                GetBitMapForMask(docColumn->GetSize(), docColumn->GetMaskData());
                width = w;
                height = h;
                canvas->CreateTexture(VAR_HASH, -1, width, height, colorData, maskTexture);
            }
        }
        visualColumn->SetMaskMode(true);
        visualColumn->SetMaskTexture(maskTexture);
    }
    else
        visualColumn->SetMaskMode(false);

    visualColumn->SetTexture(texture);
    visualColumn->SetWidth(width);        //width: the width of the original texture object
    visualColumn->SetHeight(height);      //height: the height of the origianl texture object
    
    float xPos, yPos, zPos, xSize, ySize;
    axes->GetDimensionXYPos(dimID, xPos, yPos);
    zPos = axes->GetDimensionZPos(dimID);
    axes->GetDimensionXYSize(dimID, xSize, ySize);

    visualColumn->SetXPosition(xPos);
    visualColumn->SetYPosition(yPos);
    visualColumn->SetZPosition(zPos);
    visualColumn->SetXSize(xSize);
    visualColumn->SetYSize(ySize);

    visualColumn->SetDimensionID(dimID);

    count ++;
}

//for normal data
//size: the length of array "data", data: the column data;
//pos: the bitmap to be generated. It will be used in the texture object.
void VarVisualColumnMeta::GetBitMapWithSingleDimensionInNonBinaryMode(int size, float* data, int* itemIDs)
{
    if (pos == NULL)
    {
    	pos = new int[size];
        if (glyphStyle == PIXEL_SPIRAL)
            GetSpiralPositions(size, pos, w, h);
        else if (glyphStyle == PIXEL_VERTICAL)
            GetVerticalPositions(size, pos, w, h);
        else if (glyphStyle == PIXEL_HORI)
            GetHorizontalPositions(size, pos, w, h);
        else if (glyphStyle == PIXEL_HCURVE)
            GetHCurvePositions(size, pos, w, h);

        colorData = new float[w*h*4];
        for (int i = 0; i < w*h; i++)
        {
			colorData[i*4+0] = 1;
			colorData[i*4+1] = 1;
			colorData[i*4+2] = 1;
			colorData[i*4+3] = 1;
        
        }
    }

    int position;
    RGBt color;
    for (int j = 0; j < size; j++)
	{
        float value = data[j];
		color = colorSetting->GetColor(value);
        position = pos[j];
		colorData[position*4+0] = 1- color.R;
		colorData[position*4+1] = 1-color.G;
		colorData[position*4+2] = 1-color.B;
		colorData[position*4+3] = 1;
    }
}

//for binary data
//size: the length of array "data", data: the column data;
//pos: the bitmap to be generated. It will be used in the texture object.
void VarVisualColumnMeta::GetBitMapWithSingleDimensionInBinaryMode(int size, float* data, int* itemIDs)
{
    if (pos == NULL)
    {
    	pos = new int[size];
        if (glyphStyle == PIXEL_SPIRAL)
            GetSpiralPositions(size, pos, w, h);
        else if (glyphStyle == PIXEL_VERTICAL)
            GetVerticalPositions(size, pos, w, h);
        else if (glyphStyle == PIXEL_HCURVE)
            GetHCurvePositions(size, pos, w, h);

        colorData = new float[w*h*4];
        for (int i = 0; i < w*h; i++)
        {
			colorData[i*4+0] = 1;
			colorData[i*4+1] = 1;
			colorData[i*4+2] = 1;
			colorData[i*4+3] = 1;
        }
    }

    int position;
    RGBt color;

    bool selected;
    for (int j = 0; j < size; j++)
	{
        float value = data[j];
		color = colorSetting->GetColor(value);
        position = pos[j];

//        selected = settings->IsItemSelected(itemIDs[j]);
 selected = false;
        if (selected && value > 0.5)
        {
		    colorData[position*4+0] = 1.f;
		    colorData[position*4+1] = 1.f;
		    colorData[position*4+2] = 0.f;
		    colorData[position*4+3] = 1;
        }
        else if (selected && value < 0.5)
        {
		    colorData[position*4+0] = 0.3f;
		    colorData[position*4+1] = 0.3f;
		    colorData[position*4+2] = 0.f;
		    colorData[position*4+3] = 1;
        }
        else if (!selected && value > 0.5)
        {
		    colorData[position*4+0] = 0;
		    colorData[position*4+1] = 1;
		    colorData[position*4+2] = 1;
		    colorData[position*4+3] = 1;
        }
        else if (!selected && value < 0.5)
        {
		    colorData[position*4+0] = .5f;
		    colorData[position*4+1] = .5f;
		    colorData[position*4+2] = .5f;
		    colorData[position*4+3] = 1;
        }
    }

}


//the function returns the position of data items in a spirally arranged bitmap.
//size: the number of data items
//pos: position of each data item in a bitmap
//w: the width of the bitmap
//h: the height of the bitmap
void VarVisualColumnMeta::GetSpiralPositions(int size, int* pos, int &w, int &h)
{
	w = sqrt((float)size);
	if (size > w*w)
		w += 1;

	int centerx, centery;
	centerx = w/2;
	centery = w/2;
	int add_case = 0;
	int enum_num = 2;
	int count = 0;

	int position;
    for (int j = 0; j < size; j++)
	{
		position = centery * w + centerx;

        pos[j] = position;
        switch (add_case) 
		{
		case 0: 
			centery -= 1;
			break;
		case 1: 
			centerx -= 1;
			break;
		case 2: 
			centery += 1;
			break;
		case 3: 
			centerx += 1;
			break;
		}

		count++;
		if (count == enum_num/2)
		{
			count = 0;
			enum_num ++;
			switch (add_case) 
			{
			case 0: 
				add_case = 1;
				break;
			case 1: 
				add_case = 2;
				break;
			case 2: 
				add_case = 3;
				break;
			case 3: 
				add_case = 0;
				break;
			}
		}
	}

    h = w;
}

//the function returns the position of data items in a vertical arranged bitmap.
//size: the number of data items
//pos: position of each data item in a bitmap
//w: the width of the bitmap
//h: the height of the bitmap
void VarVisualColumnMeta::GetVerticalPositions(int size, int* pos, int &w, int &h)
{
	w = sqrt((float)size);
    h = size / w;

	if (size != w*h)
		h += 1;

    int position;
    position = 0;
    for (int j = 0; j < size; j++)
	{
        pos[j] = j;
    }
}

void VarVisualColumnMeta::GetHorizontalPositions(int size, int* pos, int &w, int &h)
{
	w = sqrt((float)size);
    h = size / w;

	if (size != w*h)
		h += 1;

    int x, y;
    for (int j = 0; j < size; j++)
	{
        x = j / h;
        y = j % h;

        pos[j] = w * y + x;
    }
}


void VarVisualColumnMeta::Hilbert(int size, int *pos, int &i, int w, int h, 
                                  int x0, int y0, int xis, int xjs, int yis, int yjs, int n)
// x0 and y0 are the coordinates of the bottom left corner
// xis & xjs are the i & j components of the unit x vector this frame
// similarly yis and yjs
{
    if (n <= 0)
    {
        if (i < size)
        {
            int posx = ((x0+(xis+yis)/2) - 1) / 2;
            int posy = ((y0+(xjs+yjs)/2) - 1) / 2;
            pos[i ++] = posy * w + posx;
        }
    }
    else
    {
        Hilbert(size, pos, i, w, h, x0, y0, yis/2, yjs/2, xis/2, xjs/2, n-1);
        Hilbert(size, pos, i, w, h, x0+xis/2, y0+xjs/2 ,xis/2, xjs/2, yis/2, yjs/2, n-1);
        Hilbert(size, pos, i, w, h, x0+xis/2+yis/2, y0+xjs/2+yjs/2, xis/2, xjs/2, yis/2, yjs/2,n-1);
        Hilbert(size, pos, i, w, h, x0+xis/2+yis, y0+xjs/2+yjs, -yis/2,-yjs/2, -xis/2, -xjs/2,n-1);
    }
}

void VarVisualColumnMeta::GetHCurvePositions(int size, int* pos, int &w, int &h)
{
    int n = 0;
    w = h = 1;
    while (w * h < size)
    {
        w *= 2;
        h *= 2;
        n ++;
    }
    int i = 0;
    Hilbert(size, pos, i, w, h, 0, 0, w*2, 0, 0, h*2, n);
}

//size: the length of array "data", data: the column data;
//pos: the bitmap to be generated. It will be used in the texture object.
void VarVisualColumnMeta::GetBitMapWithTwoDimension(int size, float* data, float* baseData)
{
    int n = 24;
    w = n;
    h = n;
    int wh = w * h;

    if (colorData == NULL)
        colorData = new float[w*h*4];

    for (int i = 0; i < wh; i++)
    {
		colorData[i*4+0] = 0.0;
		colorData[i*4+1] = 0.0;
		colorData[i*4+2] = 0.0;
		colorData[i*4+3] = 1;
    }

    int columnIndex;
    int rowIndex;
    float value;
    int position;
    RGBt color;

    //note: for further improvement: handle the <0 > w case
    for (i = 0; i < size; i++)
    {
        value = data[i];
		color = colorSetting->GetColor(value);

        columnIndex = baseData[i] * w;
        rowIndex = data[i] * h;

        if (columnIndex < 0)
            columnIndex = 0;
        else if (columnIndex >= w)
            columnIndex = w-1;

        if (rowIndex < 0)
            rowIndex = 0;
        else if (rowIndex >= h)
            rowIndex = h-1;

        position = rowIndex * w + columnIndex;

        colorData[position*4+0] = 1-color.R;
		colorData[position*4+1] = 1-color.G;
		colorData[position*4+2] = 1-color.B;
		colorData[position*4+3] = color.A;
    }
}

//size: the length of array "data", data: the column data;
//pos: the bitmap to be generated. It will be used in the texture object.
void VarVisualColumnMeta::GetBitMapWithTwoDimension2(int size, float* data, float* baseData)
{
    int n = 32;
    w = n;
    h = n;
    int wh = w * h;

    if (colorData == NULL)
        colorData = new float[w*h*4];

    int *counts;
    counts = new int[w*h];
    
    for (int i = 0; i < wh; i++)
    {
		colorData[i*4+0] = 0.3f;
		colorData[i*4+1] = 0.0;
		colorData[i*4+2] = 0.0;
		colorData[i*4+3] = 0.5f;
/*
		colorData[i*4+0] = 1.f;
		colorData[i*4+1] = .0;
		colorData[i*4+2] = .0;
		colorData[i*4+3] = 1.f;
*/
        counts[i] = 0;
    }

    int columnIndex;
    int rowIndex;
    float value;
    int position;
    RGBt color;

    //note: for further improvement: handle the <0 > w case
    for (i = 0; i < size; i++)
    {
        value = data[i];
		color = colorSetting->GetColor(value);

        columnIndex = baseData[i] * w;
        rowIndex = data[i] * h;

        if (columnIndex < 0)
            columnIndex = 0;
        else if (columnIndex >= w)
            columnIndex = w-1;

        if (rowIndex < 0)
            rowIndex = 0;
        else if (rowIndex >= h)
            rowIndex = h-1;

        position = rowIndex * w + columnIndex;

        counts[position] ++;
    }

    for (i = 0; i < wh; i++)
    {
        if (counts[i] != 0)
        {
            value = counts[i] / (float) size;
		    color = colorSetting->GetColor(value);

 /*       colorData[i*4+0] = color.R;
		colorData[i*4+1] = color.G;
		colorData[i*4+2] = color.B;
		colorData[i*4+3] = color.A;
*/            colorData[i*4+0] = value * 10;
		    colorData[i*4+1] = value * 10;
		    colorData[i*4+2] = value * 10;
		    colorData[i*4+3] = 1;
        }
    }
    delete [] counts;
}


void VarVisualColumnMeta::GetBitMapWithThreeDimension(int size, float* data, float* xData, float* yData)
{
    int n = sqrt(size);
    w = n;
    h = n;
    int wh = w * h;

    if (colorData == NULL)
        colorData = new float[w*h*4];

    int *counts;
    counts = new int[w*h];
    
    for (int i = 0; i < wh; i++)
    {
		colorData[i*4+0] = 0.0;
		colorData[i*4+1] = 0.0;
		colorData[i*4+2] = 0.0;
		colorData[i*4+3] = 1.f;
    }

    int columnIndex;
    int rowIndex;
    float value;
    int position;
    RGBt color;

    for (i = 0; i < wh; i++)
    {
        columnIndex = xData[i] * w;
        rowIndex = yData[i] * h;

        if (columnIndex < 0)
            columnIndex = 0;
        else if (columnIndex >= w)
            columnIndex = w-1;

        if (rowIndex < 0)
            rowIndex = 0;
        else if (rowIndex >= h)
            rowIndex = h-1;

        position = rowIndex * w + columnIndex;

        value = data[i];
		color = colorSetting->GetColor(value);

        colorData[position*4+0] = 1-color.R;
		colorData[position*4+1] = 1-color.G;
		colorData[position*4+2] = 1-color.B;
		colorData[position*4+3] = color.A;
    }
    delete [] counts;
}



//size: the length of array "data", data: the column data;
//pos: the bitmap to be generated. It will be used in the texture object.
void VarVisualColumnMeta::GetBitMapForMask(int size, float* maskData)
{
    if (pos == NULL)
    {
    	pos = new int[size];
        if (glyphStyle == PIXEL_SPIRAL)
            GetSpiralPositions(size, pos, w, h);
        else if (glyphStyle == PIXEL_VERTICAL)
            GetVerticalPositions(size, pos, w, h);
        else if (glyphStyle == PIXEL_HORI)
            GetHorizontalPositions(size, pos, w, h);
        else if (glyphStyle == PIXEL_HCURVE)
            GetHCurvePositions(size, pos, w, h);

        colorData = new float[w*h*4];
        for (int i = 0; i < w*h; i++)
        {
			colorData[i*4+0] = 0.0;
			colorData[i*4+1] = 0.0;
			colorData[i*4+2] = 0.0;
			colorData[i*4+3] = 0.;
        }
    }

    float maskAlpha = drawer->GetSettings()->GetMaskAlpha();
    RGBt color = drawer->GetSettings()->GetMaskColor();

    int position;
    for (int j = 0; j < size; j++)
	{
        position = pos[j];
        if (maskData[j] < 0.5f)
        {
		    colorData[position*4+0] = 0;
		    colorData[position*4+1] = 0;
		    colorData[position*4+2] = 0;
		    colorData[position*4+3] = 0;
        }
        else
        {
		    colorData[position*4+0] = 1- color.R;
		    colorData[position*4+1] = 1- color.G;
		    colorData[position*4+2] = 1- color.B;
		    colorData[position*4+3] = maskAlpha;
        }
/*        colorData[position*4+0] = 0;
        colorData[position*4+1] = 0;
        colorData[position*4+2] = 0;
        colorData[position*4+3] = 1;//maskData[j];*/
    }
}
