#include <cmath>
#include "VarVisualMaskMeta.h"
#include "VarVisualMask.h"
#include "../../DataStructure/DocColumn.h"

VarVisualMaskMeta::VarVisualMaskMeta(VarDrawer* drawer, DocumentManager* docManager, bool brushed, bool hier)
{
    this->drawer = drawer;
    binaryMode = docManager->IsBinaryMode();
    settings = drawer->GetSettings();
	colorSetting = drawer->GetColorManager()->GetColorSetting(SBB, 0);
    glyphStyle = drawer->GetSettings()->GetGlyphStyle();

    axesLayer = new Layer(AXES);
    axesMeta = axesLayer->CreateVisualElementMeta(drawer, docManager);
    axesIter = docManager->GetElementsIterator(AXES, axesMeta);
    visAxesIter  = new VisualElementsIterator(axesMeta, axesIter); 
    
    axes = (VarVisualAxes *)(visAxesIter->GetNext());

    int dimNum = axes->GetDimensionNum();
    int *ids = new int[dimNum];
    axes->GetDimensionIDs(ids);
    canvas = drawer->GetCanvas();
    GLuint texture;
    int width, height;
	int dimID;

    SetUpCounter(dimNum);
    loadSpace = new DimensionSpace();
    
    if (canvas->GetTexture(VAR_HASH, -1, texture, width, height))
    {
        howGet[0] = ids[0];
    }
    else
    {
        howGet[0] = UNKNOWN;
        loadSpace->AddDimensionAtEnd(docManager->GetDimensionByID(ids[0]));
    }

    for (int i = 1; i < dimNum; i++)
    {
//    	dimID = ids[0];
        howGet[i] = ids[i];
    }

    pos = NULL;
    colorData = NULL;
    delete [] ids;
}

VarVisualMaskMeta::~VarVisualMaskMeta()
{
}

VisualElement* VarVisualMaskMeta::InstantVisualElement()
{
    VarVisualMask* visMask = new VarVisualMask();
    
    return visMask;
}

void VarVisualMaskMeta::GetVisualElement(VisualElement* visEle, int dimID) 
{
    VarVisualMask* varVisualMask= (VarVisualMask*)visEle;

    GLuint texture;
    int width;
    int height;
    if (canvas->GetTexture(VAR_HASH, -1, texture, width, height) == true)
    {
        varVisualMask->SetTexture(texture);
        varVisualMask->SetWidth(width);        //width: the width of the original texture object
        varVisualMask->SetHeight(height);      //height: the height of the origianl texture object
    
        float xPos, yPos, zPos, xSize, ySize;
        axes->GetDimensionXYPos(dimID, xPos, yPos);
        zPos = axes->GetDimensionZPos(dimID);
        axes->GetDimensionXYSize(dimID, xSize, ySize);

        varVisualMask->SetXPosition(xPos);
        varVisualMask->SetYPosition(yPos);
        varVisualMask->SetZPosition(zPos + 0.00001);
        varVisualMask->SetXSize(xSize);
        varVisualMask->SetYSize(ySize);

        varVisualMask->SetDimensionID(dimID);
    }
    else
        varVisualMask->SetTexture(-1);

}

void VarVisualMaskMeta::Convert(VisualElement* visualEle, DocElement* docEle)
{
    VarVisualMask* visualMask = (VarVisualMask *)visualEle;
    DocColumn* docColumn= (DocColumn *)docEle;
    int dimID = docColumn->GetDimensionID();

    GLuint texture;
    int width;
    int height;

    if (docColumn->IsMaskMode())
    {
        GetBitMapForMask(docColumn->GetSize(), docColumn->GetMaskData());
        width = w;
        height = h;
        canvas->CreateTexture(VAR_HASH, -1, width, height, colorData, texture);
        visualMask->SetTexture(texture);

        visualMask->SetTexture(texture);
        visualMask->SetWidth(width);        //width: the width of the original texture object
        visualMask->SetHeight(height);      //height: the height of the origianl texture object
    
        float xPos, yPos, zPos, xSize, ySize;
        axes->GetDimensionXYPos(dimID, xPos, yPos);
        zPos = axes->GetDimensionZPos(dimID);
        axes->GetDimensionXYSize(dimID, xSize, ySize);

        visualMask->SetXPosition(xPos);
        visualMask->SetYPosition(yPos);
        visualMask->SetZPosition(zPos + 0.00001);
        visualMask->SetXSize(xSize);
        visualMask->SetYSize(ySize);

        visualMask->SetDimensionID(dimID);
    }
    else
        visualMask->SetTexture(-1);
}
