#include <cmath>
#include "VarVisualTimeHistogramMeta.h"
#include "VarVisualTimeHistogram.h"
#include "../../DataStructure/DocTimeHistogram.h"
#include "../../MainApplication.h"
#include "../../Basic/macros.h"


VarVisualTimeHistogramMeta::VarVisualTimeHistogramMeta(VarDrawer* drawer, 
                          DocumentManager* docManager, bool brushed, bool hier)
{
    this->drawer = drawer;

//    colorSetting = (drawer->GetColorManager())->GetColorSetting(HIER_UNBRUSHED_DATA);
    settings = drawer->GetSettings();
//    timeDim = docManager->GetDimensionByID(settings->GetTimeDimensionID());
    timeDim = docManager->GetDimensionByID(0);
	colorSetting = drawer->GetColorManager()->GetColorSetting(SBB, 0);
//    glyphStyle = drawer->GetSettings()->GetGlyphStyle();

    axesLayer = new Layer(AXES);
    axesMeta = axesLayer->CreateVisualElementMeta(drawer, docManager);
    axesIter = docManager->GetElementsIterator(AXES, axesMeta);
    visAxesIter  = new VisualElementsIterator(axesMeta, axesIter); 
    
    axes = (VarVisualAxes *)(visAxesIter->GetNext());

    int dimNum = axes->GetDimensionNum();
    int *ids = new int[dimNum];
    axes->GetDimensionIDs(ids);
    canvas = drawer->GetCanvas();
    GLuint texture;
    int width, height;
	int dimID;

    SetUpCounter(dimNum);
    loadSpace = new DimensionSpace();
    for (int i = 0; i < dimNum; i++)
    {
		dimID = ids[i];
        if (canvas->GetTexture(TIME_HISTOGRAM_HASH, dimID, texture, width, height))
        {
            howGet[i] = dimID;
        }
        else
        {
            howGet[i] = UNKNOWN;
            loadSpace->AddDimensionAtEnd(docManager->GetDimensionByID(dimID));
        }
    }

    colorData = NULL;
    delete [] ids;

    settings->ClearDisplayedTextureHash();
}

VarVisualTimeHistogramMeta::~VarVisualTimeHistogramMeta()
{
    delete axesLayer;
    delete visAxesIter;
    delete axesMeta;
    delete axesIter; 
    if (colorData != NULL)
        delete [] colorData;
    delete loadSpace;
}

VisualElement* VarVisualTimeHistogramMeta::InstantVisualElement()
{
    VarVisualTimeHistogram* visTimeHistogram = new VarVisualTimeHistogram();
    
    return visTimeHistogram;
}

void VarVisualTimeHistogramMeta::GetVisualElement(VisualElement* visEle, int dimID) 
{
    settings->AddDisplayedTexture(dimID);
    VarVisualTimeHistogram* visualTimeHistogram = (VarVisualTimeHistogram *)visEle;

    GLuint texture;
    int width;
    int height;
    canvas->GetTexture(TIME_HISTOGRAM_HASH, dimID, texture, width, height);

    visualTimeHistogram->SetTexture(texture);
    visualTimeHistogram->SetWidth(width);        //width: the width of the original texture object
    visualTimeHistogram->SetHeight(height);      //height: the height of the origianl texture object
    
    float xPos, yPos, zPos, xSize, ySize;
    axes->GetDimensionXYPos(dimID, xPos, yPos);
    zPos = axes->GetDimensionZPos(dimID);
    axes->GetDimensionXYSize(dimID, xSize, ySize);

    visualTimeHistogram->SetXPosition(xPos);
    visualTimeHistogram->SetYPosition(yPos);
    visualTimeHistogram->SetZPosition(zPos);
    visualTimeHistogram->SetXSize(xSize);
    visualTimeHistogram->SetYSize(ySize);

    visualTimeHistogram->SetDimensionID(dimID);
}


void VarVisualTimeHistogramMeta::Convert(VisualElement* visualEle, DocElement* docEle)
{
    VarVisualTimeHistogram* visualTimeHistogram = (VarVisualTimeHistogram *)visualEle;
    DocTimeHistogram* docTimeHistogram = (DocTimeHistogram *)docEle;

    GLuint texture;
    int width;
    int height;

    int dimID = docTimeHistogram->GetDimensionID();
    settings->AddDisplayedTexture(dimID);

    GetHistogramBitMap_period(docTimeHistogram->GetBinNum(), docTimeHistogram->GetHistogram(), docTimeHistogram->GetMaxNum());
    width = w;
    height = h;
    canvas->CreateTexture(TIME_HISTOGRAM_HASH, dimID, w, h, colorData, texture);


    visualTimeHistogram->SetTexture(texture);
    visualTimeHistogram->SetWidth(width);        //width: the width of the original texture object
    visualTimeHistogram->SetHeight(height);      //height: the height of the origianl texture object
    
    float xPos, yPos, zPos, xSize, ySize;
    axes->GetDimensionXYPos(dimID, xPos, yPos);
    zPos = axes->GetDimensionZPos(dimID);
    axes->GetDimensionXYSize(dimID, xSize, ySize);

    visualTimeHistogram->SetXPosition(xPos);
    visualTimeHistogram->SetYPosition(yPos);
    visualTimeHistogram->SetZPosition(zPos);
    visualTimeHistogram->SetXSize(xSize);
    visualTimeHistogram->SetYSize(ySize);

    visualTimeHistogram->SetDimensionID(dimID);

    settings->SetItemHash(docTimeHistogram->GetItemHash(), dimID);
    settings->SetImportanceHash(docTimeHistogram->GetImportanceHash(), dimID);
}

/*void VarVisualTimeHistogramMeta::GetHistogramBitMap(int binNum, int* histogram, int maxNum)
{
    if (colorData == NULL)
    {
        binw = 8;
        h = 128;
        w = binw * binNum;

        colorData = new float[w*h*4];
    }

    wxMemoryDC dc;

    wxBitmap map(w, h); 
    dc.SelectObject(map);

    dc.SetBackground(*wxBLACK_BRUSH);
    dc.SetPen(*wxGREEN_PEN);
    dc.SetBrush(*wxMEDIUM_GREY_BRUSH);

    double height;
    for (int i = 0; i < binNum; i++)
    {
        height = histogram[i] / (double) maxNum * h;
        dc.DrawRectangle(binw * i, 0, binw, height);
    }

    //copy the pixel info of the Dc into an array named data
    float * data = new float[4*w*h];

    wxColor color;
    int j, k;
    for (j = 0; j < h; j++)
    {
        for (i = 0; i < w; i++)
        {
            k = j*w*4+i*4;
            dc.GetPixel(i, j, &color);
            colorData[k+0] = color.Red()/255.;
            colorData[k+1] = color.Green()/255.;
            colorData[k+2] = color.Blue()/255.;
            colorData[k+3] = 1;
        }
    }
}
*/

void VarVisualTimeHistogramMeta::GetHistogramBitMap_line(int binNum, int* histogram, int maxNum)
{
    if (colorData == NULL)
    {
        GetUnitTextureSize(binNum, 1, 8, 24, 1, 1, w, h);
        colorData = new float[w*h*4];
    }

    RGBt **color_matrixA, **color_matrixB;
    
    color_matrixA = new RGBt* [1];
    color_matrixB = new RGBt* [1];
    color_matrixA[0] = new RGBt [binNum];
    color_matrixB[0] = new RGBt [binNum];
    
    double** ratio_AB;
    ratio_AB = new double* [1];
    ratio_AB[0] = new double [binNum];

    for (int i = 0; i < binNum; i++)
    {
        color_matrixA[0][i] = RGBt(0.5, 1, 1, 1);
        color_matrixB[0][i] = RGBt(1, 1, 1, 0);
        ratio_AB[0][i] = 1. - histogram[i] / (double) maxNum;
    }

    FillUnitTexture(binNum, 1, color_matrixB, color_matrixA, ratio_AB, 8, 24, RGBt(1, 1, 1, 1), 1, 1, colorData);
}

/*
void VarVisualTimeHistogramMeta::GetHistogramBitMap_period(int binNum, int* histogram, int maxNum)
{
    RGBt **color_matrixA, **color_matrixB;
    double ** ratio_AB;

    int rowNum, columnNum;
    columnNum = 7;
    rowNum = ceil(binNum / (double) columnNum);
    
    color_matrixA = new RGBt* [rowNum];
    color_matrixB = new RGBt* [rowNum];
    ratio_AB = new double* [rowNum];

    int i, j;
    for (i = 0; i < rowNum; i++)
    {
        color_matrixA[i] = new RGBt [columnNum];
        color_matrixB[i] = new RGBt [columnNum];
        ratio_AB[i] = new double [columnNum];
    }
    
    for(i = 0; i < rowNum; i++)
    {
        for (j = 0; j < columnNum; j++)
        {
            if (i*rowNum + j < binNum)
            {
                color_matrixA[i][j] = RGBt(1, 0.5, 1, 1);
                color_matrixB[i][j] = RGBt(1, 1, 1, 0);
                ratio_AB[i][j] = 1. - histogram[i*rowNum + j] / (double) maxNum;
            }
            else
            {
                color_matrixA[i][j] = RGBt(1, 0.5, 1, 1);
                color_matrixB[i][j] = RGBt(1, 1, 1, 0);
                ratio_AB[i][j] = 0;
            }
        }
    }

    if (colorData == NULL)
    {
        GetUnitTextureSize(columnNum, rowNum, 8, 24, 1, 1, w, h);
        colorData = new float[w*h*4];
    }


    FillUnitTexture(columnNum, rowNum, color_matrixB, color_matrixA, ratio_AB, 8, 24, RGBt(1, 1, 1, 0), 1, 1, colorData);
}*/

void VarVisualTimeHistogramMeta::GetHistogramBitMap_period(int binNum, double* histogram, int maxNum)
{
    RGBt **color_matrixA, **color_matrixB;
    double ** ratio_AB;

    int rowNum, columnNum;
    columnNum = 7;
    rowNum = ceil(binNum / (double) columnNum);
    
    color_matrixA = new RGBt* [rowNum];
    color_matrixB = new RGBt* [rowNum];
    ratio_AB = new double* [rowNum];

    int i, j;
    for (i = 0; i < rowNum; i++)
    {
        color_matrixA[i] = new RGBt [columnNum];
        color_matrixB[i] = new RGBt [columnNum];
        ratio_AB[i] = new double [columnNum];
    }
    
    for(i = 0; i < rowNum; i++)
    {
        for (j = 0; j < columnNum; j++)
        {
            if (i*columnNum + j < binNum)
            {
                if (histogram[i*rowNum + j] == 0)
                {
                    color_matrixA[i][j] = RGBt(1,1, 1, 1);
                    color_matrixB[i][j] = RGBt(1, 1, 1, 1);
                    ratio_AB[i][j] = 0;
                }
                else
                {
                    int temp = histogram[i*columnNum + j] / (double) maxNum *9;
                    RGBt color = drawer->GetColorManager()->GetColorSetting(NUERIC, temp)->GetColor();
                    color_matrixA[i][j] = RGBt(1-color.R, 1-color.B, 1-color.B, color.A);
//                    color_matrixA[i][j] = colorSetting->GetColor(histogram[i*columnNum + j] / (double) maxNum); 
                    color_matrixB[i][j] = RGBt(1, 1, 1, 1);
                    ratio_AB[i][j] = 0;
                    //ratio_AB[i][j] = 1. - histogram[i*rowNum + j] / (double) maxNum;
                }
            }
            else
            {
                color_matrixA[i][j] = RGBt(1, 1, 1, 1);
                color_matrixB[i][j] = RGBt(1, 1, 1, 1);
                ratio_AB[i][j] = 0;
            }
        }
    }

    if (colorData == NULL)
    {
        GetUnitTextureSize(columnNum, rowNum, 8, 24, 1, 1, w, h);
        colorData = new float[w*h*4];
    }


    FillUnitTexture(columnNum, rowNum, color_matrixB, color_matrixA, ratio_AB, 8, 24, RGBt(1, 1, 1, 1), 1, 1, colorData);
}

void VarVisualTimeHistogramMeta::GetUnitTextureSize(int matrix_w, int matrix_h, int unit_w, int unit_h, int gap_w, int gap_h, int &total_w, int &total_h)
{
    total_w = matrix_w * unit_w + (matrix_w - 1) * gap_w;
    total_h = matrix_h * unit_h + (matrix_h - 1) * gap_h;
}
 
void VarVisualTimeHistogramMeta::FillUnitTexture(int matrix_w, int matrix_h, RGBt **color_matrixA, RGBt **color_matrixB, double **ratio_AB, int unit_w, int unit_h, RGBt background, int gap_w, int gap_h, float *pixels)
{
    int i, j;
    int total_w, total_h;
    int unit_h1, unit_h2;
    GetUnitTextureSize(matrix_w, matrix_h, unit_w, unit_h, gap_w, gap_h, total_w, total_h);
    for (i=0; i<matrix_h; i++)
    {
        for (j=0; j<matrix_w; j++)
        {
            if (color_matrixB == NULL || ratio_AB == NULL)
                FillRectangle(total_w, total_h, j * (unit_w + gap_w), i * (unit_h + gap_h), unit_w, unit_h, color_matrixA[i][j], pixels);
            else
            {
                unit_h1 = unit_h * ratio_AB[i][j];
                unit_h2 = unit_h - unit_h1;
                FillRectangle(total_w, total_h, j * (unit_w + gap_w), i * (unit_h + gap_h), unit_w, unit_h1, color_matrixA[i][j], pixels);
                FillRectangle(total_w, total_h, j * (unit_w + gap_w), i * (unit_h + gap_h) + unit_h1, unit_w, unit_h2, color_matrixB[i][j], pixels);
            }
        }
    }
    for (i=1; i<matrix_h; i++)
    {
        for (j=0; j<matrix_w; j++)
        {
            FillRectangle(total_w, total_h, j * (unit_w + gap_w), i * (unit_h + gap_h) - gap_h, unit_w, gap_h, background, pixels);
        }
    }
    for (i=0; i<matrix_h; i++)
    {
        for (j=1; j<matrix_w; j++)
        {
            FillRectangle(total_w, total_h, j * (unit_w + gap_w) - gap_w, i * (unit_h + gap_h), unit_w, gap_h, background, pixels);
        }
    }
    for (i=1; i<matrix_h; i++)
    {
        for (j=1; j<matrix_w; j++)
        {
            FillRectangle(total_w, total_h, j * (unit_w + gap_w) - gap_w, i * (unit_h + gap_h) - gap_h, gap_w, gap_h, background, pixels);
        }
    }
}
 
//private function
void VarVisualTimeHistogramMeta::FillRectangle(int total_w, int total_h, int left, int top, int w, int h, RGBt c, float *pixels)
{
    int i, j, x, y, pos;
    
    for (i=0; i<h; i++)
    {
        y = top + i;
        for (j=0; j<w; j++)
        {
            x = left + j;
            pos = y * total_w + x;
            pixels[4*pos] = c.R;
            pixels[4*pos+1] = c.G;
            pixels[4*pos+2] = c.B;
            pixels[4*pos+3] = c.A;
        }
    }
}
 
