//the main window of the system. 


#include "MainFrame.h"
#include "../Basic/macros.h"
#include "SBBFrame.h"
#include "../DataStructure/VideoPreProcess/VideoProcessFrame.h"
#include "../DataStructure/DocVideoShot.h"
#include "../DataStructure/VideoShotIterator.h"
#include "MainApplication.h"
#include "UserNotifier.h"

#include "ViewVideoWindow.h"
#include "ViewMultipleVideoWindow.h"


BEGIN_EVENT_TABLE(MainFrame, GLFrame)
    EVT_MENU(MENU_OPENFILE, MainFrame::OpenFile)
    EVT_MENU(MENU_SAVEFILE, MainFrame::SaveFile)
    EVT_MENU(MENU_NEWWINDOW, MainFrame::NewWin)
    EVT_MENU(MENU_VIEWPC, MainFrame::ViewPC)
    EVT_MENU(MENU_VIEWVAR, MainFrame::ViewVAR)
    EVT_MENU(MENU_VIEWGLYPH, MainFrame::ViewGlyph)
    EVT_MENU(MENU_VIEWDIMSTACK, MainFrame::ViewDimstack)
    EVT_MENU(MENU_VARCONTROL, MainFrame::OnShowVarControlFrame)
    EVT_MENU(MENU_IMAGECONTROL, MainFrame::OnShowImageControlFrame)
END_EVENT_TABLE()

MainFrame::MainFrame(const wxChar *title, int xpos, int ypos, int width, int height)
        :GLFrame((wxFrame *)NULL, -1, title, xpos, ypos, width, height)
{
	wxMenu *fileMenu = new wxMenu;
	fileMenu->Append(MENU_OPENFILE,  "&Open file");
	fileMenu->Append(MENU_SAVEFILE,  "&Save file");
	fileMenu->Append(MENU_NEWWINDOW,  "New &Window");
	menuBar->Insert(0, fileMenu, "&File");

    wxMenu *viewMenu = new wxMenu;

    viewMenu->AppendRadioItem(MENU_VIEWVAR, "VaR", "Show dataset in Value and Relation display");
    viewMenu->AppendRadioItem(MENU_VIEWPC, "Parallel Coordinates", "Show dataset in Parallel Coordinates");
    viewMenu->AppendRadioItem(MENU_VIEWGLYPH, "Image", "Show image data items");
    viewMenu->AppendRadioItem(MENU_VIEWDIMSTACK, "Dimensional Stacking", "Show dataset in dimensional stacking");

    viewMenu->AppendSeparator();

	viewMenu->Append(MENU_VARCONTROL,  "Var Control Frame");
	viewMenu->Append(MENU_IMAGECONTROL,  "Image Control Frame");

    menuBar->Insert(1, viewMenu, "&View");

    varFrame = NULL;
    mdsFrame = NULL;
}

MainFrame::~MainFrame()
{
}


void MainFrame::ResetControlFrames()
{
    if (varFrame != NULL)
        varFrame->Reset();
    if (mdsFrame != NULL)
        mdsFrame->Reset();
}

 
void MainFrame::OpenFile(wxCommandEvent& event)
{

// this is for wxwidget 2.7    wxFileDialog* dlg = new wxFileDialog(this, "Choose a file", "", "", 
//        "Flat_Data files (*.okc)|*.okc|Cluster_Data files (*.cg)|*.cg|Database_data files (*.db)|*.db", 
//        wxFD_OPEN | wxFD_CHANGE_DIR | wxFD_FILE_MUST_EXIST);

    wxFileDialog* dlg = new wxFileDialog(this);
    dlg->SetStyle(wxOPEN);
    dlg->SetWildcard("Flat_Data files (*.okc)|*.okc|Cluster_Data files (*.cg)|*.cg|Database_data files (*.db)|*.db");

    if (dlg->ShowModal() == wxID_CANCEL)
    {
        delete dlg;
        return;
    }

    std::string fileName = dlg->GetPath().c_str();

    UserNotifier::CreateProgressDialog("Open in progress", "Load dataset", 100);

    if (docManager->OpenDataSet(fileName))
    {
        UserNotifier::UpdateProgressDialog(50, "Update control frames");
        ResetControlFrames();
        UserNotifier::UpdateProgressDialog(60, "Refresh displays");
        viewManager->RefreshAll();
        UserNotifier::DeleteProgressDialog();
    }
    else
    {
        UserNotifier::DeleteProgressDialog();
        UserNotifier::ShowMessageDialog("Error Message", "File open failed");
    }

    delete dlg;
    return;
}

void MainFrame::SaveFile(wxCommandEvent& event)
{
    wxFileDialog* dlg = new wxFileDialog(this);
    dlg->SetStyle(wxSAVE|wxOVERWRITE_PROMPT);
    dlg->SetWildcard("okc files (*.okc)|*.okc");

    if (dlg->ShowModal() == wxID_CANCEL)
    {
        delete dlg;
        return;
    }

    std::string fileName = dlg->GetPath().c_str();

    docManager->WriteNewDataset(fileName);

}


void MainFrame::OnShowVarControlFrame(wxCommandEvent& event)
{
    ShowVarControlFrame();
}

void MainFrame::OnShowImageControlFrame(wxCommandEvent& event)
{
    ShowImageControlFrame();
}

void MainFrame::ShowVarControlFrame()
{
    if (varFrame == NULL)
    {
        varFrame = new VarControlFrame(this, FRAME_VAR, "VaR Control", 700, 0, 450, 900);
        varFrame->SetViewManager(viewManager);
        varFrame->SetDocumentManager(docManager);
        varFrame->Show(true);
        varFrame->Reset();
    }
    else
        varFrame->Show(true);
}

void MainFrame::ShowImageControlFrame()
{
    if (mdsFrame == NULL)
    {
        mdsFrame = new MDSControlFrame(this, FRAME_IMAGE, "Image Control", 700, 0, 450, 700);
        mdsFrame->SetViewManager(viewManager);
        mdsFrame->SetDocumentManager(docManager);
        mdsFrame->Show(true);
        mdsFrame->Reset();
    }
    else
        mdsFrame->Show(true);
}

void MainFrame::SetDisplayType(int type)
{
    viewManager->AssignDrawerToCanvas(GetCanvas(), type);

    if (type == VAR)
    {
        if (mdsFrame != NULL)
            mdsFrame->Show(false);
        ShowVarControlFrame();
    }
    else if (type == GLYPH)
    {
        if (varFrame != NULL)
            varFrame->Show(false);
        ShowImageControlFrame();
    }
    return;
}

void MainFrame::NewWin(wxCommandEvent& event)
{
	MainFrame* frame = new MainFrame("A new frame", 50, 50, 450, 300);
	frame->Show(TRUE);

 /*   wxString videoFile("C:\\yangjing\\Fan\\video\\multichannel\\Fox\\2006_07_17_X_01_52.mpg");
    ViewVideoWindow *videoWnd = new ViewVideoWindow(videoFile);
    videoWnd->Show(true);
    videoWnd->Play();*/

 /*   wxString videoFiles[8];
    wxString videoFile("C:\\yangjing\\Fan\\video\\multichannel\\Fox\\2006_07_17_X_01_52.mpg");
    wxString videoFile1("C:\\yangjing\\Fall2006\\course\\Videos\\infocanvas.mpg");
    for (int i = 0; i < 8; i++)
    {
        videoFiles[i] = videoFile;
    }
    videoFiles[0] = videoFile1;
    ViewMultipleVideoWindow *videoWnd = new ViewMultipleVideoWindow(8, videoFiles);
    videoWnd->Show(true);
    videoWnd->Play();*/

 //VideoProcessFrame* frame = new VideoProcessFrame("Video Processing", 0, 0, 300, 300, docManager->GetDBManager()); 
 // frame->Show(true);
}

void MainFrame::NewViewMultipleVideoWindow(int num, wxString *names, int *types, std::string title, std::string *captions, wxColour *captionColors)
{
/*    wxString names = new wxString [num];
    for (int i = 0; i < num; i++)
        names[i] = namearray[i].c_str();*/

    ViewMultipleVideoWindow *videoWnd = new ViewMultipleVideoWindow(num, names, types, title, captions, captionColors);
    videoWnd->Show(true);
    videoWnd->Play();
}


void MainFrame::ShowContiunsVideoPlay(VideoShotIterator *it)
{
/*   std::string globalPath = docManager->GetGlobalPath();
    ViewVideoWindow *videoWnd = new ViewVideoWindow(globalPath, it);
    videoWnd->Show(true);
    videoWnd->Play();*/
}

void MainFrame::ViewPC(wxCommandEvent& event)
{
    SetDisplayType(PARCOORD);
    GetCanvas()->Refresh();
}

void MainFrame::ViewVAR(wxCommandEvent& event)
{
    SetDisplayType(VAR);
    GetCanvas()->Refresh();
}

void MainFrame::ViewGlyph(wxCommandEvent& event)
{
    SetDisplayType(GLYPH);
    GetCanvas()->Refresh();
}

void MainFrame::ViewDimstack(wxCommandEvent& event)
{
    SetDisplayType(DIMSTACK);
    GetCanvas()->Refresh();
}


