//the main window of the system. 

#ifndef MAINFRAME_H
#define MAINFRAME_H

#include "GLFrame.h"
#include "VarControlFrame.h"
#include "MDSControlFrame.h"


class VideoShotIterator;
//ids of the interface items
enum   //use 201 - 299
{ 
    
	MENU_OPENFILE =     201,
    MENU_NEWWINDOW =    202,
    MENU_DISPLAYTYPE =  203,
    MENU_VIEWPC =       204,
    MENU_VIEWVAR =      205,
    MENU_VIEWGLYPH =    206,
    MENU_VIEWDIMSTACK = 207,
    FRAME_VAR =         208,
    FRAME_IMAGE =       209,

    MENU_VARCONTROL =   210,
    MENU_IMAGECONTROL = 211,

    MENU_SAVEFILE =     212,
};

//the main interface of the system
//It is in charge of all dialogs and frames. May change it in the future
//this is a frame with drawable canavs in it. so it is derived from GLFrame
class MainFrame : public GLFrame
{
protected:
    //the control frame for the var display
	VarControlFrame* varFrame;
    //the control frame for the image display
    MDSControlFrame* mdsFrame;
	
public:
	MainFrame(const wxChar *title, int xpos, int ypos, int width, int height);
	~MainFrame();
	
    //create a new window
	void NewWin(wxCommandEvent& event);
    //open a new dataset
    void OpenFile(wxCommandEvent& event);
    void SaveFile(wxCommandEvent& event);

    void SetDisplayType(int type);

    //set displays of different types
    void ViewPC(wxCommandEvent& event);
    void ViewVAR(wxCommandEvent& event);
    void ViewGlyph(wxCommandEvent& event);
    void ViewDimstack(wxCommandEvent& event);

    void OnShowVarControlFrame(wxCommandEvent& event);
    void OnShowImageControlFrame(wxCommandEvent& event);

    //update all the control frames
    void ResetControlFrames();

    VarControlFrame* GetVarControlFrame() {return varFrame;}
    MDSControlFrame* GetMDSControlFrame() {return mdsFrame;}
    void NewViewMultipleVideoWindow(int num, wxString *names, int *types, std::string title, std::string *captions, wxColour *captionColors);

    void SetVarFrame(VarControlFrame *frame) {varFrame = frame;}
    void SetImageFrame(MDSControlFrame *frame) {mdsFrame = frame;}

    void ShowVarControlFrame();
    void ShowImageControlFrame();
    void ShowContiunsVideoPlay(VideoShotIterator *it);

	DECLARE_EVENT_TABLE()
};


#endif

