#include <fstream>
#include "MediaCtrl.h"
#include "ViewMultipleVideoWindow.h"
#include "ViewVideoWindow.h"
#include "ViewCloseCaptionWindow.h"

BEGIN_EVENT_TABLE(MediaCtrl, wxMediaCtrl)
    EVT_LEFT_DOWN(MediaCtrl::OnLeftMouseDown)
    EVT_RIGHT_DOWN(MediaCtrl::OnRightMouseDown)
    EVT_MIDDLE_DOWN(MediaCtrl::OnMiddleMouseDown)
    EVT_LEFT_DCLICK(MediaCtrl::OnLeftMouseDoubleClick)
    EVT_MEDIA_STOP(wxID_ANY, MediaCtrl::OnMediaStop)

    EVT_MENU(MEDIACTRL_MENU_UNMUTE, MediaCtrl::OnMenuUnmute)
    EVT_MENU(MEDIACTRL_MENU_MUTE, MediaCtrl::OnMenuMute)
    EVT_MENU(MEDIACTRL_MENU_PAUSE, MediaCtrl::OnMenuPause)
    EVT_MENU(MEDIACTRL_MENU_PAUSE, MediaCtrl::OnMenuPlay)
    EVT_MENU(MEDIACTRL_MENU_STOP, MediaCtrl::OnMenuStop)
    EVT_MENU(MEDIACTRL_MENU_STOPALL, MediaCtrl::OnMenuStopAll)
    EVT_MENU(MEDIACTRL_MENU_PLAYALL, MediaCtrl::OnMenuPlayAll)
    EVT_MENU(MEDIACTRL_MENU_PAUSEALL, MediaCtrl::OnMenuPauseAll)
    EVT_MENU(MEDIACTRL_MENU_MUTEALL, MediaCtrl::OnMenuMuteAll)

    EVT_TIMER(TIMER_ID, MediaCtrl::OnTimer)

END_EVENT_TABLE()

MediaCtrl::MediaCtrl(wxWindow* parent, wxWindowID id, const wxString& fileName)
        :wxMediaCtrl(parent, id, fileName, wxDefaultPosition, wxDefaultSize, 0, wxMEDIABACKEND_DIRECTSHOW)
{
    this->startTime = 0;
    this->lastingTime = 0;
    this->fileName = fileName;
    menu = new wxMenu;
	menu->Append(MEDIACTRL_MENU_UNMUTE,  "UnMute");
	menu->Append(MEDIACTRL_MENU_MUTE,  "Mute");
    menu->Append(MEDIACTRL_MENU_PAUSE,  "Pause");
    menu->Append(MEDIACTRL_MENU_PAUSE,  "Play");
    menu->Append(MEDIACTRL_MENU_STOP,  "Stop");
    menu->Append(MEDIACTRL_MENU_PAUSEALL,  "Pause all");
    menu->Append(MEDIACTRL_MENU_PLAYALL,  "Play all");
    menu->Append(MEDIACTRL_MENU_STOPALL,  "Stop all");
    menu->Append(MEDIACTRL_MENU_MUTEALL,  "Mute all");

    timer = new wxTimer(this, TIMER_ID);

    std::string tip = GetFirstKeywordLine();
    SetToolTip(tip.c_str());
}

MediaCtrl::MediaCtrl(wxWindow* parent, wxWindowID id, const wxString& fileName, int startTime, int endTime)
        :wxMediaCtrl(parent, id, fileName, wxDefaultPosition, wxDefaultSize, 0, wxMEDIABACKEND_DIRECTSHOW)
{
    this->startTime = startTime;
    this->lastingTime = endTime - startTime;
    this->fileName = fileName;
    menu = new wxMenu;
	menu->Append(MEDIACTRL_MENU_UNMUTE,  "UnMute");
	menu->Append(MEDIACTRL_MENU_MUTE,  "Mute");
    menu->Append(MEDIACTRL_MENU_PAUSE,  "Pause");
    menu->Append(MEDIACTRL_MENU_PAUSE,  "Play");
    menu->Append(MEDIACTRL_MENU_STOP,  "Stop");
    menu->Append(MEDIACTRL_MENU_PAUSEALL,  "Pause all");
    menu->Append(MEDIACTRL_MENU_PLAYALL,  "Play all");
    menu->Append(MEDIACTRL_MENU_STOPALL,  "Stop all");
    menu->Append(MEDIACTRL_MENU_MUTEALL,  "Mute all");

    timer = new wxTimer(this, TIMER_ID);

}

MediaCtrl::~MediaCtrl()
{
    delete menu;
    delete timer;
}

void MediaCtrl::OnTimer(wxTimerEvent& event)
{
    Pause();
}


void MediaCtrl::Play()
{
    wxMediaCtrl::Play();
}

void MediaCtrl::Play(int startTime, int lastingTime)
{
    wxMediaCtrl::Play();
    if (lastingTime != 0)
    {
        Seek(startTime, wxFromStart);
        timer->Start(lastingTime, wxTIMER_ONE_SHOT);
    }
}


void MediaCtrl::OnLeftMouseDown(wxMouseEvent& event)
{
    ((ViewMultipleVideoWindow*)GetParent())->ResetCurrentMediaCtrl(this);
}

void MediaCtrl::OnRightMouseDown(wxMouseEvent& event)
{
    PopupMenu(menu);
}

void MediaCtrl::OnMiddleMouseDown(wxMouseEvent& event)
{
    std::string keyword = ((ViewMultipleVideoWindow*)GetParent())->GetKeyword();
    ViewCloseCaptionWindow* frame = new ViewCloseCaptionWindow(fileName.c_str(), keyword);
    frame->Show(true);
}


void MediaCtrl::OnLeftMouseDoubleClick(wxMouseEvent& event)
{
    ((ViewMultipleVideoWindow*)GetParent())->PauseAll();
    ViewVideoWindow *videoWnd = new ViewVideoWindow(fileName);
    videoWnd->Maximize(true);
    videoWnd->Show(true);
    videoWnd->Play();

    SetVolume(1e-5);
}

void MediaCtrl::OnMediaStop(wxMediaEvent& event)
{
    Stop();
}

void MediaCtrl::OnMenuUnmute(wxCommandEvent& event)
{
    SetVolume(1.0);
}

void MediaCtrl::OnMenuMute(wxCommandEvent& event)
{
    SetVolume(1e-5);
}

void MediaCtrl::OnMenuPause(wxCommandEvent& event)
{
    Pause();
}

void MediaCtrl::OnMenuPlay(wxCommandEvent& event)
{
    Play();
}

void MediaCtrl::OnMenuStop(wxCommandEvent& event)
{
    Stop();
}

void MediaCtrl::OnMenuStopAll(wxCommandEvent& event)
{
    ((ViewMultipleVideoWindow*)GetParent())->StopAll();
}

void MediaCtrl::OnMenuPauseAll(wxCommandEvent& event)
{
    ((ViewMultipleVideoWindow*)GetParent())->PauseAll();
}

void MediaCtrl::OnMenuPlayAll(wxCommandEvent& event)
{
    ((ViewMultipleVideoWindow*)GetParent())->PlayAll();
}

void MediaCtrl::OnMenuMuteAll(wxCommandEvent& event)
{
    ((ViewMultipleVideoWindow*)GetParent())->MuteAll();
}

std::string MediaCtrl::GetFirstKeywordLine()
{
    std::string keyword = ((ViewMultipleVideoWindow*)GetParent())->GetKeyword();
    
    std::string ccFile;
    std::string temp1 = fileName.c_str();
    ccFile = temp1.substr(0, temp1.find('.'));
    ccFile += ".cc";
    std::ifstream infile (ccFile.c_str());

    wxString temp = keyword.c_str();
    temp.Replace("_", " ");
    keyword = temp.c_str();

    char buf[1000];
    infile.getline(buf, 1000);
    wxString str;
    std::string tip = "";
    while (infile.good())
    {
        infile.getline(buf, 1000);
        str = buf; 
        std::string line = str.MakeLower().c_str();
        int pos = line.find(keyword.c_str()); 
        if (pos >= 0)
        {
            if (pos == 0 || !isalpha(str[pos-1]))
            {
                int pos2 = pos + keyword.length();
                if (pos2 == line.length() || !isalpha(line[pos2]))
                {
                    temp1 = str.c_str();
                    temp1.insert(pos2, "**");
                    temp1.insert(pos, "**");
                    temp1 = temp1.substr(46, str.length());
                    tip += temp1.c_str();
                    tip += "\n";
                }
            }
        }
    }
    return tip;
}