#include <wx/wx.h>
#include "Mediactrl.h"
#include "ViewMultipleVideoWindow.h"

BEGIN_EVENT_TABLE(ViewMultipleVideoWindow, wxFrame)
//    EVT_PAINT(ViewMultipleVideoWindow::OnPaint)
//    EVT_SIZE(ViewMultipleVideoWindow::OnSize)
END_EVENT_TABLE()

ViewMultipleVideoWindow::ViewMultipleVideoWindow(int num, wxString *videoFile, int *types, std::string title, std::string *captions, wxColour *captionColors) : 
    wxFrame(NULL, -1, videoFile[0], wxDefaultPosition, wxSize(500, 500))
{
    SetTitle(title.c_str());
    this->keyword = wxString(title.c_str()).MakeLower().c_str();
    this->captions = captions;
    this->captionColors = captionColors;
    this->num = num;
    this->videoFile = videoFile;
    this->types = types;
    int columns = 7;
    int rows = ceil(((double)num) / columns);

    sizer = new wxGridSizer(rows, columns, 1, 1);
	SetSizer( sizer );   

    mediaCtrls = new MediaCtrl* [num];
    for(int i = 0; i < num; i++)
    {
        if (videoFile[i].Cmp("") == 0)
        {
            wxStaticBox *datebox;
            datebox = new wxStaticBox(this, -1, captions[i].c_str(), wxPoint(-1,-1), wxSize(-1,-1),
                0, "staticBox");
            datebox->SetBackgroundColour(captionColors[i]);
            wxStaticBoxSizer *dateboxSizer = new wxStaticBoxSizer (datebox, wxVERTICAL);

            mediaCtrls[i] = NULL;
            wxStaticText *textLabel = new wxStaticText(this, -1, "");
            textLabel->SetForegroundColour(wxColour(0, 0, 0));
            dateboxSizer ->Add(textLabel, 1, wxEXPAND | wxALL, 0);
            sizer->Add(dateboxSizer, 1, wxEXPAND | wxALL, 0);
        }
        else
        {
            wxStaticBox *datebox;
            datebox = new wxStaticBox(this, -1, captions[i].c_str(), wxPoint(-1,-1), wxSize(-1,-1),
                0, "staticBox");
            datebox->SetBackgroundColour(captionColors[i]);
            wxStaticBoxSizer *dateboxSizer = new wxStaticBoxSizer (datebox, wxVERTICAL);
            mediaCtrls[i] = new MediaCtrl(this, i+1000, videoFile[i]);
            dateboxSizer ->Add(mediaCtrls[i], 1, wxEXPAND | wxALL, 0);
            sizer->Add(dateboxSizer , 1, wxEXPAND | wxALL, 0);
        }
    }

    int currentID = 0;
    while (mediaCtrls[currentID] == NULL)
        currentID++;

    if (currentID < num)
    {
        currMediaCtrl = mediaCtrls[currentID];
    }

    max = 15;
}

ViewMultipleVideoWindow::~ViewMultipleVideoWindow()
{
    delete [] mediaCtrls;
    delete [] captions;
    delete [] videoFile;
    delete [] types;
    delete [] captionColors;
}

/*
void ViewMultipleVideoWindow::Play()
{
    for(int i = 0; i < num; i++)
    {
        if (mediaCtrls[i] != NULL)
        {
            mediaCtrls[i]->Play();
            //wxMEDIABACKEND_DIRECTSHOW you have to use 1e-5 rather than 0. For others you cna use 0
            mediaCtrls[i]->SetVolume(1e-5);
        }
    }
    int j = 0;
    while (mediaCtrls[j] == NULL)
        j++;

    if (j < num)
    {
        mediaCtrls[j]->SetVolume(1);
        currMediaCtrl = mediaCtrls[j];
    }
}*/


void ViewMultipleVideoWindow::Play()
{
    for (int i = 0; i < num; i++)
    {
        if (mediaCtrls[i] != NULL && types[i] == 0)
        {
            mediaCtrls[i]->Play();
            //wxMEDIABACKEND_DIRECTSHOW you have to use 1e-5 rather than 0. For others you cna use 0
            mediaCtrls[i]->SetVolume(1e-5);
//            mediaCtrls[i]->SetPlaybackRate(2.0);
        }
    }

    count = 0;
    for(i = 0; i < num; i++)
    {
        if (mediaCtrls[i] != NULL && types[i] == 1)
        {
            if (count < max)
            {
                mediaCtrls[i]->Play();
                //wxMEDIABACKEND_DIRECTSHOW you have to use 1e-5 rather than 0. For others you cna use 0
                mediaCtrls[i]->SetVolume(1e-5);
//                mediaCtrls[i]->SetPlaybackRate(2.0);
                count ++;
            }
            else
            {
                mediaCtrls[i]->Play(0, 20000);
                //wxMEDIABACKEND_DIRECTSHOW you have to use 1e-5 rather than 0. For others you cna use 0
                mediaCtrls[i]->SetVolume(1e-5);
//                mediaCtrls[i]->SetPlaybackRate(2.0);
                count ++;
            }
        }
    }
 //   SetCurrentMediaCtrl();
}

void ViewMultipleVideoWindow::SetCurrentMediaCtrl()
{
    if (currMediaCtrl->GetState() != wxMEDIASTATE_PLAYING)
        currMediaCtrl->Play();
    currMediaCtrl->SetVolume(1);
//    currMediaCtrl->SetPlaybackRate(1.0);
}


bool ViewMultipleVideoWindow::ResetCurrentMediaCtrl(MediaCtrl *current)
{
    if (current == currMediaCtrl)
        return false;

    if (currMediaCtrl != NULL)
    {
        currMediaCtrl->SetVolume(1e-5);
//        currMediaCtrl->SetPlaybackRate(2.0);
    }
    currMediaCtrl = current;
    SetCurrentMediaCtrl();

    if (count < max)
        return true;
    else
    {
        int j = 0;
        while (mediaCtrls[j] != current)
            j++;

        int j1, j2;
        GetMustPlayRange(j, j1, j2);
        for(int i = 0; i < j1; i++)
        {
            if (types[i] == 1 && mediaCtrls[i]->GetState() == wxMEDIASTATE_PLAYING)
            {
                mediaCtrls[i]->Pause();
            }
        }
        for(i = j2 + 1; i < num; i++)
        {
            if (types[i] == 1 && mediaCtrls[i]->GetState() == wxMEDIASTATE_PLAYING)
            {
                mediaCtrls[i]->Pause();
            }
        }
        for (i = j1; i <= j2; i++)
        {
            if (i != j && types[i] == 1 && mediaCtrls[i]->GetState() != wxMEDIASTATE_PLAYING)
            {
                mediaCtrls[i]->Play();
                mediaCtrls[i]->SetVolume(1e-5);
//                mediaCtrls[i]->SetPlaybackRate(2.0);
            }
        }
    }
    return true;
}

void ViewMultipleVideoWindow::GetMustPlayRange(int j, int &j1, int &j2)
{
    int c = 0;
    if (types[j] == 1)
        c = 1;

    j1 = j; j2 = j;

    int dis1, dis2;
    int temp1, temp2;
    while (c < max)
    {
        temp1 = GetPrevPos(j1);
        temp2 = GetPostPos(j2);
        dis1 = j - temp1;
        dis2 = temp2 - j;
        if (dis1 < dis2)
        {
            if (temp1 >= 0)
                j1 = temp1;
            else if (temp2 < num) 
                j2 = temp2;
            else
                break;
        }
        else 
        {
            if (temp2 < num) 
                j2 = temp2;
            else if (temp1 >= 0)
                j1 = temp1;
            else
                break;
        }
        c++;
    }
}

int ViewMultipleVideoWindow::GetPrevPos(int j1)
{
    int temp = j1 - 1;
    while (temp >= 0 && types[temp] != 1)
        temp --;
    return temp;
}

int ViewMultipleVideoWindow::GetPostPos(int j1)
{
    int temp = j1 + 1;
    while (temp < num && types[temp] != 1)
        temp ++;
    return temp;
}

void ViewMultipleVideoWindow::MuteAll()
{
    for(int i = 0; i < num; i++)
    {
        if (mediaCtrls[i] != NULL)
        {
            //wxMEDIABACKEND_DIRECTSHOW you have to use 1e-5 rather than 0. For others you cna use 0
            mediaCtrls[i]->SetVolume(1e-5);
//            mediaCtrls[i]->SetPlaybackRate(2.0);
        }
    }
}

void ViewMultipleVideoWindow::PlayAll()
{
    for(int i = 0; i < num; i++)
    {
        if (mediaCtrls[i] != NULL)
        {
            //wxMEDIABACKEND_DIRECTSHOW you have to use 1e-5 rather than 0. For others you cna use 0
            mediaCtrls[i]->Play();
        }
    }
}

void ViewMultipleVideoWindow::PauseAll()
{
    for(int i = 0; i < num; i++)
    {
        if (mediaCtrls[i] != NULL)
        {
            //wxMEDIABACKEND_DIRECTSHOW you have to use 1e-5 rather than 0. For others you cna use 0
            mediaCtrls[i]->Pause();
        }
    }
}

void ViewMultipleVideoWindow::StopAll()
{
    for(int i = 0; i < num; i++)
    {
        if (mediaCtrls[i] != NULL)
        {
            //wxMEDIABACKEND_DIRECTSHOW you have to use 1e-5 rather than 0. For others you cna use 0
            mediaCtrls[i]->Stop();
        }
    }
}

