//canvas class defined by the system that allows the usage of all opengl command
//this class also privdes functions for drawing common shapes such as lines, circles, and others
//it keeps tracks of the texture objects created

#include <wx/wx.h>
#include <wx/image.h>
#include <wx/wfstream.h>
#include "grCanvas.h"
#include "GLFrame.h"
#include "../Basic/ARR.h"
#include "../Basic/macros.h"
#include "../ViewManager.h"

#define SLICES 2
#define SLICES2 1


BEGIN_EVENT_TABLE(grCanvas, wxGLCanvas)
    EVT_PAINT(grCanvas::OnPaint)
    EVT_SIZE(grCanvas::OnSize)
	EVT_LEFT_DOWN(grCanvas::OnLeftButtonDown) //Process a wxEVT_LEFT_DOWN event.  
	EVT_LEFT_UP(grCanvas::OnLeftButtonUp) //Process a wxEVT_LEFT_UP event.  
	EVT_LEFT_DCLICK(grCanvas::OnLeftButtonDoubleClick) //Process a wxEVT_LEFT_DCLICK event.  
	EVT_MIDDLE_DOWN(grCanvas::OnMiddleButtonDown) //Process a wxEVT_MIDDLE_DOWN event.  
	EVT_MIDDLE_UP(grCanvas::OnMiddleButtonUp) //Process a wxEVT_MIDDLE_UP event.  
	EVT_MIDDLE_DCLICK(grCanvas::OnMiddleButtonDoubleClick) //Process a wxEVT_MIDDLE_DCLICK event.  
	EVT_RIGHT_DOWN(grCanvas::OnRightButtonDown) //Process a wxEVT_RIGHT_DOWN event.  
	EVT_RIGHT_UP(grCanvas::OnRightButtonUp) //Process a wxEVT_RIGHT_UP event.  
	EVT_RIGHT_DCLICK(grCanvas::OnRightButtonDoubleClick) //Process a wxEVT_RIGHT_DCLICK event.  
	EVT_MOTION(grCanvas::OnMouseMotion) //Process a wxEVT_MOTION event.  */
//    EVT_KEY_DOWN(grCanvas::OnKeyPress)
//    EVT_KEY_UP(grCanvas::OnKeyRelease)
END_EVENT_TABLE()

grCanvas::grCanvas(wxWindow* parent, wxWindowID id, const wxPoint& pos, 
				   const wxSize& size, long style, const wxString& name)
	:wxGLCanvas(parent, id, pos, size, style, name)
{
    viewManager = NULL;
    zoom.X = 1;
    zoom.Y = 1;
    translate.X = 0;
    translate.Y = 0;

    // Setup some default values
    fg_color = RGBt(1.0, 0.5, 0.5);
    bg_color = RGBt(1.0, 1.0, 1.0);
    xxor = false;
    translucent = false;
    alpha = 1.0;
    clear_bits = GL_COLOR_BUFFER_BIT;
    origin_at_lower_left = false;

    shiftkey = false;
    controlkey = false;
    altkey = false;
    capitalkey = false;

    viewPortType = VAR;

    textureHashs[0] = varHash;
    textureHashs[1] = glyphHash;
    textureHashs[2] = dimstackHash;
    textureHashs[3] = timeHistogramHash;

}

grCanvas::~grCanvas()
{
    DeleteTextTextures();
}

void grCanvas::SetViewManager(ViewManager* viewManager)
{
    this->viewManager = viewManager;
}

void grCanvas::SetupTransformMatrix()
{
    SetCurrent();
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glScalef((float) zoom.X, (float) zoom.Y,1.0);
    glTranslatef((float) translate.X, (float) translate.Y,1.0);
}


void grCanvas::SetViewportInt()
{
    int W, H;
    GetClientSize(&W, &H);
#ifndef __WXMOTIF__
    if (GetContext())
#endif
    {
		SetCurrent();
		glViewport( 0, 0, W, H );
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		if (origin_at_lower_left) {
			glOrtho(0,W,0,H,-1,1);
		} else {
			glOrtho(0,W,H,0,-1,1);
		}
		glMatrixMode(GL_MODELVIEW);
	}
}

/*
void grCanvas::SetViewportInt()
{
    int W, H;
    GetClientSize(&W, &H);
#ifndef __WXMOTIF__
    if (GetContext())
#endif
    {
		SetCurrent();
		glViewport( 0, 0, W, H );
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();

        if (viewPortType == VAR)
        {
            int bor = 0;
		    if (origin_at_lower_left) {
			    glOrtho(-bor,W+bor,-bor,H+bor,-1,1);
		    } else {
			    glOrtho(-bor,W+bor,H+bor,-bor,-1,1);
		    }
        }
        else
        {
            int bor = 0;
 		    if (origin_at_lower_left) {
			    glOrtho(-bor,W+bor,-bor,H+bor*4,-1,1);
		    } else {
			    glOrtho(-bor,W+bor,H+bor * 4,-bor,-1,1);
		    }
        }

		glMatrixMode(GL_MODELVIEW);
	}
}*/

/*
void grCanvas::CreateTextTexture(const char* str, RGBt bgColor, RGBt fgColor,
                                   GLuint &texture, int &width, int &height, 
                                   int &width1, int &height1)
{
    //create a memory DC so that the text can be written on the DC. 
    wxMemoryDC dc;
    
    int i, j, k;

    dc.SetFont(wxFont(12, wxSWISS, wxNORMAL, wxBOLD));
    dc.GetTextExtent(str, &width1, &height1);
   
    //decide the size of the DC
    i = 1; 
    while (i < width1)
    {
        i *= 2;
    }
    width = i;

    i = 1; 
    while (i < height1)
    {
        i *= 2;
    }
    height = i;
    
    wxBitmap map(width, height); 
    dc.SelectObject(map);

    //set the background and front color of the text
    wxColor bcolor(bgColor.R * 255, bgColor.G * 255, bgColor.B * 255);
    wxColor fcolor(fgColor.R * 255, fgColor.G * 255, fgColor.B * 255);
    dc.SetBackgroundMode(wxSOLID);
    dc.SetTextBackground(bcolor);
    dc.SetTextForeground(fcolor);

    dc.Clear();
    //write the text into the DC
    dc.DrawText(str, 0, 0);
	
    //copy the pixel info of the Dc into an array named data
    float * data = new float[4*width*height];

    wxColor color;
    for (j = 0; j < height1; j++)
    {
        for (i = 0; i < width1; i++)
        {
            k = j*width*4+i*4;
            dc.GetPixel(i, j, &color);
            data[k+0] = color.Red()/255.;
            data[k+1] = color.Green()/255.;
            data[k+2] = color.Blue()/255.;
            data[k+3] = 1;
        }
    }

    for (j = 0; j < height1; j++)
    {
        for (i = width1; i < width; i++)
        {
            k = j*width*4+i*4;
            data[k+0] = bgColor.R;
            data[k+1] = bgColor.G;
            data[k+2] = bgColor.B;
            data[k+3] = 0;
        }
    }

    for (j = height1; j < height; j++)
    {
        for (i = 0; i < width; i++)
        {
            k = j*width*4+i*4;
            data[k+0] = bgColor.R;
            data[k+1] = bgColor.G;
            data[k+2] = bgColor.B;
            data[k+3] = 0;
        }
    }

    CreateTexture(width, height, data, texture);

    delete [] data;
}
*/

void grCanvas::CreateTextTexture(const char* str, RGBt bgColor, RGBt fgColor,
                                   GLuint &texture, int &width, int &height, 
                                   int &width1, int &height1)
{
    //create a memory DC so that the text can be written on the DC. 
    wxMemoryDC dc;
    
    int i, j, k;

    dc.SetFont(wxFont(12, wxSWISS, wxNORMAL, wxBOLD));
    dc.GetTextExtent(str, &width1, &height1);
   
    //decide the size of the DC
    i = 1; 
    while (i < width1)
    {
        i *= 2;
    }
    width = i;

    i = 1; 
    while (i < height1)
    {
        i *= 2;
    }
    height = i;
    
    wxBitmap map(width, height); 
    dc.SelectObject(map);

    //set the background and front color of the text
    wxColor bcolor(200, 0, 0);
    wxColor fcolor(0, 0, 0);
    dc.SetBackgroundMode(wxSOLID);
    dc.SetTextBackground(bcolor);
    dc.SetTextForeground(fcolor);

    dc.Clear();
    //write the text into the DC
    dc.DrawText(str, 0, 0);
	
    //copy the pixel info of the Dc into an array named data
    float * data = new float[4*width*height];

    wxColor color;
    for (j = 0; j < height1; j++)
    {
        for (i = 0; i < width1; i++)
        {
            k = j*width*4+i*4;
            dc.GetPixel(i, j, &color);
            if (color.Red() > 100) //background
            {
                data[k+0] = 1.;
                data[k+1] = 1.;
                data[k+2] = 1.;
                data[k+3] = 0.;
            }
            else
            {
                data[k+0] = 0.;
                data[k+1] = 0.;
                data[k+2] = 0.;
                data[k+3] = 1.;
            }
        }
    }

    for (j = 0; j < height1; j++)
    {
        for (i = width1; i < width; i++)
        {
            k = j*width*4+i*4;
            data[k+0] = bgColor.R;
            data[k+1] = bgColor.G;
            data[k+2] = bgColor.B;
            data[k+3] = 0;
        }
    }

    for (j = height1; j < height; j++)
    {
        for (i = 0; i < width; i++)
        {
            k = j*width*4+i*4;
            data[k+0] = bgColor.R;
            data[k+1] = bgColor.G;
            data[k+2] = bgColor.B;
            data[k+3] = 0;
        }
    }

    CreateTexture(width, height, data, texture);

    delete [] data;
}


void grCanvas::CreateTexture(int width, int height, float* data, GLuint &texture)
{
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

	// allocate a name for a texture object
	glGenTextures(1, &texture);

//    glColor4f(1.0, 1.0, 1.0, 1);
    glEnable( GL_TEXTURE_2D );
    glBindTexture( GL_TEXTURE_2D, texture);
//	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
	// when texture area is small, bilinear filter the closest mipmap
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
					 GL_LINEAR_MIPMAP_NEAREST );
	// when texture area is large, bilinear filter the original
//	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	// the texture wraps over at the edges (repeat)
//	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
//	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );

    // build the texture 
    //it is at the highest precision if the texture
    //is displayed at its orignal size
/*GLenum glErr;
int errCode;
glErr = glGetError();
while (glErr != GL_NO_ERROR) {
	if (glErr == GL_NO_ERROR) {
		errCode = 0;
	} else {
		errCode = 1;
	}
	glErr = glGetError();
}
glErr = glGetError();
if (glErr == GL_NO_ERROR) {
	errCode = 0;
} else {
	errCode = 1;
}*/
    gluBuild2DMipmaps( GL_TEXTURE_2D, 4, width, height,
                               GL_RGBA, GL_FLOAT, data );
   // glTexImage2D( GL_TEXTURE_2D, 0, 4, width, height, 0, GL_RGBA, GL_FLOAT, data);
/*glErr = glGetError();
if (glErr == GL_NO_ERROR) {
	errCode = 0;
} else {
	errCode = 1;
}*/
    glDisable( GL_TEXTURE_2D );

}


void grCanvas::GetTextTexture(std::string str, GLuint &texture, int &width, int &height, int &width1, int &height1)
{
    TextHash::iterator it;
    wxString wStr(str.c_str(), str.length());
    it = textHash.find(wStr);
    if (it == textHash.end())
    {
        RGBt color1 = bg_color;
        RGBt color2 = fg_color;
        
        CreateTextTexture(str.c_str(), color1, color2, texture, width, height, width1, height1);
        textHash[wStr] = grTexture(texture, width, height, width1, height1);
    }
    else
    {
        grTexture tex = it->second;
        texture = tex.id;
        width = tex.width;
        height = tex.height;
        width1 = tex.width1;
        height1 = tex.height1;
    }
}


void grCanvas::DrawTextTexture(GLuint texture, int width, int height, int width1, int height1,
                               int leftX, int bottomY, float angle, float scale)
{
    SetCurrent();

    //in pixels rather than 0-1
    SetViewportInt();


    float x, y;
    x = (float)width1/width;
    y = (float)height1/height;

    if (origin_at_lower_left)
    {
        glMatrixMode(GL_MODELVIEW);
        glPushMatrix();
        glLoadIdentity();

        glTranslatef(leftX, bottomY, 0);
        glRotated(angle, 0, 0, 1);
        glScalef(scale, scale, scale);
        glTranslatef(-1 * leftX, -1 * bottomY, 0);

        glEnable( GL_TEXTURE_2D );
        glBindTexture( GL_TEXTURE_2D, texture);

        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex2d(leftX, bottomY);
	    glTexCoord2d(x,y); glVertex2d(leftX + width1, bottomY);
	    glTexCoord2d(x,0.0); glVertex2d(leftX + width1, bottomY + height1);
	    glTexCoord2d(0.0,0.0); glVertex2d(leftX, bottomY + height1);
	    glEnd();
    }
    else
    {
        int w, h;
        GetClientSize(&w, &h);

        glMatrixMode(GL_MODELVIEW);
        glPushMatrix();
        glLoadIdentity();

        glTranslatef(leftX, h - bottomY, 0);
        glRotated(angle, 0, 0, 1);
        glScalef(scale, scale, scale);
        glTranslatef(-leftX, -h + bottomY, 0);

        glEnable( GL_TEXTURE_2D );
        glBindTexture( GL_TEXTURE_2D, texture);

        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex2d(leftX, h-bottomY);
	    glTexCoord2d(x,y); glVertex2d(leftX + width1, h-bottomY);
	    glTexCoord2d(x,0.); glVertex2d(leftX + width1, h-bottomY - height1);
	    glTexCoord2d(0.0,0.); glVertex2d(leftX, h-bottomY - height1);
	    glEnd();
    }
    glDisable( GL_TEXTURE_2D );

    glPopMatrix();

    SetViewport();
}

/*
void grCanvas::DrawTextTexture(GLuint texture, int width, int height, int width1, int height1,
                               int leftX, int bottomY, float angle, float scale)
{
    SetCurrent();

    //in pixels rather than 0-1
    SetViewportInt();

    glEnable( GL_TEXTURE_2D );
    glBindTexture( GL_TEXTURE_2D, texture);

    float x, y;
    x = (float)width1/width;
    y = (float)height1/height;
    if (origin_at_lower_left)
    {
        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex2d(0, 0);
	    glTexCoord2d(x,y); glVertex2d(width1, 0);
	    glTexCoord2d(x,0.0); glVertex2d(width1, height1);
	    glTexCoord2d(0.0,0.0); glVertex2d(0, height1);
	    glEnd();
    }
    else
    {
        int w, h;
        GetClientSize(&w, &h);

        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex2d(0, 0);
	    glTexCoord2d(x,y); glVertex2d(width1, 0);
	    glTexCoord2d(x,0.); glVertex2d(width1, - height1);
	    glTexCoord2d(0.0,0.); glVertex2d(0, - height1);
	    glEnd();
    }
    glDisable( GL_TEXTURE_2D );

    SetViewport();
}*/

/*
void grCanvas::DrawTextTexture(GLuint texture, int width, int height, int width1, int height1,
                               int leftX, int bottomY, float z, float angle, float scale)
{
    SetCurrent();

    //in pixels rather than 0-1
    SetViewportInt();

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();

    glTranslatef(leftX, bottomY, 0);
    glRotated(angle, 0, 0, 1);
    glScalef(scale, scale, scale);
    glTranslatef(-1 * leftX, -1 * bottomY, 0);

    glEnable( GL_TEXTURE_2D );
    glBindTexture( GL_TEXTURE_2D, texture);

    float x, y;
    x = (float)width1/width;
    y = (float)height1/height;
    if (origin_at_lower_left)
    {
        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex3d(leftX, bottomY, z);
	    glTexCoord2d(x,y); glVertex3d(leftX + width1, bottomY, z);
	    glTexCoord2d(x,0.0); glVertex3d(leftX + width1, bottomY + height1, z);
	    glTexCoord2d(0.0,0.0); glVertex3d(leftX, bottomY + height1, z);
	    glEnd();
    }
    else
    {
        int w, h;
        GetClientSize(&w, &h);

        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex3d(leftX, h-bottomY, z);
	    glTexCoord2d(x,y); glVertex3d(leftX + width1, h-bottomY, z);
	    glTexCoord2d(x,0.); glVertex3d(leftX + width1, h-bottomY - height1, z);
	    glTexCoord2d(0.0,0.); glVertex3d(leftX, h-bottomY - height1, z);
	    glEnd();
    }
    glDisable( GL_TEXTURE_2D );

    glPopMatrix();

    SetViewport();
}*/

//good one
void grCanvas::DrawTextTexture(GLuint texture, int width, int height, int width1, int height1,
                               float leftX, float bottomY, float z, float angle, float scale)
{
/*
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();

    glTranslatef(leftX, bottomY, 0);
    glRotated(angle, 0, 0, 1);
    glScalef(scale, scale, scale);
    glTranslatef(-leftX, -bottomY, 0);*/

    glEnable( GL_TEXTURE_2D );
    glBindTexture( GL_TEXTURE_2D, texture);

//    glEnable(GL_DEPTH_TEST);
    glClear(GL_DEPTH_BUFFER_BIT);

    float x, y;
    x = (float)width1/width;
    y = (float)height1/height;

    Vec2 vec = MapScreeVectorToDataVector(Vec2(width1, height1));

    if (origin_at_lower_left)
    {
        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex3d(leftX, bottomY, z);
	    glTexCoord2d(x,y); glVertex3d(leftX + vec.X, bottomY, z);
	    glTexCoord2d(x,0.0); glVertex3d(leftX + vec.X, bottomY + vec.Y, z);
	    glTexCoord2d(0.0,0.0); glVertex3d(leftX, bottomY + vec.Y, z);
	    glEnd();
    }
    else
    {
//        int w, h;
//        GetClientSize(&w, &h);
        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex3d(leftX, bottomY, z);
	    glTexCoord2d(x,y); glVertex3d(leftX + vec.X, bottomY, z);
	    glTexCoord2d(x,0.); glVertex3d(leftX + vec.X, bottomY + vec.Y, z);
	    glTexCoord2d(0.0,0.); glVertex3d(leftX, bottomY + vec.Y, z);
	    glEnd();
    }
    glDisable( GL_TEXTURE_2D );

//    glPopMatrix();
}

/*bad
void grCanvas::DrawTextTexture(GLuint texture, int width, int height, int width1, int height1,
                               float leftX, float bottomY, float angle, float scale)
{
    SetViewport();
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();

    glTranslatef(leftX, bottomY, 0);
    glRotated(angle, 0, 0, 1);
    glScalef(scale, scale, scale);
    glTranslatef(-leftX, -bottomY, 0);

    glEnable( GL_TEXTURE_2D );
    glBindTexture( GL_TEXTURE_2D, texture);

    float x, y;
    x = (float)width1/width;
    y = (float)height1/height;

    Vec2 vec = MapScreeVectorToDataVector(Vec2(width1, height1));

    if (origin_at_lower_left)
    {
        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex2d(leftX, bottomY);
	    glTexCoord2d(x,y); glVertex2d(leftX + vec.X, bottomY);
	    glTexCoord2d(x,0.0); glVertex2d(leftX + vec.X, bottomY + vec.Y);
	    glTexCoord2d(0.0,0.0); glVertex2d(leftX, bottomY + vec.Y);
	    glEnd();
    }
    else
    {
//        int w, h;
//        GetClientSize(&w, &h);
        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex2d(leftX, bottomY);
	    glTexCoord2d(x,y); glVertex2d(leftX + vec.X, bottomY);
	    glTexCoord2d(x,0.); glVertex2d(leftX + vec.X, bottomY + vec.Y);
	    glTexCoord2d(0.0,0.); glVertex2d(leftX, bottomY + vec.Y);
	    glEnd();
    }
    glDisable( GL_TEXTURE_2D );

    glPopMatrix();
    SetViewport();
}*/

/*
void grCanvas::DrawTextTexture(GLuint texture, int width, int height, int width1, int height1,
                               float leftX, float bottomY, float z, float angle, float scale)
{
    SetViewport();
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();

    glTranslatef(leftX, bottomY, 0);
    glRotated(145, 0, 0, 1);
    glScalef(scale, scale, scale);
    glTranslatef(-leftX, -bottomY, 0);

    glEnable( GL_TEXTURE_2D );
    glBindTexture( GL_TEXTURE_2D, texture);

//    glEnable(GL_DEPTH_TEST);
//    glClear(GL_DEPTH_BUFFER_BIT);
    glDisable(GL_DEPTH_TEST);

    float x, y;
    x = (float)width1/width;
    y = (float)height1/height;

    Vec2 vec = MapScreeVectorToDataVector(Vec2(width1, height1));

    if (origin_at_lower_left)
    {
        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex2d(leftX, bottomY);
	    glTexCoord2d(x,y); glVertex2d(leftX + vec.X, bottomY);
	    glTexCoord2d(x,0.0); glVertex2d(leftX + vec.X, bottomY + vec.Y);
	    glTexCoord2d(0.0,0.0); glVertex2d(leftX, bottomY + vec.Y);
	    glEnd();
    }
    else
    {
//        int w, h;
//        GetClientSize(&w, &h);
        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex2d(leftX, bottomY);
	    glTexCoord2d(x,y); glVertex2d(leftX + vec.X, bottomY);
	    glTexCoord2d(x,0.); glVertex2d(leftX + vec.X, bottomY + vec.Y);
	    glTexCoord2d(0.0,0.); glVertex2d(leftX, bottomY + vec.Y);
	    glEnd();
    }
    glDisable( GL_TEXTURE_2D );

    glPopMatrix();
    SetViewport();
}*/

void grCanvas::DeleteTextTextures()
{
    TextHash::iterator it;
    it = textHash.begin();
    while (it != textHash.end())
    {
        glDeleteTextures(1, &(it->second.id));
        it++;
    }
}

Vec2 grCanvas::MapScreeVectorToDataVector(const Vec2 p)
{
    Vec2 p1, p2;
    p1 = MapScreenToData(p);
    p2 = MapScreenToData(Vec2(0, 0));
    return Vec2(p1.X - p2.X, p1.Y - p2.Y);
}

Vec2 grCanvas::GetMousePositionInDataSpace(wxMouseEvent event)
{
    int x = event.GetX();
    int y = event.GetY();
    SetupTransformMatrix();
    Vec2 temp = MapScreenToData( Vec2(x,y) );
    float x1 = 1/zoom.Y - temp.Y;
    float y1 = -1 * translate.Y;
    x1 = x1 + y1 * 2;
//    return Vec2(temp.X, 1/zoom.Y-temp.Y);
    return Vec2(temp.X, x1);
}

void grCanvas::BeginDraw()
{
    SetCurrent();

    if (xxor == true)
    {
        // Setup XOR drawing mode
        glEnable(GL_COLOR_LOGIC_OP);
        glLogicOp(GL_INVERT);
        glDrawBuffer(GL_FRONT);
    } else {
        glColor4f((float) fg_color.R, (float) fg_color.G, (float) fg_color.B, (float) alpha );
    }

    if (translucent == true) 
    {
        // Turn on Translucency
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    }
}

void grCanvas::SetTranslucent(bool trans)
{
    translucent = trans;
    if (translucent == true) 
    {
        // Turn on Translucency
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    }
    else
    {
        glDisable(GL_BLEND);

    }
}

void grCanvas::EndDraw()
{
    SetCurrent();

    if (xxor == true) 
    {
        glDisable(GL_COLOR_LOGIC_OP);
        glDrawBuffer(GL_BACK);
    }
    if (translucent == true) 
    {
        glDisable(GL_BLEND);
    }
}

//------------ event handlers ------------------------------------------
void grCanvas::OnSize(wxSizeEvent& event)
{
    // this is also necessary to update the context on some platforms
    wxGLCanvas::OnSize(event);
//    Refresh();
}


void grCanvas::OnPaint(wxPaintEvent& event)
{
    wxPaintDC dc(this);

#ifndef __WXMOTIF__
    if (!GetContext()) return;
#endif
    Refresh();
}

void grCanvas::CheckAltShiftControl(wxMouseEvent &event)
{
    if (event.ShiftDown())
        shiftkey = true;
    else
        shiftkey = false;

    if (event.AltDown())
        altkey = true;
    else
        altkey = false;
    
    if (event.ControlDown())
        controlkey = true;
    else
        controlkey = false;
}

//Process a wxEVT_LEFT_DOWN event.  
void grCanvas::OnLeftButtonDown(wxMouseEvent &event) 
{
    if (viewManager == NULL)
        return;

    CheckAltShiftControl(event);
    Vec2 p = GetMousePositionInDataSpace(event);
    viewManager->ButtonDown(this, 0, p);
}

//Process a wxEVT_LEFT_UP event.  
void grCanvas::OnLeftButtonUp(wxMouseEvent &event) 
{
    if (viewManager == NULL)
        return;

//    CheckAltShiftControl(event);
    Vec2 p = GetMousePositionInDataSpace(event);
    viewManager->ButtonUp(this, 0, p);
}


//Process a wxEVT_LEFT_DCLICK event.
void grCanvas::OnLeftButtonDoubleClick(wxMouseEvent &event) 
{
    if (viewManager == NULL)
        return;

    CheckAltShiftControl(event);
    Vec2 p = GetMousePositionInDataSpace(event);
    viewManager->DoubleClick(this, 0, p);
}

//Process a wxEVT_MIDDLE_DOWN event. 
void grCanvas::OnMiddleButtonDown(wxMouseEvent &event) 
{
    if (viewManager == NULL)
        return;

    CheckAltShiftControl(event);
    Vec2 p = GetMousePositionInDataSpace(event);
    viewManager->ButtonDown(this, 1, p);
}

//Process a wxEVT_MIDDLE_UP event. 
void grCanvas::OnMiddleButtonUp(wxMouseEvent &event) 
{
    if (viewManager == NULL)
        return;

    Vec2 p = GetMousePositionInDataSpace(event);
    viewManager->ButtonUp(this, 1, p);
}

//Process a wxEVT_MIDDLE_DCLICK event. 
void grCanvas::OnMiddleButtonDoubleClick(wxMouseEvent &event) 
{
    if (viewManager == NULL)
        return;

    CheckAltShiftControl(event);
    Vec2 p = GetMousePositionInDataSpace(event);
    viewManager->DoubleClick(this, 1, p);
}

//Process a wxEVT_MIDDLE_DCLICK event. 
void grCanvas::OnRightButtonDown(wxMouseEvent &event) 
{
    if (viewManager == NULL)
        return;

    CheckAltShiftControl(event);
    Vec2 p = GetMousePositionInDataSpace(event);
    viewManager->ButtonDown(this, 2, p);
}

//Process a wxEVT_RIGHT_UP event. 
void grCanvas::OnRightButtonUp(wxMouseEvent &event) 
{
    if (viewManager == NULL)
        return;

    Vec2 p = GetMousePositionInDataSpace(event);
    viewManager->ButtonUp(this, 2, p);
}

//Process a wxEVT_RIGHT_DCLICK event. 
void grCanvas::OnRightButtonDoubleClick(wxMouseEvent &event) 
{
   if (viewManager == NULL)
        return;

    CheckAltShiftControl(event);
    Vec2 p = GetMousePositionInDataSpace(event);
    viewManager->DoubleClick(this, 2, p);
}

//Process a wxEVT_MOTION event. 
void grCanvas::OnMouseMotion(wxMouseEvent &event) 
{
    if (viewManager == NULL)
        return;

    Vec2 p = GetMousePositionInDataSpace(event);
    viewManager->MouseMotion(this, p);
}

/* it is abnormal. bugs in wxwindows?
void grCanvas::OnKeyPress(wxKeyEvent &event)
{
    if (viewManager == NULL)
        return;
    viewManager->KeyPress(this, event.GetKeyCode());
}

void grCanvas::OnKeyRelease(wxKeyEvent &event)
{
    if (viewManager == NULL)
        return;
    viewManager->KeyPress(this, event.GetKeyCode());
}*/

//---------public functions----------------------------------------
void grCanvas::SetViewport()
{
    int W, H;
    GetClientSize(&W, &H);
#ifndef __WXMOTIF__
    if (GetContext())
#endif
    {
		SetCurrent();
		glViewport( 0, 0, W, H );
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();

        if (viewPortType == VAR)
        {
            float bor = 30;
            float wBor = bor/W;
            float hBor = bor/H;
            const double BORD = 0.004;
		    if (origin_at_lower_left) {
			    glOrtho(-wBor,1+wBor,-hBor,1+hBor,-1,1);
		    } else {
			    glOrtho(-wBor,1+wBor,1+hBor,-hBor,-1,1);
		    }
        }
        else
        {
            float bor = 30;
            float wBor = bor/W;
            float hBor = bor/H;
            const double BORD = 0.004;
		    if (origin_at_lower_left) {
			    glOrtho(-wBor,1+wBor,-hBor,1+hBor*4,-1,1);
		    } else {
			    glOrtho(-wBor,1+wBor,1+hBor * 4,-hBor,-1,1);
		    }
        }

		glMatrixMode(GL_MODELVIEW);
	}
}

void grCanvas::SetViewPortMode(int type)
{
    viewPortType = type;
}

void grCanvas::SetLowerLeftOrigin()
{
    origin_at_lower_left = true;
}

void grCanvas::SetTopLeftOrigin()
{
    origin_at_lower_left = false;
}


Vec2 grCanvas::MapScreenToData( const Vec2 &sc )
{
    GLdouble modelMatrix[16], projMatrix[16];
    GLint viewport[4];
    glGetDoublev(GL_MODELVIEW_MATRIX,modelMatrix);
    glGetDoublev(GL_PROJECTION_MATRIX,projMatrix);
    glGetIntegerv(GL_VIEWPORT,viewport);

    double wc[3];
    int ret = gluUnProject(sc.X,sc.Y,0,modelMatrix,projMatrix,viewport,wc,wc+1,wc+2);

    return Vec2(wc[0],wc[1]);
}

Vec2 grCanvas::MapDataToScreen( const Vec2 &p )
{
    GLdouble modelMatrix[16], projMatrix[16];
    GLint viewport[4];
    glGetDoublev(GL_MODELVIEW_MATRIX,modelMatrix);
    glGetDoublev(GL_PROJECTION_MATRIX,projMatrix);
    glGetIntegerv(GL_VIEWPORT,viewport);

    double sc[3];
    int ret = gluProject(p.X,p.Y,0,modelMatrix,projMatrix,viewport,sc,sc+1,sc+2);
    return Vec2(sc[0],sc[1]);
}

Vec3 grCanvas::MapDataToScreen( const Vec3 &p )
{
    GLdouble modelMatrix[16], projMatrix[16];
    GLint viewport[4];
    glGetDoublev(GL_MODELVIEW_MATRIX,modelMatrix);
    glGetDoublev(GL_PROJECTION_MATRIX,projMatrix);
    glGetIntegerv(GL_VIEWPORT,viewport);

    double sc[3];
    int ret = gluProject(p.X,p.Y,p.Z,modelMatrix,projMatrix,viewport,sc,sc+1,sc+2);
    return Vec3(sc[0],sc[1],sc[2]);
}

//get current x translate (for panning)
float grCanvas::GetXTranslate()
{
    return translate.X;
}

//get current y translate (for panning)
float grCanvas::GetYTranslate()
{
    return translate.Y;
}

//get current zoom in x coordinate
float grCanvas::GetXZoom()
{
    return zoom.X;
}

//get current zoom in y coordinate
float grCanvas::GetYZoom()
{
    return zoom.Y;
}

//set current x translate (for panning)
void grCanvas::SetXTranslate(float value)
{
    translate.X = value;
//    Refresh();
}

//set current y translate (for panning)
void grCanvas::SetYTranslate(float value)
{
    translate.Y = value;
//    Refresh();
}

//set current zoom in x coordinate
void grCanvas::SetXZoom(float value)
{
    zoom.X = value;
//    Refresh();
}

//set current zoom in x coordinate
void grCanvas::SetYZoom(float value)
{
    zoom.Y = value;
//    Refresh();
}

//set the background of your screen
void grCanvas::SetBackground( const RGBt &color)
{
    // Set normal background
    bg_color = RGBt(color);
}

//set the foreground of your screen
void grCanvas::SetForeground( const RGBt &color)
{
    // Set normal background
    fg_color = RGBt(color);
    glColor4f((float) color.R, (float) color.G, (float) color.B, (float) color.A);
}

void grCanvas::SetLineWidth(float width)
{
	glLineWidth(width);
}

//clear the canvas using background color
void grCanvas::Clear()
{
    SetCurrent();
    glClearColor((float) bg_color.R, (float) bg_color.G, (float) bg_color.B,0);
    glClear(GL_COLOR_BUFFER_BIT);
}

void grCanvas::SetHandCursor()
{
    SetCursor(wxCursor(wxCURSOR_HAND));
}

/*void grCanvas::SetBullseyeCursor()
{
    SetCursor(wxCursor(wxCURSOR_MAGNIFIER));
}*/

void grCanvas::SetDefaultCursor()
{
    SetCursor(wxCursor(wxCURSOR_ARROW));
}

void grCanvas::Refresh()
{
/*    if (viewManager == NULL)
        return;

    viewManager->RefreshCanvas(this);
*/	SetCurrent();
    Clear();
    SetupTransformMatrix();

    if (viewManager == NULL)
        return;

    viewManager->RefreshCanvas(this);	    
    glFlush();
 	SwapBuffers();
   // }  
}

//write the image in the canvas into a file whose name is stored in stream
void grCanvas::WriteImage(std::string fileName)
{
    SetCurrent();

    int width, height;
    GetSize(&width,&height);

	unsigned char* rgb = new unsigned char[4*width*height];;
    glReadPixels(0, 0, width, height, GL_RGBA, GL_UNSIGNED_BYTE, rgb);

	unsigned char* data = new unsigned char[3*width*height];
	
	int index1, index2;
    int i, j;
	for (j = height-1; j >= 0; j--) 
	{
		index1 = j*width*4;
		index2 = (height-j-1)*width*3;
		
		for (i = 0; i < width ; i++)
		{
			data[index2] = rgb[index1]; 
			data[index2+1] = rgb[index1+1]; 

			data[index2+2] = rgb[index1+2]; 
			
			index1 += 4;
			index2 += 3;
		}
	}

	wxImage* image = new wxImage(width, height, data);
	
	wxFileOutputStream* stream = new wxFileOutputStream(fileName.c_str());

	image->SaveFile(*stream, wxBITMAP_TYPE_BMP);

    delete stream;
	delete image;
}

//In hortizonal, baseX is the leftmost point, baseY is the lowest point. You can rotate the text by setting a non-zero angle. You can change the size of the fonts by changing scale
void grCanvas::DrawString(std::string str, float baseX, float baseY, float angle, float scale)
{
    Vec2 base;
    base = MapDataToScreen(Vec2(baseX, baseY));

    GLuint texture;
    int width, height, width1, height1;
    GetTextTexture(str, texture, width, height, width1, height1);

    DrawTextTexture(texture, width, height, width1, height1, base.X, base.Y, angle, scale);
}

/*
//In hortizonal, baseX is the leftmost point, baseY is the lowest point. You can rotate the text by setting a non-zero angle. You can change the size of the fonts by changing scale
void grCanvas::DrawString(std::string str, float baseX, float baseY, float angle, float scale)
{
    Vec2 base;
    base = MapDataToScreen(Vec2(baseX, baseY));

    GLuint texture;
    int width, height, width1, height1;
    GetTextTexture(str, texture, width, height, width1, height1);

    Vec2 vec = MapScreeVectorToDataVector(Vec2(width1, height1));

    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();

    glTranslatef(baseX, baseY, 0);
    glRotated(angle, 0, 0, 1);
    glScalef(scale, scale, scale);

//    DrawTextTexture(texture, width, height, width1, height1, base.X, base.Y, angle, scale);

    glClear(GL_DEPTH_BUFFER_BIT);
    glEnable( GL_TEXTURE_2D );
    glBindTexture( GL_TEXTURE_2D, texture);

    float x, y;
    x = (float)width1/width;
    y = (float)height1/height;
    if (origin_at_lower_left)
    {
        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex2d(0, 0);
	    glTexCoord2d(x,y); glVertex2d(vec.X, 0);
	    glTexCoord2d(x,0.0); glVertex2d(vec.X, vec.Y);
	    glTexCoord2d(0.0,0.0); glVertex2d(0, vec.Y);
	    glEnd();
    }
    else
    {
        glBegin(GL_QUADS);
	    glTexCoord2d(0.0,y); glVertex2d(0, 0);
	    glTexCoord2d(x,y); glVertex2d(vec.X, 0);
	    glTexCoord2d(x,0.); glVertex2d(vec.X, vec.Y);
	    glTexCoord2d(0.0,0.); glVertex2d(0, vec.Y);
	    glEnd();
    }
    glDisable( GL_TEXTURE_2D );

    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
}
*/

//In hortizonal, baseX is the leftmost point, baseY is the lowest point. 
//z is the z position of the string in 3D space
//You can rotate the text by setting a non-zero angle. You can change the size of the fonts by changing scale
void grCanvas::DrawString(std::string str, float baseX, float baseY, float z, float angle, float scale)
{
    GLuint texture;
    int width, height, width1, height1;
    GetTextTexture(str, texture, width, height, width1, height1);

    DrawTextTexture(texture, width, height, width1, height1, baseX, baseY, z, angle, scale);
}

/*
//In hortizonal, baseX is the leftmost point, baseY is the lowest point. 
//z is the z position of the string in 3D space
//You can rotate the text by setting a non-zero angle. You can change the size of the fonts by changing scale
void grCanvas::DrawString(std::string str, float baseX, float baseY, float z, float angle, float scale)
{
    Vec3 base;
    base = MapDataToScreen(Vec3(baseX, baseY, z));

    GLuint texture;
    int width, height, width1, height1;
    GetTextTexture(str, texture, width, height, width1, height1);

    DrawTextTexture(texture, width, height, width1, height1, base.X, base.Y, z, angle, scale);
}*/

float grCanvas::GetFontHeight()
{
    GLuint texture;
    int width, height, width1, height1;
    GetTextTexture("Lg", texture, width, height, width1, height1);
    Vec2 p = MapScreeVectorToDataVector(Vec2(0, height1));
    return p.Y;
}

void grCanvas::DrawLine( double x1, double y1, double x2, double y2)
{
    BeginDraw();
    glBegin(GL_LINES);
    glVertex2f((float) x1, (float) y1);
    glVertex2f((float) x2, (float) y2);
    glEnd();
    EndDraw();
}

//the circle is acturally a disk, inner radius: radius-offset, out radius: radius+offset
void grCanvas::DrawCircle( double centerx, double centery, double radius, double offset)
{
    BeginDraw();
	glPushMatrix();
		
	glTranslatef((float) centerx, (float) centery, 0.0f);
	GLUquadricObj* quadObj;
	quadObj=gluNewQuadric();
	gluQuadricDrawStyle(quadObj,GLU_FILL);
	gluQuadricNormals(quadObj,GLU_FLAT);
	int slice = (int) (SLICES * radius * 360 * MAX2(zoom.X, zoom.Y));
	if ( slice < 6 )
		slice = 6;
	gluDisk(quadObj,radius-offset, radius+offset, slice, 1);

	glPopMatrix();
    EndDraw();
}

void grCanvas::DrawArc( double centerx, double centery, double radius, double start_angle, double sweep_angle)
{
    BeginDraw();
	glPushMatrix();
		
	glTranslatef((float) centerx, (float) centery, 0.0f);
	GLUquadricObj* quadObj;
	quadObj=gluNewQuadric();
	gluQuadricDrawStyle(quadObj,GLU_LINE);
	gluQuadricNormals(quadObj,GLU_FLAT);
	int slice = (int) (SLICES * sweep_angle * radius * MAX2(zoom.X, zoom.Y));
	if ( slice < 2 )
		slice = 2;
	gluPartialDisk(quadObj,radius, radius, slice, 1, start_angle, sweep_angle);

	glPopMatrix();
    EndDraw();
}

void grCanvas::DrawPartialDisk( double centerx, double centery, double inner_radius, 
							    double outer_radius, double start_angle, double sweep_angle)
{
    BeginDraw();
	glPushMatrix();
		
	glTranslatef((float) centerx, (float) centery, 0.0f);
	GLUquadricObj* quadObj;
	quadObj=gluNewQuadric();
	gluQuadricDrawStyle(quadObj,GLU_FILL);
	gluQuadricNormals(quadObj,GLU_SMOOTH);
	int slice = (int)(sweep_angle * SLICES2 * outer_radius * MAX2(zoom.X, zoom.Y));
	if ( slice < 2 )
		slice = 2;
	gluPartialDisk(quadObj,inner_radius, outer_radius, slice, 1, start_angle, sweep_angle);

	glPopMatrix();
    EndDraw();
}


void grCanvas::DrawLines( const ARR<Vec2> &pts )
{
    if (pts.Len()<=0) return;
    BeginDraw();
    glBegin(GL_LINE_STRIP);
    glVertex2f((float) pts[0].X, (float) pts[0].Y);
    for (int i=1; i<pts.Len(); i++) {
        glVertex2f((float) pts[i].X, (float) pts[i].Y);
    }
    glEnd();
    EndDraw();
}

void grCanvas::DrawRectangle(double x, double y, double width, double ht )
{
    BeginDraw();
    glBegin(GL_LINE_LOOP);
    glVertex2f((float)  x, (float) y);
    glVertex2f((float) (x+width), (float) y);
    glVertex2f((float) (x+width), (float) (y+ht));
    glVertex2f((float)  x, (float) (y+ht));
    glEnd();
    EndDraw();
}


void grCanvas::DrawRectangle2(double x1, double x2, double y1, double y2)
{
    BeginDraw();
    glBegin(GL_LINE_LOOP);
    glVertex2f((float) x1, (float) y1);
    glVertex2f((float) x2, (float) y1);
    glVertex2f((float) x2, (float) y2);
    glVertex2f((float) x1, (float) y2);
    glEnd();
    EndDraw();
}


void grCanvas::FillRectangle(double x, double y, double width, double ht)
{
    BeginDraw();
    glBegin(GL_QUADS);
    glVertex2f((float)  x, (float) y);
    glVertex2f((float) (x+width), (float) y);
    glVertex2f((float) (x+width), (float) (y+ht));
    glVertex2f((float)  x, (float) (y+ht));
    glEnd();
    EndDraw();
}

void grCanvas::DrawPoint( double x, double y, double size)
{
    double half = 0.5*size;

    BeginDraw();
    glPointSize((float) size);
    glBegin(GL_POINTS);
    glVertex2f((float) (x+half), (float) (y+half));
    glEnd();
    EndDraw();
}

void grCanvas::DrawPoints( const ARR<Vec2> &pts, double size)
{
    BeginDraw();
    glPointSize((float) size);
    glBegin(GL_POINTS);
    for (int i=0;i<pts.Len();i++) {
        glVertex2f((float) pts[i].X, (float) pts[i].Y);
    }
    glEnd();
    EndDraw();
}

void grCanvas::DrawPolygon( const ARR<Vec2> &pts)
{
    BeginDraw();
    glBegin(GL_LINE_LOOP);
    for (int i=0;i<pts.Len();i++) {
        glVertex2f((float) pts[i].X, (float) pts[i].Y);
    }
    glEnd();
    EndDraw();
}

void grCanvas::FillPolygon( const ARR<Vec2> &pts)
{
    BeginDraw();
    glBegin(GL_POLYGON);
    for (int i=0;i<pts.Len();i++) {
        glVertex2f((float) pts[i].X, (float) pts[i].Y);
    }
    glEnd();
    EndDraw();
}

//the length of pts must be 4.
void grCanvas::DrawTransparentBand( const ARR<Vec2> &pts,
                        double opacity_bottom, double opacity_top,
                        RGBt color_bottom, RGBt color_top)
{
    assert(pts.Len()==4);
    assert(xxor == false);

    translucent = true;

    BeginDraw();
    glBegin(GL_QUAD_STRIP);
    glColor4f((float) color_bottom.R, (float) color_bottom.G, (float) color_bottom.B, (float) (alpha*opacity_bottom) );
    glVertex2f((float) pts[1].X, (float) pts[1].Y);
    glVertex2f((float) pts[0].X, (float) pts[0].Y);

    glColor4f((float) color_top.R, (float) color_top.G, (float) color_top.B, (float) (alpha*opacity_top) );
    glVertex2f((float) pts[2].X, (float) pts[2].Y);
    glVertex2f((float) pts[3].X, (float) pts[3].Y);
    glEnd();

    EndDraw();

    translucent = false;
}

void grCanvas::CreateTexture(int groupID, int glyphID, int width, int height, float* data, GLuint &texture)
{
    CreateTexture(width, height, data, texture);
    textureHashs[groupID][glyphID] = grTexture(texture, width, height, width, height);
}

bool grCanvas::GetTexture(int groupID, int glyphID, GLuint &texture, int &width, int &height)
{
    ITHash::iterator it;
    it = textureHashs[groupID].find(glyphID);
    if (it == textureHashs[groupID].end())
    {
        return false;
    }
    else
    {
        grTexture tex = it->second;
        texture = tex.id;
        width = tex.width;
        height = tex.height;
        return true;
    }
}

void grCanvas::DeleteTexture(int groupID, int ID)
{
    ITHash::iterator it = textureHashs[groupID].find(ID);
    if (it != textureHashs[groupID].end())
    {
        glDeleteTextures(1, &(it->second.id));
        textureHashs[groupID].erase(ID);
    }
}

void grCanvas::DeleteTextures(int groupID)
{
    ITHash::iterator it;
    it = textureHashs[groupID].begin();
    while (it != textureHashs[groupID].end())
    {
        glDeleteTextures(1, &(it->second.id));
        it++;
    }

    textureHashs[groupID].clear();
}



//i can be 0, 1, or 2, since there are 3 message place in the frame status bar
void grCanvas::WriteMessage(std::string message, int i)
{
    frame->SetStatusText(message.c_str(), i);
}


void grCanvas::KeyPress(int key)
{        
    switch (key) {
        case WXK_SHIFT:
            shiftkey = true;
            break;
        case WXK_CONTROL:
            controlkey = true;
        case WXK_CAPITAL:
            capitalkey = !capitalkey;
            break;
    }
}

void grCanvas::KeyRelease(int key)
{        
    switch (key) {
        case WXK_SHIFT:
            shiftkey = false;
            break;
        case WXK_CONTROL:
            controlkey = false;
            break;
    }
}
